!******************************************************************************
!  Module ATM_MOD contains variables and routines for the atmospheric component
!  of a box model developed by D. Abbot and E. Tziperman.  
!  Code written by D. Abbot
!
!  Notes:
!  (01) see file /home/abbot/tex/PROJECTS/Feedbacks/MODEL/model.pdf for details
!  (02) j is the latitude variable, it increases northward
!  (03) k is the height variable, it increases DOWNWARD
!  (04) all units in MKS!
!  (05) KMAX should not be changed, too many things hardwired. (e.g. radiation scheme, clds, convection)
!       JMAX could be changed with minor effort
!  (06) Q is the specific humidity - the mass water vapor per unit mass air 
!       (including vapor and dry air)
!  (07) Many things with R in them, here is a short list of how they are used
!       in this code:
!
!       R      - Radius of the Earth
!       R_FRIC - non-scale selective simple frictional term
!       RD     - Gas constant for dry air
!       RV     - Gas constant for water vapor
!       R_TOT  - Gas constant for air, including dry and vapor 
!                (depends on amount water vapor, so is a (JMAX,KMAX) size array)
!       R_MIX  - Mixing ratio of water vapor
!
!  (08) ALPHA is the specific volume.  ALPHA_something (like ALPHA_ATM) are albedos
!  (09) Open your emacs window very wide (~115-120 cols) to view this code best
!  (10) modified code so that eqns are solved in spherical coordinates (6/17/05)
!
!  (dsa, 3/21/05)
!******************************************************************************
module atm_mod

! MODULE FUNCTIONS AND SUBROUTINES:
! (01) subroutine open_files
! (02) subroutine get_parameters
! (03) subroutine init_atm
! (04) subroutine cleanup_atm
! (05) subroutine write_restart
! (06) subroutine initialize_vars
! (07) subroutine atm_step
! (08) subroutine update_r_tot
! (09) subroutine update_cp_tot
! (10) subroutine update_t
! (11) subroutine update_theta
! (12) subroutine update_alpha
! (13) subroutine run_model
! (14) subroutine print_diags
! (15) subroutine update_atm_emissivity
! (16) function   rad_tend_dse
! (17) function   baroclinic_tend_dse
! (18) function   baroclinic_tend_Q
! (19) function   derivative
! (20) subroutine check_advect
! (21) subroutine update_air_surface_flux
! (22) subroutine investigate_steady_state
! (23) function   calc_e_star
! (24) function   calc_q_from_e
! (25) function   calc_e_from_q
! (26) function   evaporation_tend_q
! (27) function   sens_heat_tend_dse
! (28) subroutine debug_routine
! (29) subroutine get_precip
! (30) function   precipitation_tend_dse
! (31) function   precipitation_tend_q
! (32) subroutine update_all_vars
! (33) subroutine update_moisture
! (34) subroutine update_theta_e
! (35) subroutine update_lv
! (36) function   convection_tend_dse
! (37) function   convection_tend_q
! (38) function   emissivity_h2o
! (39) function   emissivity_co2
! (40) function   transmissivity_h2o_15mu
! (41) function   advect_y
! (42) function   advect_p
! (43) function   advection_tend_dse
! (44) function   advection_tend_q
! (45) function   re_evap_tend_dse
! (46) function   re_evap_tend_q
! (47) subroutine update_re_evap
! (48) function   calc_tend_ts
! (49) function   calc_tend_sst
! (50) function   btrop_compute
! (51) function   btrop_compute1
! (52) function   bclin_compute
! (53) function   bclin_compute1
! (54) function   tendency_btrop_u
! (55) function   tendency_bclin_u
! (56) function   tendency_bclin_v
! (57) function   center_interp
! (58) function   edge_int
! (59) function   edge_interp1
! (60) subroutine update_phi
! (61) subroutine update_velocities
! (62) subroutine investigate_steady_v
! (63) subroutine investigate_steady_temps
! (64) subroutine investigate_steady_moist
! (65) function   emissivity_o3
! (66) function   path_length_o3

implicit none

! MODULE VARIABLES

!JMAX must be set at compile time!!!!!  
!input arrays of size JMAXmust be changed in atmosphere.dat 
!so that they are in accordence with
!JMAX as set here!!!!  
integer, parameter :: JMAX=3

integer, parameter :: KMAX=2,ROBERT=3,ATM_FID=15,IN_FID=16,            &
                      RESTART_IN_FID=17,DIAGS_FID=18,DIAGS_FID2=19,    &
                      DEBUG_FID=20,DEBUG_FID2=21,SS_BAL_FID=22,        &
                      SS_VEL_FID=23,SS_TEMP_FID=24,SS_MOIST_FID=25,    &
                      RADCHEAT_IN_FID=26,PARAM_VARY_FID=27,CLD_FID=28, &
                      JMAX_FID=29,JMAX_LBL_FID=30,JMAXp1_FID=31,       &
                      JMAXp1_LBL_FID=32,JMAXm1_FID=33,                 &
                      JMAXm1_LBL_FID=34,SS_BAL_PLOT_FID=35,            &
                      SS_VEL_PLOT_FID=36,RESTART_OUT_FID=37,           &
                      RADCHEAT_OUT_FID=38,PARAM_VARY_FID2=39

!arrays:
real*8, allocatable :: LAT(:)              !latitude at centers
real*8, allocatable :: LAT_EDGE(:)         !latitude at centersedges
real*8, allocatable :: Y(:)                !y at centers
real*8, allocatable :: Y_EDGE(:)           !y at edges
real*8, allocatable :: FRAC_LAND(:)        !land fraction
real*8, allocatable :: V_EDGE(:,:)         !meridional velocity, defined at edges in y, centers in p
real*8, allocatable :: OMEGA(:)            !vertical velocity, defined at centers in y, edges in p
real*8, allocatable :: ALPHA(:,:)          !specific volume
real*8, allocatable :: THETA(:,:)          !potential temperature
real*8, allocatable :: DSE(:,:,:)          !dry static energy (J/kg)
real*8, allocatable :: MSE(:,:)            !moist static energy (J/kg)
real*8, allocatable :: MSE_STAR(:,:)       !saturation moist static energy (J/kg)
real*8, allocatable :: MSE_STAR_TILDE(:)   !special saturation MSE for convection w/ descent (J/kg)
real*8, allocatable :: Q(:,:,:)            !specific humidity (mass H2O vapor per mass air, including vapor)
real*8, allocatable :: QSTART(:,:)         !specific humidity at start (mass H2O vapor per mass air, including vapor)
real*8, allocatable :: F(:)                !coliolis parameter at box centers
real*8, allocatable :: F_EDGE(:)           !coriolis parameter on box edges
real*8, allocatable :: R_TOT(:,:)          !gas constant including H2O vapor effects
real*8, allocatable :: CP_TOT(:,:)         !Cp including H2O vapor effects
real*8, allocatable :: T(:,:)              !temperature (K)
real*8, allocatable :: T_H_CLD(:)          !temperature of high clds (K)
real*8, allocatable :: VOL_M3(:,:)         !approximate box volume in m^3
real*8, allocatable :: EMISS(:,:)          !atm emissivity 
real*8, allocatable :: EMISS_CLEAR(:,:)    !clear sky emissivity 
real*8, allocatable :: EMISS_O3(:,:)       !emissivity due to ozone
real*8, allocatable :: EMISS_CLEAR_REF1(:,:) !reference clear-sky emissivity 
real*8, allocatable :: EMISS_CLEAR_REF2(:,:) !reference clear-sky emissivity 
real*8, allocatable :: EMISS_H(:)          !total high cloud emissivity
real*8, allocatable :: ALPHA_LAND(:)       !land albedo
real*8, allocatable :: ALPHA_OCN(:)        !ocn albedo
real*8, allocatable :: FRAC_CLD(:,:)       !cld fraction (at levels 1 and 2 (mid and low))
real*8, allocatable :: FRAC_H_CLD(:)       !high cld fraction
real*8, allocatable :: Z(:)                !approximate height of pressure levels 
real*8, allocatable :: Z_EDGE(:)           !approximate height of pressure edges
real*8, allocatable :: SST(:,:)            !sea surface temperature (in Kelvin)
real*8, allocatable :: TS(:,:)             !land temperature
real*8, allocatable :: OCN_SH_FLUX(:)      !ocean sensible heat flux (J m^-2 s^-1)
real*8, allocatable :: OCN_EVAP_FLUX(:)    !ocean evaporation flux ((kg H20 vap) m^-2 s^-1)
real*8, allocatable :: LAND_SH_FLUX(:)     !land sensible heat flux (J m^-2 s^-1)
real*8, allocatable :: LAND_EVAP_FLUX(:)   !land evaportaion (specific humidity flux) ((kg H20 vap) m^-2 s^-1)
real*8, allocatable :: RUNOFF(:)           !land runoff (kg H20 / s)
real*8, allocatable :: PRECIP(:,:)         !precipitation generated in each box (Kg H2O s^-1)
real*8, allocatable :: FW_FLUX(:)          !fresh-water flux into ocean from precipitation (Kg H2O/s)
real*8, allocatable :: RH(:,:)             !relative humidity
real*8, allocatable :: SW_FLUX(:)          !solar short wave radiation flux (J m^-2 s^-1)
real*8, allocatable :: LAT_RADS(:)         !latitude at centers in radians
real*8, allocatable :: LAT_EDGE_RADS(:)    !latitude at centers in radians
real*8, allocatable :: SURF_AREA(:)        !surface area of boxes (m^2)
real*8, allocatable :: R_MIX(:,:)          !mixing ratio (kg/kg) (mass water vapor per mass DRY air)
real*8, allocatable :: THETA_E(:,:)        !equivalent potential temperature (K)
real*8, allocatable :: THETA_E_STAR(:,:)   !saturation equivalent potential temperature (K)
real*8, allocatable :: LV(:,:)             !temperature dependent latent heat of vaporization (J kg^-1)
real*8, allocatable :: E(:,:)              !vapor pressure (Pa)
real*8, allocatable :: E_STAR(:,:)         !saturation vapor pressure (Pa)
real*8, allocatable :: Q_STAR(:,:)         !saturation specific humidity (kg/kg)
real*8, allocatable :: R_MIX_STAR(:,:)     !saturation mixing ratio (kg/kg)
real*8, allocatable :: EVAP_FRAC(:)        !evaporation fraction, should depend on soil moisture and vegetation type
real*8, allocatable :: EDDY_HEAT(:,:)      !eddy heat trans. (W)
real*8, allocatable :: EDDY_MOIST(:,:)     !eddy moisture trans. (kg H2O s^-1)
real*8, allocatable :: ADV_MOIST(:,:)      !advective moisture trans. (kg H2O s^-1)
real*8, allocatable :: EDDY_LATENT(:,:)    !eddy latent energy trans. (W)
real*8, allocatable :: ADVECT_HEAT_Y(:,:)  !merid. advect. heat trans. (W)
real*8, allocatable :: ADVECT_HEAT_P(:,:)  !vert.  advect. heat trans. (W)
real*8, allocatable :: ADVECT_LATENT_Y(:,:)!merid. advect. moisture trans. (W)
real*8, allocatable :: ADVECT_LATENT_P(:,:)!vert.  advect. moisture trans. (W)
real*8, allocatable :: OLR(:)              !outgoing longwave radiation (J m^-2 s^-1)
real*8, allocatable :: F_LW_DOWN(:)        !downward lw rad. from atm at surface (J m^-2 s^-1)
real*8, allocatable :: F_LW_SURF_UP(:)     !upward lw rad. from surface (J m^-2 s^-1)
real*8, allocatable :: F_SW_LAND_DOWN(:)   !downward SW flux absorbed by land (J m^-2 s^-1)
real*8, allocatable :: F_SW_OCN_DOWN(:)    !downward SW flux absorbed by ocn (J m^-2 s^-1)
real*8, allocatable :: ABSORB_SW(:)        !absorbed shortwave radiation (J m^-2 s^-1)
real*8, allocatable :: FLUX_IN_OCN(:)      !flux of energy into the ocean (J m^-2 s^-1)
real*8, allocatable :: EPS_RE_EVAP(:)      !re-evap. fraction in lower layers (assume 2 layers)
real*8, allocatable :: RE_EVAP(:)          !rate of re-evap. of precip. from upper layer in lower layer (kg H2O s^-1)
real*8, allocatable :: HEATING(:)          !heat flux into atm at each latitude (J m^-2 s^-1)
real*8, allocatable :: PRECIP_G(:)         !precipitation that hits the ground at each lat. (kg H2O s^-1)
real*8, allocatable :: DX(:)               !zonal distance along inter-box boundaries (m)      
real*8, allocatable :: BTROP_U(:,:)        !barotropic component of the zonal velocity (m s^-1)
real*8, allocatable :: BCLIN_U(:,:,:)      !baroclinic component of the zonal velocity (m s^-1)     
real*8, allocatable :: U(:,:)              !zonal velocity (m s^-1)
real*8, allocatable :: BCLIN_V(:,:,:)      !baroclinic component of the meridional velocity (m s^-1)       
real*8, allocatable :: BTROP_PHI(:)        !barotropic component of the geopotential (m^2 s^-2)
real*8, allocatable :: BCLIN_PHI(:,:)      !baroclinic component of the geopotential (m^2 s^-2)   
real*8, allocatable :: PHI(:,:)            !total geopotential (m^2 s^-2)
real*8, allocatable :: MASS(:,:)           !mass air in each box (kg)
real*8, allocatable :: ANG_MOM(:,:)        !angular momentum per unit mass (m^2 s^-1) 
real*8, allocatable :: K_CON(:)            !convective constant (s^-1)
real*8, allocatable :: OHT(:)              !assumed ocean heat transport (W)
real*8, allocatable :: OHT_S(:)            !ocean heat transport w/o gaussian (W)


!general parameters and constants
!included in compilation b/c these shouldn't be changed
real*8, parameter :: R         = 6.38d6,     & !m 
                     CPD       = 1005.7d0,   & !J Kg^-1 K^-1
                     CVD       = 719.0d0,    & !J Kg^-1 K^-1
                     CPV       = 1870.0d0,   & !J Kg^-1 K^-1
                     CVV       = 1410.0d0,   & !J Kg^-1 K^-1
                     CL        = 4190.0d0,   & !J Kg^-1 K^-1
                     RHO_H2O   = 1000.0d0,   & !kg m^-3
                     OMEGA_ROT = 7.2722d-5,  & !s^-1
                     RD        = 287.0d0,    & !J Kg^-1 K^-1
                     RV        = 461.5d0,    & !J Kg^-1 K^-1
                     SIGMA     = 5.670d-8,   & !J K^-4 m^-2 s^-1
                     G         = 9.81d0,     & !m s^-2
                     LV0       = 2.501d6,    & !J kg^-1  
                     YEAR      = 3.15576d7,  & !s year^-1
                     DUMMY     = -999.d0    

!for box definition namelist 
logical :: AUTO_LAT,HEMI_S,CONST_FRAC_LAND 
real*8  :: P(KMAX),P_EDGE(KMAX+1),LAT_EDGE0(JMAX+1),DLON,MIN_LAT,MAX_LAT,   &
           FRAC_LAND0
namelist /nl_boxdef/ &
     P,P_EDGE,AUTO_LAT,MIN_LAT,MAX_LAT,LAT_EDGE0,DLON,CONST_FRAC_LAND,      &
     FRAC_LAND0,HEMI_S

!for options namelist
logical :: WRITE_RESTART_DATA,READ_RESTART_DATA,TIME_WRITE
real*8  :: ROBERT_FILTER,DT,TSTART,TPRINT,TEND,T_START_VARY
integer :: F_BTROP_DT,F_BCLIN_DT
namelist /nl_options/ &
     WRITE_RESTART_DATA,READ_RESTART_DATA,TIME_WRITE,ROBERT_FILTER,DT,      &
     F_BTROP_DT,F_BCLIN_DT,TSTART,TPRINT,TEND,T_START_VARY

!namelist for cloud parameters
real*8  :: EMISS_CLD(KMAX),EMISS_H_CLD,ALPHA_CLD(KMAX),ALPHA_H_CLD,         &
           C_C_MAX,M_MIN,C_S_MAX(KMAX),RH0(KMAX)
logical :: CLD_VARY_BOOL,DIAG_CLDS,CLD_OPT
namelist /nl_clouds/ &
     CLD_VARY_BOOL,EMISS_CLD,EMISS_H_CLD,ALPHA_CLD,ALPHA_H_CLD,DIAG_CLDS,   &
     C_C_MAX,M_MIN,C_S_MAX,RH0,CLD_OPT

!namelist for radiation parameters
real*8  :: ALPHA_ATM,ALPHA_LAND_ICE,ALPHA_LAND_NOICE,ALPHA_OCN_ICE,         &
           ALPHA_OCN_NOICE,LAT_ICE_OCN,LAT_ICE_LAND,RAD_CHEAT_FAC1,         &
           RAD_CHEAT_FAC2,EM_OFFSET(KMAX),CO2_REF,H2O_REF(KMAX),A_H2O,      &
           B_H2O,C_H2O,A_CO2,B_CO2,CO20,EMISS_LAND,EMISS_OCN,EPS_RE_EVAP0
logical :: Q_EM,RAD_CHEAT1,RAD_CHEAT2,RAD_CHEAT3,RAD_CHEAT_WRITE,           &
           CONST_OFFSET
namelist /nl_radiation/ &
     ALPHA_ATM,ALPHA_LAND_NOICE,ALPHA_LAND_ICE,ALPHA_OCN_NOICE,             &
     ALPHA_OCN_ICE,LAT_ICE_OCN,LAT_ICE_LAND,EMISS_LAND,EMISS_OCN,CO20,      &
     CO2_REF,H2O_REF,Q_EM,RAD_CHEAT1,RAD_CHEAT2,RAD_CHEAT3,A_H2O,B_H2O,     &
     C_H2O,A_CO2,B_CO2,RAD_CHEAT_WRITE,RAD_CHEAT_FAC1,RAD_CHEAT_FAC2,       &
     CONST_OFFSET,EM_OFFSET

!namelist for tunable parameters 
real*8  :: R_FRIC(KMAX),KY_BCLIN_TH,KY_BCLIN_Q,KY_BCLIN_MOM,N_BCLIN,        &
           BCLIN_FRIC(KMAX),C_SH_OCN,C_SH_LAND,C_Q_OCN,C_Q_LAND,            &
           CRIT_RH(KMAX),V_SURF,PRECIP_T_SCALE,PRECIP_SCALE,Q0,Q2,          &
           K_CON_MIN,K_CON_MAX,EPS_CON,C_LAND,D_LAND,EFRAC,CONV_SQUASH,     &
           VEL_VISC,P_TILDE,BTROP_PHI0
namelist /nl_tune_params/ &
     R_FRIC,VEL_VISC,BTROP_PHI0,KY_BCLIN_TH,KY_BCLIN_Q,KY_BCLIN_MOM,        &
     N_BCLIN,BCLIN_FRIC,C_SH_OCN,C_SH_LAND,C_Q_OCN,C_Q_LAND,CRIT_RH,V_SURF, &
     PRECIP_T_SCALE,PRECIP_SCALE,Q0,Q2,K_CON_MIN,K_CON_MAX,EPS_CON,         &
     CONV_SQUASH,P_TILDE,EPS_RE_EVAP0,C_LAND,D_LAND,EFRAC

!namelist for ocean parameters 
real*8  :: OHT0,OHT_DECAY,D_OCN,LAT_OHT_LIN,LAT_OHT_ZERO,LAT_GAUSS,         &
           SIG_GAUSS,OHT0_GAUSS
logical :: INT_OCN
namelist /nl_ocn_params/ &
     D_OCN,INT_OCN,OHT0,OHT_DECAY,LAT_OHT_LIN,LAT_OHT_ZERO,LAT_GAUSS,       &
     SIG_GAUSS,OHT0_GAUSS

!namelist for hysteresis parameters 
real*8  :: OHT_VARY_MAX,CO2_1,CO2_2 
logical :: OHT_VARY_BOOL,CO2_VARY_BOOL
namelist /nl_hyst/ &
     OHT_VARY_BOOL,OHT_VARY_MAX,CO2_VARY_BOOL,CO2_1,CO2_2

real*8 :: PI,EPSILON,TIME,L,DLON_RADS,DT_BCLIN,DT_BTROP,CO2,OMEGA_NO_CONV(JMAX),OHT_B,OHT_GMAX

!MODULE ROUTINES
contains

!******************************************************************************
! (01) subroutine open_files
!******************************************************************************
subroutine open_files
  integer    :: IOS
  

!-----------!
!INPUT FILES!
!-----------!
  open(UNIT=ATM_FID,         FILE='INPUT/atmosphere.data',    FORM='FORMATTED', &
       STATUS='OLD',         ACTION='READ',                   RECL=100,         &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: INPUT/atmosphere.data'
  

  open(UNIT=RESTART_IN_FID,  FILE='INPUT/restart_in',         FORM='FORMATTED', &
       STATUS='OLD',         ACTION='READ',                   RECL=400,         &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: INPUT/restart_in'


  open(UNIT=RADCHEAT_IN_FID, FILE='INPUT/radcheat_in',        FORM='FORMATTED', &
       STATUS='OLD',         ACTION='READ',                   RECL=400,         &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: INPUT/radchat_in'




!------------!
!OUTPUT FILES!
!------------!

  open(UNIT=IN_FID,          FILE='OUTPUT/input_bounce.data', FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=400,         &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/input_bounce.data'


  open(UNIT=RESTART_OUT_FID, FILE='OUTPUT/restart_out',       FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=400,         &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/restart_out'


  open(UNIT=DIAGS_FID,       FILE='OUTPUT/diags.out',         FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=8000,        &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/diags.out'


  open(UNIT=DIAGS_FID2,      FILE='OUTPUT/diags2.out',        FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=2000,        &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/diags2.out'


  open(UNIT=DEBUG_FID,       FILE='OUTPUT/debug',             FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=500,         &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/debug'


  open(UNIT=DEBUG_FID2,      FILE='OUTPUT/debug2',            FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=500,         &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/debug'


  open(UNIT=SS_BAL_FID,      FILE='OUTPUT/ss_bal',            FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=1100,        &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/ss_bal'


  open(UNIT=SS_VEL_FID,      FILE='OUTPUT/ss_vel',            FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=1000,        &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/ss_vel'


  open(UNIT=SS_TEMP_FID,     FILE='OUTPUT/ss_temp',           FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=1000,        &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/ss_temp'


  open(UNIT=SS_MOIST_FID,    FILE='OUTPUT/ss_moist',          FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=1000,        &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/ss_moist'


  open(UNIT=RADCHEAT_OUT_FID,FILE='OUTPUT/radcheat_out',      FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=400,         &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/radcheat_out'


  open(UNIT=PARAM_VARY_FID,  FILE='OUTPUT/param_vary',        FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=3250,        &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/param_vary'


  open(UNIT=PARAM_VARY_FID2, FILE='OUTPUT/param_vary2',       FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=52,          &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/param_vary2'

! open(UNIT=CLD_FID,         FILE='OUTPUT/cld',               FORM='FORMATTED', &
!      STATUS='REPLACE',     ACTION='WRITE',                  RECL=100,         &
!      IOSTAT=IOS)
! if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/cld'


  open(UNIT=JMAX_FID,        FILE='OUTPUT/JMAX_out',          FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=2000,        &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/JMAX_out'


  open(UNIT=JMAX_LBL_FID,    FILE='OUTPUT/JMAX_LBL_out',      FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=100,         &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/JMAX_LBL_out'


  open(UNIT=JMAXm1_FID,      FILE='OUTPUT/JMAXm1_out',        FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=2000,        &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/JMAXm1_out'


  open(UNIT=JMAXm1_LBL_FID,  FILE='OUTPUT/JMAXm1_LBL_out',    FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=100,         &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/JMAXm1_LBL_out'


  open(UNIT=JMAXp1_FID,      FILE='OUTPUT/JMAXp1_out',        FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=2000,        &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/JMAXp1_out'


  open(UNIT=JMAXp1_LBL_FID,  FILE='OUTPUT/JMAXp1_LBL_out',    FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=100,         &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/JMAXp1_LBL_out'


  open(UNIT=SS_BAL_PLOT_FID, FILE='OUTPUT/ss_bal_plot',       FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=1100,        &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/ss_bal_plot'


  open(UNIT=SS_VEL_PLOT_FID, FILE='OUTPUT/ss_vel_plot',       FORM='FORMATTED', &
       STATUS='REPLACE',     ACTION='WRITE',                  RECL=1100,        &
       IOSTAT=IOS)
  if( IOS /= 0 ) write(6,*) 'Error opening file: OUTPUT/ss_vel_plot'


end subroutine open_files



!******************************************************************************
! (02) subroutine get_parameters
!******************************************************************************

subroutine get_parameters

  integer                   :: J,K
  real*8                    :: U1,U2,AVG_SURF_ALB,ALB0_AVG,DLAT,OHT_LIN
  real*8, dimension(JMAX+1) :: TMP_ARR,UCO2,UH2O,OHT_GAUSS
  real*8, dimension(JMAX)   :: UO3_1,UO3_2,ALB0
  real*8, dimension(18)     :: FRAC_LAND_5
  real*8, parameter         :: FAC_CO2 = 51.6043d0, V2M = 1.5278d0, &
                               FAC_H2O = 0.1d0

  real*8, dimension(19)     :: LAT_EDGE_5  
!http://www.atmos.washington.edu/~ignatius/CloudMap/WebO/index.html
  real*8, parameter, dimension(18) :: PER_OCN_S = (/ 74.0d0,  76.0d0,  79.0d0,  76.0d0,  75.0d0, &
                                                     78.0d0,  84.0d0,  93.0d0,  96.0d0,  97.0d0, &
                                                     98.0d0,  100.0d0, 100.0d0, 78.0d0,  40.0d0, &
                                                     12.0d0,  0.0d0,   0.0d0 /),                 &
                                      PER_OCN_N = (/ 78.0d0,  74.0d0,  75.0d0,  70.0d0,  64.0d0, &
                                                     59.0d0,  57.0d0,  55.0d0,  50.0d0,  43.0d0, &
                                                     39.0d0,  43.0d0,  28.0d0,  25.0d0,  59.0d0, &
                                                     71.0d0,  83.0d0,  100.0d0 /)


!-----------------------------------------------------
!read input files
!-----------------------------------------------------
  read(ATM_FID,nl_boxdef)
  read(ATM_FID,nl_options)
  read(ATM_FID,nl_clouds)
  read(ATM_FID,nl_radiation)
  read(ATM_FID,nl_tune_params)
  read(ATM_FID,nl_ocn_params)
  read(ATM_FID,nl_hyst)



!-----------------------------------------------------
!Miscalaneous (that stuff to come needs)
!-----------------------------------------------------
  PI = acos(-1.0)

!equator to pole distance
  L = R * PI / 2.0d0 
 
  CO2 = CO20

  EPSILON = RD / RV

  DT_BTROP  = DT / dble(F_BTROP_DT)
  DT_BCLIN  = DT / dble(F_BCLIN_DT)



!-----------------------------------------------------
!Latitudes and Land fractions
!-----------------------------------------------------

!set latitude of edges of boxes, AUTO_LAT sets boxes to be equal in latitudinal extent
  if ( AUTO_LAT ) then
     DLAT = ( MAX_LAT - MIN_LAT ) / dble(JMAX)
     LAT_EDGE = (/ ( MIN_LAT + DLAT * dble(J), J=0, JMAX ) /)
  else
     LAT_EDGE(:) = LAT_EDGE0(:)  
  endif
  
  LAT(:) = 0.5d0 * ( LAT_EDGE(1:JMAX) + LAT_EDGE(2:JMAX+1) )

!need to write latitudes to these files because the matlab code that 
!reads them reads and uses these latitudes for plotting

  call write_for_plotting(JMAX,LAT(:),DUMMY,'Lat Cent')
  call write_for_plotting(JMAX+1,LAT_EDGE(:),DUMMY,'Lat Edge')

!latitudes in radians
  LAT_RADS(:)      = LAT(:)      * PI / 180.0d0
  LAT_EDGE_RADS(:) = LAT_EDGE(:) * PI / 180.0d0

  if( CONST_FRAC_LAND ) then
     FRAC_LAND(:) = FRAC_LAND0
  else
     !put land fractions on correct grid
     if( HEMI_S ) then
        FRAC_LAND_5(:) = 1.0d0 - PER_OCN_S(:) / 100.0d0
     else
        FRAC_LAND_5(:) = 1.0d0 - PER_OCN_N(:) / 100.0d0
     endif
   
     LAT_EDGE_5 = (/ ( dble(J) * 5.0d0, J=0, 18 ) /) * PI / 180.0d0
   
     FRAC_LAND(:) = interpolate_frac( LAT_EDGE_5, LAT_EDGE_RADS, FRAC_LAND_5 )
  endif

!longitude range in radients
  DLON_RADS    = DLON * PI / 180.0d0

!calculate latitudinal distances
  Y(:)      = R * LAT_RADS(:) 
  Y_EDGE(:) = R * LAT_EDGE_RADS(:)  

  F(:)      = 2 * OMEGA_ROT * sin( LAT_RADS(:) )
  F_EDGE(:) = 2 * OMEGA_ROT * sin( LAT_EDGE_RADS(:) )


  SURF_AREA(1:JMAX) = R * R * DLON_RADS * ( sin( LAT_EDGE_RADS(2:JMAX+1) ) - sin( LAT_EDGE_RADS(1:JMAX) ) )

  SW_FLUX(1:JMAX)   = ( Q0 / 4.0d0 ) * ( 1.0d0 - Q2 / 2.0d0 + ( Q2 / 2.0d0 )                    * &
                 ( sin( LAT_EDGE_RADS(2:JMAX+1) ) ** 3.0d0 - sin( LAT_EDGE_RADS(1:JMAX) ) ** 3.0d0 ) / &
                 ( sin( LAT_EDGE_RADS(2:JMAX+1) ) - sin( LAT_EDGE_RADS(1:JMAX) ) ) )

  do K=1,KMAX
     MASS(:,K) = SURF_AREA(:) * ( P_EDGE(K+1) - P_EDGE(K) ) / G
  enddo


  DX(:) = R * DLON_RADS * cos( LAT_EDGE_RADS(:) )






!-----------------------------------------------------
!surface albedo stuff
!-----------------------------------------------------

!  do J = 1, JMAX
!     if( LAT_EDGE( J+1 ) .le. LAT_ICE ) then
!        ALPHA_LAND( J ) = ALPHA_LAND_NOICE
!        ALPHA_OCN(  J ) = ALPHA_OCN_NOICE
!     elseif( ( LAT_EDGE( J ) .lt. LAT_ICE ) .and. ( LAT_EDGE( J+1 ) .gt. LAT_ICE ) ) then
!        ALPHA_LAND( J ) = ( ( sin( LAT_ICE ) - sin( LAT_EDGE( J ) ) ) * ALPHA_LAND_NOICE + &
!                            ( sin( LAT_EDGE( J+1 ) ) - sin( LAT_ICE ) ) * ALPHA_LAND_ICE ) / &
!                          ( sin( LAT_EDGE( J+1 ) ) - sin( LAT_EDGE( J ) ) )
!        ALPHA_OCN(  J ) = ( ( sin( LAT_ICE ) - sin( LAT_EDGE( J ) ) ) * ALPHA_LAND_NOICE + &
!                            ( sin( LAT_EDGE( J+1 ) ) - sin( LAT_ICE ) ) * ALPHA_LAND_ICE ) / &
!                          ( sin( LAT_EDGE( J+1 ) ) - sin( LAT_EDGE( J ) ) )
!     else
!        ALPHA_LAND( J ) = ALPHA_LAND_ICE
!        ALPHA_OCN(  J ) = ALPHA_OCN_ICE
!     endif
!  enddo



!using a new surface albedo relation. For each land and ocean, set to
!const. south of ice latitude, then go linearly to maximum at pole
!above ice latitude. (dsa, 10/26/06)

  where( LAT(:) .le. LAT_ICE_OCN )
     ALPHA_OCN(:) = ALPHA_OCN_NOICE
  elsewhere
     ALPHA_OCN(:) = ALPHA_OCN_NOICE + ( ALPHA_OCN_ICE - ALPHA_OCN_NOICE ) * ( LAT(:) - LAT_ICE_OCN ) & 
                    / ( MAX_LAT - LAT_ICE_OCN )
  endwhere

  where( LAT(:) .le. LAT_ICE_LAND )
     ALPHA_LAND(:) = ALPHA_LAND_NOICE
  elsewhere
     ALPHA_LAND(:) = ALPHA_LAND_NOICE + ( ALPHA_LAND_ICE - ALPHA_LAND_NOICE ) * ( LAT(:) - LAT_ICE_LAND ) & 
                     / ( MAX_LAT - LAT_ICE_LAND )
  endwhere


!average surface albedo
  AVG_SURF_ALB = sum( ( FRAC_LAND(:) * ALPHA_LAND(:) + ( 1.0d0 - FRAC_LAND(:) ) * ALPHA_OCN(:) ) &
                        * SURF_AREA(:), 1 ) / sum( SURF_AREA(:), 1 )

!initial albedo
! ALB0(:) =  1.0d0 - ( FRAC_LAND(:) * (1.0d0 - ALPHA_LAND(:) )               + &                       
!                    ( 1.0d0 - FRAC_LAND(:) ) * ( 1.0d0 - ALPHA_OCN(:) ) )   * &
!                    ( 1.0d0 - ALPHA_ATM )                                  * &                            
!                    ( 1.0d0 - HCLD0(:) * ALPHA_H_CLD )                     * &
!                    ( 1.0d0 - MCLD0(:) * ALPHA_CLD(1) )                    * &
!                    ( 1.0d0 - LCLD0(:) * ALPHA_CLD(2) )                    
!  ALB0(:) = 0.0d0

!  ALB0_AVG = SUM( ALB0(:) * SURF_AREA(:), 1 ) / SUM( SURF_AREA(:), 1 )






!-----------------------------------------------------
!Assumed Ocean heat transport
!-----------------------------------------------------

!for ocean heat transport use a function that roughly fits Figure 7 from Trenberth and Caron 2001
!also ensure no OHT across boundaries of domain (dsa, 12/29/05)
  if( INT_OCN ) then
!     OHT(2:JMAX) = OHT0 * 1.0d15 * exp(1.0d0) * ( LAT_EDGE(2:JMAX) / OHT_DECAY ) * &
!                   exp( - LAT_EDGE(2:JMAX) / OHT_DECAY )
!     OHT(1)      = 0.0d0
!     OHT(JMAX+1) = 0.0d0

     !added a linear portion on the northern end that goes zero to
     !avoid excessive OHT convergence at the pole 
     !(dsa,1/4/06)

     OHT_LIN = OHT0 * 1.0d15 * exp(1.0d0) * ( LAT_OHT_LIN / OHT_DECAY ) * &
               exp( - LAT_OHT_LIN / OHT_DECAY )

     OHT(1)  = 0.0d0

     where( LAT_EDGE(2:JMAX) .lt. LAT_OHT_LIN )

        OHT(2:JMAX) = OHT0 * 1.0d15 * exp(1.0d0) * ( LAT_EDGE(2:JMAX) / OHT_DECAY ) * &
                        exp( - LAT_EDGE(2:JMAX) / OHT_DECAY )

     elsewhere

        where( ( LAT_EDGE(2:JMAX) .ge. LAT_OHT_LIN ) .and. &
               ( LAT_EDGE(2:JMAX) .le. LAT_OHT_ZERO ) )

           OHT(2:JMAX) = OHT_LIN * ( ( LAT_OHT_ZERO - LAT_EDGE(2:JMAX) ) / &
                                       ( LAT_OHT_ZERO - LAT_OHT_LIN ) )
        
        elsewhere
        
           OHT(2:JMAX) = 0.0d0

        endwhere

     endwhere

  else
     OHT(:) = 0.0d0
  endif
  
  OHT(JMAX+1)=0.0d0

  OHT_S(:) = OHT(:)

  OHT_B = OHT0_GAUSS

  OHT_GAUSS(2:JMAX) = OHT0_GAUSS * 1.0d15 * exp( - ( LAT_EDGE(2:JMAX) - LAT_GAUSS ) ** 2.0d0 / &
                      ( 2 * SIG_GAUSS ** 2.0d0) )

  OHT(2:JMAX) = OHT(2:JMAX) + OHT_GAUSS(2:JMAX)

!  write(6,*) OHT(:)/1.0d15
!  stop


  
!-----------------------------------------------------
!stuff for ozone emissivity
!-----------------------------------------------------

!calculate approximate heights
!assume dry adiabat here, for simplicity
!assume surface temp is 300 K, want to associate one height with each level
  Z(:)      = ( 1 - ( P(:)      / P_EDGE(3) ) ** ( RD / CPD ) ) * ( CPD * 300.0d0 / G )
  Z_EDGE(:) = ( 1 - ( P_EDGE(:) / P_EDGE(3) ) ** ( RD / CPD ) ) * ( CPD * 300.0d0 / G )

!use these approximate heights for ozone emissivity calculation.  This is very rough at this point.  
!Put the call at the begining so that it only must occur once
  U1        = path_length_o3( Z_EDGE(2) / 1000.0d0 ) - path_length_o3( Z_EDGE(1) / 1000.0d0 )
  U2        = path_length_o3( Z_EDGE(3) / 1000.0d0 ) - path_length_o3( Z_EDGE(2) / 1000.0d0 )

  UO3_1     =  (/ (U1,J=1,JMAX) /)
  UO3_2     =  (/ (U2,J=1,JMAX) /)

  EMISS_O3(:,1) = emissivity_o3( UO3_1 )
  EMISS_O3(:,2) = emissivity_o3( UO3_2 )


  do K = 1, KMAX
     UCO2(:) = ( FAC_CO2 * V2M * CO2_REF / ( 2.0d0 * P_EDGE(3) * G) ) * &
            ( P_EDGE(K+1) ** 2.0d0 - P_EDGE(K) ** 2.0d0 )
     UH2O(:) = ( FAC_H2O * H2O_REF(K) / ( 2.0d0 * P_EDGE(3) * G) ) * &
            ( P_EDGE(K+1) ** 2.0d0 - P_EDGE(K) ** 2.0d0 )


     EMISS_CLEAR_REF2(:,K) = emissivity_co2( UCO2(:) )                                      + &
                             emissivity_h2o( UH2O(:) ) * transmissivity_h2o_15mu( UH2O(:) ) + &
                             EMISS_O3(:,K)
  enddo





!-----------------------------------------------------
!More Miscalaneous (That needs previous stuff)
!-----------------------------------------------------
 
!vertical descent beyond which convection is shut off
  OMEGA_NO_CONV(:) = 1.0d9 * CONV_SQUASH * G / SURF_AREA(:)


!write to file to see that starting variables are as expected
  write(IN_FID,*) "P:"
  write(IN_FID,*) P
  write(IN_FID,*) "P_EDGE:"
  write(IN_FID,*) P_EDGE
  write(IN_FID,*) "LAT:"
  write(IN_FID,*) LAT
  write(IN_FID,*) "LAT_EDGE:"
  write(IN_FID,*) LAT_EDGE
  write(IN_FID,*) "SURF_AREA [m^2]:"
  write(IN_FID,*) SURF_AREA
  write(IN_FID,*) "DX [m]:"
  write(IN_FID,*) DX
  write(IN_FID,*) "F [s^-1]:"
  write(IN_FID,*) F
  write(IN_FID,*) "F_EDGE [s^-1]:"
  write(IN_FID,*) F_EDGE
  write(IN_FID,*) "Land Fraction:"
  write(IN_FID,*) FRAC_LAND
  write(IN_FID,*) "Global Avg. Land Frac:"
  write(IN_FID,*) sum( FRAC_LAND(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  write(IN_FID,*) "Land Albedo:"
  write(IN_FID,*) ALPHA_LAND(:)
  write(IN_FID,*) "Ocean Albedo:"
  write(IN_FID,*) ALPHA_OCN(:)
  write(IN_FID,*) "Surface  Albedo:"
  write(IN_FID,*) FRAC_LAND(:) * ALPHA_LAND + ( 1.0d0 - FRAC_LAND(:) ) * ALPHA_OCN(:)
  write(IN_FID,*) "Global Avg. Surface Albedo:"
  write(IN_FID,*) AVG_SURF_ALB
!  write(IN_FID,*) "Initial Total Albedo:"
!  write(IN_FID,*) ALB0
!  write(IN_FID,*) "Initial Global Avg. Total Albedo:"
!  write(IN_FID,*) ALB0_AVG
  write(IN_FID,*) "Ocean Heat Transport [PW]:"
  write(IN_FID,*) OHT/1.0d15

!  stop

end subroutine get_parameters


!******************************************************************************
! (03) subroutine init_atm 
!      must be called FIRST in main.f90
!******************************************************************************
subroutine init_atm
  integer :: AS

  allocate( LAT(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:LAT'
  LAT(:) = 0.0d0

  allocate( LAT_EDGE(JMAX+1), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:LAT_EDGE'
  LAT_EDGE(:) = 0.0d0

  allocate( Y(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:Y'
  Y(:) = 0.0d0

  allocate( Y_EDGE(JMAX+1), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:Y_EDGE'
  Y_EDGE(:) = 0.0d0

  allocate( FRAC_LAND(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:FRAC_LAND'
  FRAC_LAND(:) = 0.0d0

  allocate( V_EDGE(JMAX+1,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:V_EDGE'
  V_EDGE(:,:) = 0.0d0

  allocate( OMEGA(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:OMEGA'
  OMEGA(:) = 0.0d0

  allocate( ALPHA(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:ALPHA'
  ALPHA(:,:) = 0.0d0

  allocate( THETA(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:THETA'
  THETA(:,:) = 0.0d0

  allocate( DSE(JMAX,KMAX,ROBERT), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:DSE'
  DSE(:,:,:) = 0.0d0

  allocate( MSE(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:MSE'
  MSE(:,:) = 0.0d0

  allocate( MSE_STAR(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:MSE_STAR'
  MSE_STAR(:,:) = 0.0d0

  allocate( MSE_STAR_TILDE(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:MSE_STAR_TILDE'
  MSE_STAR_TILDE(:) = 0.0d0

  allocate( Q(JMAX,KMAX,ROBERT), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:Q'
  Q(:,:,:) = 0.0d0

  allocate( QSTART(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:QSTART'
  QSTART(:,:) = 0.0d0

  allocate( F(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:F'
  F(:) = 0.0d0

  allocate( F_EDGE(JMAX+1), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:F_EDGE'
  F_EDGE(:) = 0.0d0

  allocate( R_TOT(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:R_TOT'
  R_TOT(:,:) = 0.0d0

  allocate( CP_TOT(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:CP_TOT'
  CP_TOT(:,:) = 0.0d0

  allocate( T(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:T'
  T(:,:) = 0.0d0

  allocate( T_H_CLD(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:T_H_CLD'
  T_H_CLD(:) = 0.0d0

  allocate( VOL_M3(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:VOL_M3'
  VOL_M3(:,:) = 0.0d0

  allocate( EMISS(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:EMISS'
  EMISS(:,:) = 0.0d0

  allocate( EMISS_CLEAR(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:EMISS_CLEAR'
  EMISS_CLEAR(:,:) = 0.0d0

  allocate( EMISS_O3(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:EMISS_O3'
  EMISS_O3(:,:) = 0.0d0

  allocate( EMISS_CLEAR_REF1(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:EMISS_CLEAR_REF1'
  EMISS_CLEAR_REF1(:,:) = 0.0d0

  allocate( EMISS_CLEAR_REF2(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:EMISS_CLEAR_REF2'
  EMISS_CLEAR_REF2(:,:) = 0.0d0

  allocate( EMISS_H(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:EMISS_H'
  EMISS_H(:) = 0.0d0

  allocate( ALPHA_LAND(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:ALPHA_LAND'
  ALPHA_LAND(:) = 0.0d0

  allocate( ALPHA_OCN(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:ALPHA_OCN'
  ALPHA_OCN(:) = 0.0d0

  allocate( FRAC_CLD(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:FRAC_CLD'
  FRAC_CLD(:,:) = 0.0d0

  allocate( FRAC_H_CLD(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:FRAC_H_CLD'
  FRAC_H_CLD(:) = 0.0d0

  allocate( Z(KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:Z'
  Z(:) = 0.0d0

  allocate( Z_EDGE(KMAX+1), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:Z_EDGE'
  Z_EDGE(:) = 0.0d0

  allocate( SST(JMAX,ROBERT), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:SST'
  SST(:,:) = 0.0d0

  allocate( TS(JMAX,ROBERT), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:TS'
  TS(:,:) = 0.0d0

  allocate( OCN_SH_FLUX(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:OCN_SH_FLUX'
  OCN_SH_FLUX(:) = 0.0d0

  allocate( OCN_EVAP_FLUX(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:OCN_EVAP_FLUX'
  OCN_EVAP_FLUX(:) = 0.0d0

  allocate( LAND_SH_FLUX(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:LAND_SH_FLUX'
  LAND_SH_FLUX(:) = 0.0d0

  allocate( LAND_EVAP_FLUX(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:LAND_EVAP_FLUX'
  LAND_EVAP_FLUX(:) = 0.0d0

  allocate( RUNOFF(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:RUNOFF'
  RUNOFF(:) = 0.0d0

  allocate( PRECIP(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:PRECIP'
  PRECIP(:,:) = 0.0d0

  allocate( FW_FLUX(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:FW_FLUX'
  FW_FLUX(:) = 0.0d0

  allocate(RH(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:RH'
  RH(:,:) = 0.0d0

  allocate(SW_FLUX(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:SW_FLUX'
  SW_FLUX(:) = 0.0d0

  allocate(LAT_RADS(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:LAT_RADS'
  LAT_RADS(:) = 0.0d0

  allocate(LAT_EDGE_RADS(JMAX+1), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:LAT_EDGE_RADS'
  LAT_EDGE_RADS(:) = 0.0d0

  allocate(SURF_AREA(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:SURF_AREA'
  SURF_AREA(:) = 0.0d0

  allocate(R_MIX(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:R_MIX'
  R_MIX(:,:) = 0.0d0

  allocate(THETA_E(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:THETA_E'
  THETA_E(:,:) = 0.0d0

  allocate(THETA_E_STAR(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:THETA_E_STAR'
  THETA_E_STAR(:,:) = 0.0d0

  allocate(LV(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:LV'
  LV(:,:) = 0.0d0

  allocate(E(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:E'
  E(:,:) = 0.0d0

  allocate(E_STAR(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:E_STAR'
  E_STAR(:,:) = 0.0d0
 
  allocate(Q_STAR(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:Q_STAR'
  Q_STAR(:,:) = 0.0d0

  allocate(R_MIX_STAR(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:R_MIX_STAR'
  R_MIX_STAR(:,:) = 0.0d0

  allocate(EVAP_FRAC(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:EVAP_FRAC'
  EVAP_FRAC(:) = 0.0d0

  allocate(EDDY_HEAT(JMAX+1,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:EDDY_HEAT'
  EDDY_HEAT(:,:) = 0.0d0

  allocate(EDDY_MOIST(JMAX+1,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:EDDY_MOIST'
  EDDY_MOIST(:,:) = 0.0d0

  allocate(ADV_MOIST(JMAX+1,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:ADV_MOIST'
  ADV_MOIST(:,:) = 0.0d0

  allocate(EDDY_LATENT(JMAX+1,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:EDDY_LATENT'
  EDDY_LATENT(:,:) = 0.0d0

  allocate(ADVECT_HEAT_Y(JMAX+1,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:ADVECT_HEAT_Y'
  ADVECT_HEAT_Y(:,:) = 0.0d0

  allocate(ADVECT_HEAT_P(JMAX,KMAX-1), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:ADVECT_HEAT_P'
  ADVECT_HEAT_P(:,:) = 0.0d0

  allocate(ADVECT_LATENT_Y(JMAX+1,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:ADVECT_LATENT_Y'
  ADVECT_LATENT_Y(:,:) = 0.0d0

  allocate(ADVECT_LATENT_P(JMAX,KMAX-1), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:ADVECT_LATENT_P'
  ADVECT_LATENT_P(:,:) = 0.0d0

  allocate(OLR(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:OLR'
  OLR(:) = 0.0d0

  allocate(F_LW_SURF_UP(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:F_LW_SURF_UP'
  F_LW_SURF_UP(:) = 0.0d0

  allocate(F_LW_DOWN(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:F_LW_DOWN'
  F_LW_DOWN(:) = 0.0d0

  allocate(ABSORB_SW(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:ABSORB_SW'
  ABSORB_SW(:) = 0.0d0

  allocate(F_SW_LAND_DOWN(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:F_SW_LAND_DOWN'
  F_SW_LAND_DOWN(:) = 0.0d0

  allocate(F_SW_OCN_DOWN(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:F_SW_OCN_DOWN'
  F_SW_OCN_DOWN(:) = 0.0d0

  allocate(FLUX_IN_OCN(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:FLUX_IN_OCN'
  FLUX_IN_OCN(:) = 0.0d0

  allocate(EPS_RE_EVAP(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:EPS_RE_EVAP'
  EPS_RE_EVAP(:) = 0.0d0

  allocate(RE_EVAP(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:RE_EVAP'
  RE_EVAP(:) = 0.0d0

  allocate(HEATING(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:HEATING'
  HEATING(:) = 0.0d0

  allocate(PRECIP_G(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:PRECIP_G'
  PRECIP_G(:) = 0.0d0

  allocate(DX(JMAX+1), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:DX'
  DX(:) = 0.0d0

  allocate(BTROP_U(JMAX,ROBERT), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:BTROP_U'
  BTROP_U(:,:) = 0.0d0

  allocate(BCLIN_U(JMAX,KMAX,ROBERT), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:BCLIN_U'
  BCLIN_U(:,:,:) = 0.0d0

  allocate(U(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:U'
  U(:,:) = 0.0d0

  allocate(BCLIN_V(JMAX+1,KMAX,ROBERT), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:BCLIN_V'
  BCLIN_V(:,:,:) = 0.0d0

  allocate(BTROP_PHI(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:BTROP_PHI'
  BTROP_PHI(:) = 0.0d0

  allocate(BCLIN_PHI(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:BCLIN_PHI'
  BCLIN_PHI(:,:) = 0.0d0

  allocate(PHI(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:PHI'
  PHI(:,:) = 0.0d0

  allocate(MASS(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:MASS'
  MASS(:,:) = 0.0d0

  allocate(ANG_MOM(JMAX,KMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:ANG_MOM'
  ANG_MOM(:,:) = 0.0d0

  allocate(K_CON(JMAX), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:K_CON'
  K_CON(:) = 0.0d0

  allocate(OHT(JMAX+1), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:OHT'
  OHT(:) = 0.0d0

  allocate(OHT_S(JMAX+1), STAT=AS )
  if ( AS /=0 ) write(6,*) 'ALLOCATION ERROR, INIT_ATM:OHT_S'
  OHT_S(:) = 0.0d0

end subroutine init_atm



!******************************************************************************
! (04) subroutine cleanup_atm
!      must be called LAST in main.f90
!******************************************************************************
subroutine cleanup_atm

  if( allocated( LAT              ) ) deallocate( LAT              )
  if( allocated( LAT_EDGE         ) ) deallocate( LAT_EDGE         )
  if( allocated( Y                ) ) deallocate( Y                )
  if( allocated( Y_EDGE           ) ) deallocate( Y_EDGE           )
  if( allocated( FRAC_LAND        ) ) deallocate( FRAC_LAND        )
  if( allocated( V_EDGE           ) ) deallocate( V_EDGE           )
  if( allocated( OMEGA            ) ) deallocate( OMEGA            )
  if( allocated( ALPHA            ) ) deallocate( ALPHA            )
  if( allocated( THETA            ) ) deallocate( THETA            )
  if( allocated( DSE              ) ) deallocate( DSE              )
  if( allocated( MSE              ) ) deallocate( MSE              )
  if( allocated( MSE_STAR         ) ) deallocate( MSE_STAR         )
  if( allocated( MSE_STAR_TILDE   ) ) deallocate( MSE_STAR_TILDE   )
  if( allocated( Q                ) ) deallocate( Q                )
  if( allocated( QSTART           ) ) deallocate( QSTART           )
  if( allocated( F                ) ) deallocate( F                )
  if( allocated( F_EDGE           ) ) deallocate( F_EDGE           )
  if( allocated( R_TOT            ) ) deallocate( R_TOT            )
  if( allocated( CP_TOT           ) ) deallocate( CP_TOT           )
  if( allocated( T                ) ) deallocate( T                )
  if( allocated( T_H_CLD          ) ) deallocate( T_H_CLD          )
  if( allocated( VOL_M3           ) ) deallocate( VOL_M3           )
  if( allocated( EMISS            ) ) deallocate( EMISS            )
  if( allocated( EMISS_CLEAR      ) ) deallocate( EMISS_CLEAR      )
  if( allocated( EMISS_O3         ) ) deallocate( EMISS_O3         )
  if( allocated( EMISS_CLEAR_REF1 ) ) deallocate( EMISS_CLEAR_REF1 )
  if( allocated( EMISS_CLEAR_REF2 ) ) deallocate( EMISS_CLEAR_REF2 )
  if( allocated( EMISS_H          ) ) deallocate( EMISS_H          )
  if( allocated( ALPHA_LAND       ) ) deallocate( ALPHA_LAND       )
  if( allocated( ALPHA_OCN        ) ) deallocate( ALPHA_OCN        )
  if( allocated( FRAC_CLD         ) ) deallocate( FRAC_CLD         )
  if( allocated( FRAC_H_CLD       ) ) deallocate( FRAC_H_CLD       )
  if( allocated( Z                ) ) deallocate( Z                )
  if( allocated( Z_EDGE           ) ) deallocate( Z_EDGE           )
  if( allocated( SST              ) ) deallocate( SST              )
  if( allocated( TS               ) ) deallocate( TS               )
  if( allocated( OCN_SH_FLUX      ) ) deallocate( OCN_SH_FLUX      )
  if( allocated( OCN_EVAP_FLUX    ) ) deallocate( OCN_EVAP_FLUX    )
  if( allocated( LAND_SH_FLUX     ) ) deallocate( LAND_SH_FLUX     )
  if( allocated( LAND_EVAP_FLUX   ) ) deallocate( LAND_EVAP_FLUX   )
  if( allocated( RUNOFF           ) ) deallocate( RUNOFF           )
  if( allocated( PRECIP           ) ) deallocate( PRECIP           )
  if( allocated( FW_FLUX          ) ) deallocate( FW_FLUX          )
  if( allocated( RH               ) ) deallocate( RH               )
  if( allocated( SW_FLUX          ) ) deallocate( SW_FLUX          )
  if( allocated( LAT_RADS         ) ) deallocate( LAT_RADS         )
  if( allocated( LAT_EDGE_RADS    ) ) deallocate( LAT_EDGE_RADS    )
  if( allocated( SURF_AREA        ) ) deallocate( SURF_AREA        )
  if( allocated( R_MIX            ) ) deallocate( R_MIX            )
  if( allocated( LV               ) ) deallocate( LV               )
  if( allocated( E                ) ) deallocate( E                )
  if( allocated( E_STAR           ) ) deallocate( E_STAR           )
  if( allocated( Q_STAR           ) ) deallocate( Q_STAR           )
  if( allocated( R_MIX_STAR       ) ) deallocate( R_MIX_STAR       )
  if( allocated( EVAP_FRAC        ) ) deallocate( EVAP_FRAC        )
  if( allocated( EDDY_HEAT        ) ) deallocate( EDDY_HEAT        )
  if( allocated( ADV_MOIST        ) ) deallocate( ADV_MOIST        )
  if( allocated( EDDY_MOIST       ) ) deallocate( EDDY_MOIST       )
  if( allocated( EDDY_LATENT      ) ) deallocate( EDDY_LATENT      )
  if( allocated( ADVECT_HEAT_Y    ) ) deallocate( ADVECT_HEAT_Y    )
  if( allocated( ADVECT_HEAT_P    ) ) deallocate( ADVECT_HEAT_P    )
  if( allocated( ADVECT_LATENT_Y  ) ) deallocate( ADVECT_LATENT_Y  )
  if( allocated( ADVECT_LATENT_P  ) ) deallocate( ADVECT_LATENT_P  )
  if( allocated( OLR              ) ) deallocate( OLR              )
  if( allocated( F_LW_SURF_UP     ) ) deallocate( F_LW_SURF_UP     )
  if( allocated( F_LW_DOWN        ) ) deallocate( F_LW_DOWN        )
  if( allocated( ABSORB_SW        ) ) deallocate( ABSORB_SW        )
  if( allocated( F_SW_LAND_DOWN   ) ) deallocate( F_SW_LAND_DOWN   )
  if( allocated( F_SW_OCN_DOWN    ) ) deallocate( F_SW_OCN_DOWN    )
  if( allocated( FLUX_IN_OCN      ) ) deallocate( FLUX_IN_OCN      )
  if( allocated( EPS_RE_EVAP      ) ) deallocate( EPS_RE_EVAP      )
  if( allocated( RE_EVAP          ) ) deallocate( RE_EVAP          )
  if( allocated( HEATING          ) ) deallocate( HEATING          )
  if( allocated( PRECIP_G         ) ) deallocate( PRECIP_G         )
  if( allocated( DX               ) ) deallocate( DX               )
  if( allocated( BTROP_U          ) ) deallocate( BTROP_U          )
  if( allocated( BCLIN_U          ) ) deallocate( BCLIN_U          )
  if( allocated( U                ) ) deallocate( U                )
  if( allocated( BCLIN_V          ) ) deallocate( BCLIN_V          )
  if( allocated( BTROP_PHI        ) ) deallocate( BTROP_PHI        )
  if( allocated( BCLIN_PHI        ) ) deallocate( BCLIN_PHI        )
  if( allocated( PHI              ) ) deallocate( PHI              )
  if( allocated( MASS             ) ) deallocate( MASS             )
  if( allocated( ANG_MOM          ) ) deallocate( ANG_MOM          )
  if( allocated( K_CON            ) ) deallocate( K_CON            )
  if( allocated( OHT              ) ) deallocate( OHT              )
  if( allocated( OHT_S            ) ) deallocate( OHT_S            )


  close(UNIT=ATM_FID)
  close(UNIT=IN_FID)
  close(UNIT=RESTART_IN_FID)
  close(UNIT=RESTART_OUT_FID)
  close(UNIT=DIAGS_FID)
  close(UNIT=DIAGS_FID2)
  close(UNIT=DEBUG_FID)
  close(UNIT=DEBUG_FID2)
  close(UNIT=SS_BAL_FID)
  close(UNIT=SS_VEL_FID)
  close(UNIT=SS_TEMP_FID)
  close(UNIT=SS_MOIST_FID)
  close(UNIT=RADCHEAT_IN_FID)
  close(UNIT=RADCHEAT_OUT_FID)
  close(UNIT=PARAM_VARY_FID)
  close(UNIT=PARAM_VARY_FID2)
!  close(UNIT=CLD_FID)
  close(UNIT=JMAX_FID)
  close(UNIT=JMAX_LBL_FID)
  close(UNIT=JMAXm1_FID)
  close(UNIT=JMAXm1_LBL_FID)
  close(UNIT=SS_BAL_PLOT_FID)
  close(UNIT=SS_VEL_PLOT_FID)

end subroutine cleanup_atm



!******************************************************************************
! (05) subroutine write_restart
!******************************************************************************
subroutine write_restart

  if ( RAD_CHEAT_WRITE ) then
     write(RADCHEAT_OUT_FID,*) "EMISS_CLEAR_REF1:"
     write(RADCHEAT_OUT_FID,*) EMISS_CLEAR
     write(RADCHEAT_OUT_FID,*) "QSTART:"
     write(RADCHEAT_OUT_FID,*) Q(:,:,2) 
  endif


  if ( WRITE_RESTART_DATA ) then
     write(RESTART_OUT_FID,*) "ALPHA:"
     write(RESTART_OUT_FID,*) ALPHA
     write(RESTART_OUT_FID,*) "DSE:"
     write(RESTART_OUT_FID,*) DSE
     write(RESTART_OUT_FID,*) "Q:"
     write(RESTART_OUT_FID,*) Q
     write(RESTART_OUT_FID,*) "BTROP_U"
     write(RESTART_OUT_FID,*) BTROP_U
     write(RESTART_OUT_FID,*) "BCLIN_U"
     write(RESTART_OUT_FID,*) BCLIN_U
     write(RESTART_OUT_FID,*) "BCLIN_V"
     write(RESTART_OUT_FID,*) BCLIN_V
     write(RESTART_OUT_FID,*) "FRAC_CLD(:,1)"
     write(RESTART_OUT_FID,*) FRAC_CLD(:,1)
     write(RESTART_OUT_FID,*) "FRAC_CLD(:,2)"
     write(RESTART_OUT_FID,*) FRAC_CLD(:,2)
     write(RESTART_OUT_FID,*) "FRAC_H_CLD"
     write(RESTART_OUT_FID,*) FRAC_H_CLD
     write(RESTART_OUT_FID,*) "T_H_CLD"
     write(RESTART_OUT_FID,*) T_H_CLD
     write(RESTART_OUT_FID,*) "TS"
     write(RESTART_OUT_FID,*) TS
     write(RESTART_OUT_FID,*) "SST"
     write(RESTART_OUT_FID,*) SST
  end if 
end subroutine write_restart

!******************************************************************************
! (06) subroutine initialize_vars
!******************************************************************************
subroutine initialize_vars
  integer  :: I

  if ( ( RAD_CHEAT1   .eq. .true.  ) .or. &
       ( CONST_OFFSET .eq. .true.  ) .or. &
       ( Q_EM         .eq. .false. ) ) then
     read(RADCHEAT_IN_FID,*) !"EMISS_CLEAR_REF1:"
     read(RADCHEAT_IN_FID,*) EMISS_CLEAR_REF1
     read(RADCHEAT_IN_FID,*) !"QSTART:"
     read(RADCHEAT_IN_FID,*) QSTART 
  endif

  if ( READ_RESTART_DATA ) then
     read(RESTART_IN_FID,*) !"ALPHA:"
     read(RESTART_IN_FID,*) ALPHA
     read(RESTART_IN_FID,*) !"DSE:"
     read(RESTART_IN_FID,*) DSE
     read(RESTART_IN_FID,*) !"Q:"
     read(RESTART_IN_FID,*) Q
     read(RESTART_IN_FID,*) !"BTROP_U"
     read(RESTART_IN_FID,*) BTROP_U
     read(RESTART_IN_FID,*) !"BCLIN_U"
     read(RESTART_IN_FID,*) BCLIN_U
     read(RESTART_IN_FID,*) !"BCLIN_V"
     read(RESTART_IN_FID,*) BCLIN_V
     read(RESTART_IN_FID,*) !"FRAC_CLD(:,1)"
     read(RESTART_IN_FID,*) FRAC_CLD(:,1)
     read(RESTART_IN_FID,*) !"FRAC_CLD(:,2)"
     read(RESTART_IN_FID,*) FRAC_CLD(:,2)
     read(RESTART_IN_FID,*) !"FRAC_H_CLD"
     read(RESTART_IN_FID,*) FRAC_H_CLD
     read(RESTART_IN_FID,*) !"T_H_CLD"
     read(RESTART_IN_FID,*) T_H_CLD
     read(RESTART_IN_FID,*) !"TS"
     read(RESTART_IN_FID,*) TS
     read(RESTART_IN_FID,*) !"SST"
     read(RESTART_IN_FID,*) SST

  else 

     DSE(:,1,:) = 3.0d5
     DSE(:,2,:) = 3.05d5
     
     Q(:,1,1) = 0.003d0
     Q(:,1,2) = 0.003d0
     Q(:,1,3) = 0.003d0
     Q(:,2,1) = 0.02d0
     Q(:,2,2) = 0.02d0
     Q(:,2,3) = 0.02d0

     FRAC_CLD(:,1) = 0.1d0
     FRAC_CLD(:,2) = 0.4d0
     FRAC_H_CLD(:) = 0.0d0

     !land temps
     TS(:,:)  = 310.0d0

     !initialize SST's  
     SST(:,:) = 310.0d0

!     DSE(:,1,:) = 3.0d5
!     DSE(:,2,:) = 3.05d5
!     
!     Q(:,1,1) = 0.001d0
!     Q(:,1,2) = 0.001d0
!     Q(:,1,3) = 0.001d0
!     Q(:,2,1) = 0.005d0
!     Q(:,2,2) = 0.005d0
!     Q(:,2,3) = 0.005d0
!
!     FRAC_CLD(:,1) = 0.1d0
!     FRAC_CLD(:,2) = 0.4d0
!     FRAC_H_CLD(:) = 0.0d0
!
!     !land temps
!     TS(:,:)  = 270.0d0
!
!     !initialize SST's  
!     SST(:,:) = 270.0d0


  end if

  if( .not. INT_OCN ) then
     SST(:,:) = 295.0d0
  endif

  !for now set evaporative fraction to 0.5, will need to re-consider later
  EVAP_FRAC(:) = (/ (EFRAC, I=1,JMAX) /)

  !get variables ready for run
  call update_all_vars

!---------------------------------------------------------------------
!---------------------------------------------------------------------
!this is just for some special variation runs. It should generally be
!commented out. (dsa, 11/14/06)
!---------------------------------------------------------------------
!  FRAC_CLD(1:5,1)  = C_C_MAX
!  FRAC_CLD(1:5,2)  = 0.0d0
!  FRAC_H_CLD(1:5)  = C_C_MAX
!  FRAC_CLD(6:12,1) = 0.0d0
!  FRAC_CLD(6:12,2) = C_S_MAX(2)
!  FRAC_H_CLD(6:12) = 0.0d0
!
!  if( DIAG_CLDS ) then
!     FRAC_CLD(13:30,1) = C_C_MAX
!     FRAC_CLD(13:30,2) = 0.0d0
!     FRAC_H_CLD(13:30) = C_C_MAX
!  else
!     FRAC_CLD(13:30,1) = 0.0d0
!     FRAC_CLD(13:30,2) = C_S_MAX(2)
!     FRAC_H_CLD(13:30) = 0.0d0
!  endif
!---------------------------------------------------------------------
!---------------------------------------------------------------------

end subroutine initialize_vars



!******************************************************************************
! (07) subroutine atm_step
!      *a VERY important subroutine!
!      *should be run each time we wish to step the model forward
!******************************************************************************
subroutine atm_step

  integer      :: J,K
  real*8       :: TEND_DSE(JMAX,KMAX),TEND_Q(JMAX,KMAX), &
                  TEND_TS(JMAX),TEND_SST(JMAX),          &
                  TEND_BTROP_U(JMAX),                    &
                  TEND_BCLIN_U(JMAX,KMAX),               &
                  TEND_BCLIN_V(JMAX+1,KMAX)
  TEND_DSE(:,:)     = 0.0d0
  TEND_Q(:,:)       = 0.0d0
  TEND_TS(:)        = 0.0d0
  TEND_SST(:)       = 0.0d0
  TEND_BTROP_U(:)   = 0.0d0
  TEND_BCLIN_U(:,:) = 0.0d0
  TEND_BCLIN_V(:,:) = 0.0d0

  call update_all_vars



!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!step forward THETA and Q
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

!calculate advective tendencies
  TEND_DSE = advection_tend_dse()
  TEND_Q   = advection_tend_q()

!add radiative tendencies
  TEND_DSE = TEND_DSE + rad_tend_dse()

!add baroclinic eddy tendencies
!  TEND_DSE = TEND_DSE + baroclinic_tend_dse()
!  TEND_Q   = TEND_Q   + baroclinic_tend_q()

  TEND_DSE = TEND_DSE + get_eddy_tend( DSE(:,:,2), 1 )
  TEND_Q   = TEND_Q   + get_eddy_tend( Q(:,:,2),   2 )

!add surface flux tendencies
  TEND_DSE = TEND_DSE + sens_heat_tend_dse()
  TEND_Q   = TEND_Q   + evaporation_tend_q()

!add convection tendencies
  TEND_DSE = TEND_DSE + convection_tend_dse()
  TEND_Q   = TEND_Q   + convection_tend_q()

!add precipitation tendencies
  TEND_DSE = TEND_DSE + precipitation_tend_dse()
  TEND_Q   = TEND_Q   + precipitation_tend_q()

!add re-evaporation of precipitation tendencies
  TEND_DSE = TEND_DSE + re_evaporation_tend_dse()
  TEND_Q   = TEND_Q   + re_evaporation_tend_q()


  !inserted call here to see tendency balances at very last call before 
  !theta and q have been updated 
  if ( TIME .gt. ( TEND - 2.0d0 * DT ) ) then
     call investigate_steady_state
     call investigate_steady_v
     call investigate_steady_temps
     call investigate_steady_moist
  end if

!integrate DSE forward in time using a robert filter
  DSE(:,:,3) = DSE(:,:,1) + 2.0d0 * DT * TEND_DSE(:,:)
  DSE(:,:,2) = DSE(:,:,2) + ROBERT_FILTER * ( DSE(:,:,3) - 2.0d0 * DSE(:,:,2) + DSE(:,:,1) )
  DSE(:,:,1) = DSE(:,:,2)
  DSE(:,:,2) = DSE(:,:,3)

!integrate Q
  Q(:,:,3) = Q(:,:,1) + 2.0d0 * DT * TEND_Q(:,:)
  Q(:,:,2) = Q(:,:,2) + ROBERT_FILTER * ( Q(:,:,3) - 2.0d0 * Q(:,:,2) + Q(:,:,1) )
  Q(:,:,1) = Q(:,:,2)
  Q(:,:,2) = Q(:,:,3)


!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!step forward TS
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

!land surface temperature tendency
  TEND_TS = calc_tend_ts()

  TS(:,3) = TS(:,1) + 2.0d0 * DT * TEND_TS(:)
  TS(:,2) = TS(:,2) + ROBERT_FILTER * ( TS(:,3) - 2.0d0 * TS(:,2) + TS(:,1) )
  TS(:,1) = TS(:,2)
  TS(:,2) = TS(:,3)




!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!step forward SST
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

  if( INT_OCN ) then
     !sea surface temperature tendency
     TEND_SST = calc_tend_sst()
   
     SST(:,3) = SST(:,1) + 2.0d0 * DT * TEND_SST(:)
     SST(:,2) = SST(:,2) + ROBERT_FILTER * ( SST(:,3) - 2.0d0 * SST(:,2) + SST(:,1) )
     SST(:,1) = SST(:,2)
     SST(:,2) = SST(:,3)
  endif


!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!step forward BTROP_U,BCLIN_U,BCLIN_V
!notice the modified physics method of finding a 
!steady-state!!!!  See Bryan (1984)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

   TEND_BTROP_U(:) = tendency_btrop_u()
 
   BTROP_U(:,3) = BTROP_U(:,1) + 2.0d0 * DT_BTROP * TEND_BTROP_U(:)
   BTROP_U(:,2) = BTROP_U(:,2) + ROBERT_FILTER * ( BTROP_U(:,3) - 2.0d0 * BTROP_U(:,2) + BTROP_U(:,1) )
   BTROP_U(:,1) = BTROP_U(:,2)
   BTROP_U(:,2) = BTROP_U(:,3)


!update baroclinic zonal velocity
  TEND_BCLIN_U(:,:) = tendency_bclin_u()

  BCLIN_U(:,:,3) = BCLIN_U(:,:,1) + 2.0d0 * DT_BCLIN * TEND_BCLIN_U(:,:)
  BCLIN_U(:,:,2) = BCLIN_U(:,:,2) + ROBERT_FILTER * ( BCLIN_U(:,:,3) - 2.0d0 * BCLIN_U(:,:,2) + BCLIN_U(:,:,1) )
  BCLIN_U(:,:,1) = BCLIN_U(:,:,2)
  BCLIN_U(:,:,2) = BCLIN_U(:,:,3)

!update baroclinic meridional velocity
  TEND_BCLIN_V(:,:) = tendency_bclin_v()

  BCLIN_V(:,:,3) = BCLIN_V(:,:,1) + 2.0d0 * DT_BCLIN * TEND_BCLIN_V(:,:)
  BCLIN_V(:,:,2) = BCLIN_V(:,:,2) + ROBERT_FILTER * ( BCLIN_V(:,:,3) - 2.0d0 * BCLIN_V(:,:,2) + BCLIN_V(:,:,1) )
  BCLIN_V(:,:,1) = BCLIN_V(:,:,2)
  BCLIN_V(:,:,2) = BCLIN_V(:,:,3)

end subroutine atm_step


!******************************************************************************
! (08) subroutine update_r_tot
!      *notice that the gas constant has a different value in each box
!      *should be called each time Q is calculated
!      *see equations 4.2.9 and 4.1.4 in Atmospheric Convection by Emanuel
!******************************************************************************
subroutine update_r_tot
  R_TOT(:,:) = RD + Q(:,:,2) * ( RV - RD )
!  R_TOT(:,:) = RD
end subroutine update_r_tot

!******************************************************************************
! (09) subroutine update_cp_tot
!      *should be called each time Q is calculated 
!      *see equations 4.2.5 and 4.1.4 in Atmospheric Convection by Emanuel
!******************************************************************************
subroutine update_cp_tot
!  CP_TOT(:,:) = CPD + Q(:,:,2) * ( CPV - CPD )
  CP_TOT(:,:) = CPD
end subroutine update_cp_tot

!******************************************************************************
! (10) subroutine update_t
!      *calculate temperature from dry static energy
!******************************************************************************

subroutine update_t
  integer K

  do K = 1, KMAX
     T(:,K) = ( DSE(:,K,2) - PHI(:,K) ) / CP_TOT(:,K)
  enddo

end subroutine update_t


!******************************************************************************
! (11) subroutine update_theta
!      *calculates potential temperature from temperature
!******************************************************************************
subroutine update_theta
  integer K
  do K = 1, KMAX
     THETA(:,K) = T(:,K) * ( P_EDGE(3) / P(K) ) ** ( R_TOT(:,K) / CP_TOT(:,K) )
  end do 

end subroutine update_theta


!******************************************************************************
! (12) subroutine update_alpha
!      *uses equation of state to calculate specific volume
!      *MAKE SURE TO CALL:
!           subroutine update_r_tot
!           subroutine update_cp_tot
!           subroutine update_t
!       FIRST!!!!     
!******************************************************************************
subroutine update_alpha
  integer K
  do K = 1, KMAX
     ALPHA(:,K) = R_TOT(:,K) * T(:,K) / P(K)
  end do
end subroutine update_alpha


!******************************************************************************
! (13) subroutine run_model
!******************************************************************************
subroutine run_model
  TIME = TSTART
!  call print_diags
  do while ( TIME .le. ( TEND - DT ) )
     call atm_step
     TIME = TIME + DT

     call print_diags

     if( CO2_VARY_BOOL ) then
        call co2_vary_exp
     elseif( CLD_VARY_BOOL ) then
        call cld_vary_exp
     elseif( OHT_VARY_BOOL ) then
        call oht_vary_exp 
     endif

     if ( abs ( TIME - int( TIME / ( 2.0d0 * TPRINT ) ) * 2.0d0 * TPRINT ) .lt. DT / 2.0d0 ) then
        write(6,'(A,1E10.3,A,1F6.3,A)') 'TIME = ', TIME,' seconds or ', TIME/YEAR,' years'
!        write(6,*) 'YEAR = ', TIME/YEAR
     endif

  end do
end subroutine run_model

!******************************************************************************
! (14) subroutine print_diags
!      -only outputting essential variables as of 2/23/06 (dsa)
!      -print-out will work as long as JMAX<=30
!******************************************************************************
subroutine print_diags
              
!now time ptint out is an option
  if ( abs ( TIME - int( TIME / TPRINT ) * TPRINT ) .lt. DT / 2.0d0 .and. TIME_WRITE ) then

     write(UNIT=DIAGS_FID,FMT='(400g20.12)') TIME/YEAR,DSE(:,:,2),Q(:,:,2),BTROP_U(:,2),  &
                                             BCLIN_U(:,:,2),BCLIN_V(:,:,2),FRAC_CLD(:,:), &
                                             TS(:,2),SST(:,2)
  end if

end subroutine print_diags


!******************************************************************************
! (14) subroutine print_diags
!******************************************************************************
subroutine print_diags_old
  integer  :: K,J
  real*8   :: PRECIP_PRINT(JMAX,KMAX),                   &
              OCN_EVAP_FLUX_PRINT(JMAX),                 &
              LAND_EVAP_FLUX_PRINT(JMAX),                &
              P_MINUS_E_PRINT(JMAX),                     &
              M_FLUX_Y(JMAX+1,KMAX),M_FLUX_Z(JMAX),      &
              RE_EVAP_PRINT(JMAX),AVG_HEAT_FLUX,         &
              AVG_SWMLW,PRECIP_G_PRINT(JMAX),            &
              FW_FLUX_PRINT(JMAX),AVG_FINOCN,ALB(JMAX),  &
              ALB_AVG,AVG_PME
              
!is there a better way to do this?
!this is set up to work for JMAX=3 right now only, so include the second part of this switch.  
!may want to make it work for other JMAX later, too busy now. (dsa, 12/20/05)
  if ( abs ( TIME - int( TIME / TPRINT ) * TPRINT ) .lt. DT / 2.0d0 .and. JMAX .eq. 3) then

   !average albedo of earth
     ALB(:)  = 1.d0 - ABSORB_SW(:) / SW_FLUX(:)
     ALB_AVG = sum( ALB(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
   
     do K=1,KMAX
        !convert from kg/s to m/yr each box would produce
        PRECIP_PRINT(:,K) = PRECIP(:,K) * YEAR / RHO_H2O / SURF_AREA(:)
   
        !meridional mass flux (10^9 kg/s)
        M_FLUX_Y(:,K)     = V_EDGE(:,K) * DX(:) * ( P_EDGE(K+1) - P_EDGE(K) ) / G / 1.0d9
   
     enddo 
   
     RE_EVAP_PRINT(:)  = RE_EVAP(:)  * YEAR / RHO_H2O / SURF_AREA(:)
     PRECIP_G_PRINT(:) = PRECIP_G(:) * YEAR / RHO_H2O / SURF_AREA(:)
     FW_FLUX_PRINT(:)  = FW_FLUX(:)  * YEAR / RHO_H2O / SURF_AREA(:)
     !vertical mass flux (10^9 kg/s)
     M_FLUX_Z(:) = - OMEGA(:) * SURF_AREA(:) / G / 1.0d9
   
   
     !convert from ((kg H20 vap) m^-2 s^-1)) to m/yr
     OCN_EVAP_FLUX_PRINT(:)  = OCN_EVAP_FLUX(:)  * YEAR / RHO_H2O
     LAND_EVAP_FLUX_PRINT(:) = LAND_EVAP_FLUX(:) * YEAR / RHO_H2O
   
   
     !make sure to take into account re-evaporated precipitation
     P_MINUS_E_PRINT(:)      = PRECIP_G_PRINT(:)                                - &
                               ( 1.0d0 - FRAC_LAND(:) ) * OCN_EVAP_FLUX_PRINT(:) - &
                               FRAC_LAND(:) * LAND_EVAP_FLUX_PRINT(:)
   
     !area weighted average precip. minus evap.
     AVG_PME = sum( P_MINUS_E_PRINT(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
   
     !area weighted average heat flux
     AVG_HEAT_FLUX = sum( HEATING(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
   
     !area weighted average incoming SW minus outgoing LW
     AVG_SWMLW = sum( ( ABSORB_SW(:) - OLR(:) ) * SURF_AREA(:), 1 ) / sum( SURF_AREA(:), 1 )
   
     !area weighted average flux into the ocean
     !notice that this is averaged over the area of the whole box.  
     !This is so it can be compared to the average SW-OLR
     AVG_FINOCN = sum( FLUX_IN_OCN(:) * ( 1.0d0 - FRAC_LAND(:) ) * SURF_AREA(:), 1) / &
                  sum( SURF_AREA(:), 1 )


     write(DIAGS_FID,'(100g20.12)')     TIME / YEAR,                  &
                                        THETA(:,:),Q(:,:,2),          &
                                        M_FLUX_Y(:,:),M_FLUX_Z(:),    &
                                        PRECIP_PRINT(:,:),RH(:,:),    &
!                                        THETA_E(:,2),                 &
!                                        THETA_E_STAR(:,1),            &
                                        MSE(:,2),                     &
                                        MSE_STAR(:,1),                &
                                        OCN_SH_FLUX(:),               &
                                        LAND_SH_FLUX(:),              &
                                        OCN_EVAP_FLUX_PRINT(:),       &
                                        LAND_EVAP_FLUX_PRINT(:),      &
                                        TS(:,2)-273.15d0,             &
                                        P_MINUS_E_PRINT(:),           &
                                        EMISS(:,:), T(:,:),           &
                                        EDDY_HEAT(:,:)/1.0d15,        &
                                        EDDY_LATENT(:,:)/1.0d15,      &
                                        ADVECT_HEAT_Y(:,:)/1.0d15,    &
                                        ADVECT_HEAT_P(:,:)/1.0d15,    &
                                        ADVECT_LATENT_Y(:,:)/1.0d15,  &
                                        ADVECT_LATENT_P(:,:)/1.0d15,  &
                                        ABSORB_SW(:),OLR(:)

     write(DIAGS_FID2,'(83g20.12)')     FLUX_IN_OCN(:),HEATING(:),    &
                                        RE_EVAP_PRINT(:),             &
                                        EPS_RE_EVAP(:),               &
                                        EDDY_MOIST(:,:)/1.0d9,        &
                                        AVG_HEAT_FLUX,                &
                                        PRECIP_G_PRINT(:),            &
                                        AVG_SWMLW,FW_FLUX_PRINT(:),   &
                                        AVG_FINOCN,                   &
                                        SST(:,2)-273.15d0,            &
                                        ALB(:),ALB_AVG,AVG_PME,       &
                                        OHT(2:JMAX)/1.0d15,           &
                                        FRAC_CLD(:,:),                &
                                        EMISS_CLEAR(:,:),             &
                                        BTROP_U(:,2),BCLIN_U(:,:,2),  &
                                        U(:,:),ANG_MOM(:,:),          &
                                        BTROP_PHI(:)/G/1000.d0,       &
                                        BCLIN_PHI(:,:)/G/1000.d0,     &
                                        PHI(:,:)/G/1000.d0


!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! uncomment if you want to check tracer conservation by advection
! (remember to turn off all other tendencies)
!     call check_advect
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  end if

end subroutine print_diags_old

!******************************************************************************
! (15) subroutine update_atm_emissivity
!      *call to update atmospheric emissivities based on current CO2 and H20
!      *based on Sasamori (1968) and the PUMA model
!      *CO2 should be in vol/vol mixing ratio! (e.g. 350x10^-6)
!      *meaning of variables:
!        1. FAC_H20 - converts effective path length of water vapor 
!                     from kg m^-2 to gm cm^-2
!        2. FAC_CO2 - converts effective path length of CO2 
!                     from kg m^-2 to cm at STP (10^5 Pa, 273.15 K)
!        3. V2M     - (44.0 g/mol)/(28.8 g/mol) converts co2 mixing ratio 
!                     from vol/vol to kg/kg
!      *effective path length notes:
!        1. Given in eqn (3) in Sasamori (1968)
!        2. Factors above must be inserted to get units correct
!        3. mixing ratio is assumed to be constant within boxes, 
!           so the equation can be easily integrated
!******************************************************************************
subroutine update_atm_emissivity
  real*8, parameter  :: FAC_H2O = 0.1d0,     &
                        FAC_CO2 = 51.6043d0, &
                        V2M     = 1.5278d0
  integer            :: K,J
  real*8             :: UH2O(JMAX), UCO2(JMAX), EMISS_H2O(JMAX), &
                        EMISS_CO2(JMAX), TAU_H2O_15(JMAX)

  do K = 1, KMAX

     if( Q_EM ) then
        UH2O(:) = ( FAC_H2O * Q(:,K,2)  / ( 2.0d0 * P_EDGE(3) * G) ) * &
                  ( P_EDGE(K+1) ** 2.0d0 - P_EDGE(K) ** 2.0d0 )
     else
        UH2O(:) = ( FAC_H2O * QSTART(:,K)  / ( 2.0d0 * P_EDGE(3) * G) ) * &
                  ( P_EDGE(K+1) ** 2.0d0 - P_EDGE(K) ** 2.0d0 )
     endif

     UCO2(:) = ( FAC_CO2 * V2M * (/ (CO2,J=1,JMAX) /) / ( 2.0d0 * P_EDGE(3) * G) ) * &
               ( P_EDGE(K+1) ** 2.0d0 - P_EDGE(K) ** 2.0d0 )
     
     EMISS_H2O(:)     = emissivity_h2o( UH2O(:) )
     EMISS_CO2(:)     = emissivity_co2( UCO2(:) )
     TAU_H2O_15(:)    = transmissivity_h2o_15mu( UH2O(:) )

        
     !clear-sky emissivity
     !can choose which version I want to use here... 
     ! BUT REMEMBER TO CHANGE EMISS_CLEAR_REF2!!! (dsa, 10/08/05)
!     EMISS_CLEAR(:,K) = EMISS_H2O(:) + EMISS_CO2 * TAU_H2O_15(:) + EMISS_O3(:,K)
!     EMISS_CLEAR(:,K) = EMISS_H2O(:) + EMISS_CO2  + EMISS_O3(:,K)
     EMISS_CLEAR(:,K) = EMISS_H2O(:) + EMISS_CO2 * TAU_H2O_15(:) 
!     EMISS_CLEAR(:,K) = EMISS_H2O(:) + EMISS_CO2  
     
     !I'm cheating here!!!  want to get sensitivity to doubling CO2 higher!  (dsa,8/24/05)
     if ( RAD_CHEAT1 ) then
        EMISS_CLEAR(:,K) = RAD_CHEAT_FAC1 * ( EMISS_CLEAR(:,K) - EMISS_CLEAR_REF1(:,K) ) + EMISS_CLEAR_REF1(:,K)
     endif
     
     if ( RAD_CHEAT2 ) then
        EMISS_CLEAR(:,K) = RAD_CHEAT_FAC2 * ( EMISS_CLEAR(:,K) - EMISS_CLEAR_REF2(:,K) ) + EMISS_CLEAR_REF2(:,K)
     endif

     where ( EMISS_CLEAR(:,K) .gt. 1.0d0 )
        EMISS_CLEAR(:,K) = 1.0d0
     endwhere

     !this is for testing result of offsetting emissivities at each 
     !level by a constant amount (dsa, 8/26/05)
     if ( CONST_OFFSET ) then
        EMISS_CLEAR(:,K) = EMISS_CLEAR_REF1(:,K)
     endif

     !make correction for cloud emissivity
     EMISS(:,K)       = EMISS_CLEAR(:,K) + FRAC_CLD(:,K) * EMISS_CLD(K) - &
                        EMISS_CLEAR(:,K) * FRAC_CLD(:,K) * EMISS_CLD(K) 

     !this is for testing result of offsetting emissivities at each 
     !level by a constant amount (dsa, 8/26/05)
     if ( CONST_OFFSET ) then
        EMISS(:,K) = EMISS(:,K) + EM_OFFSET(K)
     endif



     if( ( TEND - TIME ) / DT .lt. 2.0d0 ) then 
   
        write(DEBUG_FID,*)'++++++++++++++++++++++++++++++++++++'
        write(DEBUG_FID,*)'++++++++++++++++++++++++++++++++++++'
        write(DEBUG_FID,*) ( TEND - TIME ) / DT
        write(DEBUG_FID,*)'------------------------------------'
        
          do J=1,JMAX
             write(UNIT=DEBUG_FID,FMT='(2I3,7f9.3)') J,K,UH2O(J),UCO2(J),EMISS_H2O(J),&
                  EMISS_CO2(J),TAU_H2O_15(J),EMISS_CLEAR(J,K),EMISS(J,K)
          enddo
   
     endif
     

  enddo

  !high cloud emissivity
  EMISS_H(:) = FRAC_H_CLD(:) * EMISS_H_CLD


  !just for playing with code, make sure to remove!!! (dsa, 8/22/05)
!  EMISS(:,:) = 1.0d0
  
end subroutine update_atm_emissivity


!******************************************************************************
! (16) function rad_tend_dse
!      *returns the dry static energy tendency due to radiation 
!       (SW and LW flux) in each box
!      *units of rad_tend_dse are J/kg/s
!      *assumes KMAX=2 - all these equations would have to be 
!       changed if KMAX is changed
!
!      *modified to return a dry static energy tendency (dsa, 10/7/05)
!******************************************************************************
function rad_tend_dse result ( TEND_RAD_DSE )
  !return 
  real*8       :: TEND_RAD_DSE(JMAX,KMAX)
  !internal          
  real*8       :: FLUX(JMAX,KMAX)

  !assume high clds are in radiative balance alone
  T_H_CLD(:) = ( ( F_LW_SURF_UP(:) * ( 1.0d0 - EMISS(:,2) ) * ( 1.0d0 - EMISS(:,1) )     + &
                 EMISS(:,2) * ( 1.0d0 - EMISS(:,1) ) * SIGMA * T(:,2) ** 4.0d0           + &
                 EMISS(:,1) * SIGMA * T(:,1) ** 4.0d0 ) / ( 2.0d0 * SIGMA ) ) ** 0.25d0

  !assumes dry adiabatic lapse rate and constant density within
  !layer
  !(dsa, 6/9/06)
!  T_H_CLD(:) = T(:,1) +  ALPHA(:,1) * ( P_EDGE(1) - P(1) ) / CPD


  F_LW_DOWN(:)     = EMISS(:,2) * SIGMA * T(:,2) ** 4.0d0                          + &
                     ( 1.0d0 - EMISS(:,2) ) * EMISS(:,1) * SIGMA * T(:,1) ** 4.0d0 + &
                     EMISS_H(:) * ( 1.0d0 - EMISS(:,2) ) * ( 1.0d0 - EMISS(:,1) )  * &
                     SIGMA * T_H_CLD(:) ** 4.0d0
  
  !includes longwave reflected back upward (dsa, 6/6/05)
  F_LW_SURF_UP(:)  = FRAC_LAND(:) * ( EMISS_LAND * SIGMA * TS(:,2) ** 4.0d0                  + &
                     ( 1.0d0 - EMISS_LAND ) * F_LW_DOWN(:) )                                + &
                     ( 1.0d0 - FRAC_LAND(:) ) * ( EMISS_OCN * SIGMA * SST(:,2) ** 4.0d0      + &
                     ( 1.0d0 - EMISS_OCN ) * F_LW_DOWN(:) )
                     


!calculate total flux due to radiation
!then convert flux to potential temperature tendency

  !lower level
  FLUX(:,2)         = EMISS(:,2) * ( F_LW_SURF_UP(:) + EMISS(:,1) * SIGMA * T(:,1) ** 4.0d0 + &
                      EMISS_H(:) * ( 1.0d0 - EMISS(:,1) ) * SIGMA * T_H_CLD(:) ** 4.0d0     - &
                      2.0d0 * SIGMA * T(:,2) ** 4.0d0 )

  TEND_RAD_DSE(:,2) = FLUX(:,2) * G / ( P_EDGE(3) - P_EDGE(2) )

!an artificial addition to play with effect of radiation
!  TEND_RAD(:,2) = 0.5d0 * TEND_RAD(:,2)

  !upper level
  FLUX(:,1)         = EMISS(:,1) * ( ( 1.0d0 - EMISS(:,2) ) * F_LW_SURF_UP(:)  + &
                      EMISS(:,2) * SIGMA * T(:,2) ** 4.0d0                     + &
                      EMISS_H(:) * SIGMA * T_H_CLD(:) ** 4.0d0                 - &
                      2.0d0 * SIGMA * T(:,1) ** 4.0d0 )

  TEND_RAD_DSE(:,1) = FLUX(:,1) * G / ( P_EDGE(2) - P_EDGE(1) )

  !now update diagnostics
  !outgoing longwave radiation
  OLR(:)            = EMISS_H(:) * SIGMA * T_H_CLD(:) ** 4.0d0                      + &
                      EMISS(:,1) * ( 1.0d0 - EMISS_H(:) ) * SIGMA * T(:,1) ** 4.0d0 + &
                      ( 1.0d0 - EMISS_H(:) ) * ( 1.0d0 - EMISS(:,1) )               * &
                      EMISS(:,2) * SIGMA * T(:,2) ** 4.0d0                          + &
                      ( 1.0d0 - EMISS_H(:) ) * ( 1.0d0 - EMISS(:,1) )               * &
                      ( 1.0d0 - EMISS(:,2) ) * F_LW_SURF_UP(:)
 
  F_SW_OCN_DOWN(:)  = SW_FLUX(:) * ( 1.0d0 - ALPHA_OCN(:) ) * ( 1.0d0 - ALPHA_ATM ) * &
                      ( 1.0d0 - FRAC_CLD(:,1) * ALPHA_CLD(1) )                      * &
                      ( 1.0d0 - FRAC_CLD(:,2) * ALPHA_CLD(2) )                      * &
                      ( 1.0d0 - FRAC_H_CLD(:) * ALPHA_H_CLD )

  F_SW_LAND_DOWN(:) = SW_FLUX(:) * ( 1.0d0 - ALPHA_LAND(:) ) * ( 1.0d0 - ALPHA_ATM ) * &
                      ( 1.0d0 - FRAC_CLD(:,1) * ALPHA_CLD(1) )                       * &    
                      ( 1.0d0 - FRAC_CLD(:,2) * ALPHA_CLD(2) )                       * &
                      ( 1.0d0 - FRAC_H_CLD(:) * ALPHA_H_CLD )


  ABSORB_SW(:)      = ( 1.0d0 - FRAC_LAND(:) ) * F_SW_OCN_DOWN(:) + &
                      FRAC_LAND(:) * F_SW_LAND_DOWN(:)


end function rad_tend_dse


!******************************************************************************
! (17) function baroclinic_tend_dse
!      *meant to parameterize the dry static energy tendency due to
!       heat transport by baroclinic eddies
!      *returns a tendency of theta in units J/kg/s
!      *tune later by assuming meridional heat transport is roughly 2-5x10^15 J/s
!       in the mid-latitudes
!      *neglect w-theta correlation (assume convection takes care of this term)
!
!      *modified to return a dry static energy tendency (dsa, 10/8/05)
!******************************************************************************
function baroclinic_tend_dse result ( BCLIN_TEND_DSE )
  !return 
  real*8    :: BCLIN_TEND_DSE(JMAX,KMAX)
  !internal 
  real*8    :: F_Y0(JMAX+1),F_Y(JMAX+1),D_DSE_DY0(JMAX-1),D_DSE_DY(JMAX-1)
  integer   :: K

  BCLIN_TEND_DSE(:,:)    = 0.0d0
  D_DSE_DY0(:)           = 0.0d0
  D_DSE_DY(:)            = 0.0d0
  F_Y0(:)                = 0.0d0
  F_Y(:)                 = 0.0d0
  !use lower level DSE, see notes (dsa, 5/27/05) 
  D_DSE_DY0(:)           = derivative( DSE(:,2,2), Y )
  !like a diffusion coefficient
  F_Y0(2:JMAX)           = - KY_BCLIN_TH * ( abs( D_DSE_DY0(:) ) ** N_BCLIN )  

  do K = 1, KMAX

     !derivative at level of interest
     D_DSE_DY(:)         = derivative( DSE(:,K,2), Y )

     !add friction effect and diffuse
     F_Y(2:JMAX)         = BCLIN_FRIC(K) * F_Y0(2:JMAX) * D_DSE_DY(:)

     !since represents correlation treat like advection instead of like other fluxes
     BCLIN_TEND_DSE(:,K) = ( F_Y(1:JMAX  ) * cos( LAT_EDGE_RADS(1:JMAX  ) )   - &
                            F_Y(2:JMAX+1) * cos( LAT_EDGE_RADS(2:JMAX+1) ) ) / &
                          R / ( sin( LAT_EDGE_RADS(2:JMAX+1) )               - &
                               sin( LAT_EDGE_RADS(1:JMAX  ) ) )


     EDDY_HEAT(:,K)      = F_Y(:) * DX(:) * ( P_EDGE(K+1) - P_EDGE(K) ) / G

  end do
  
!  BCLIN_TEND_DSE(:,:)    = 0.0d0
end function baroclinic_tend_dse


!******************************************************************************
! (18) function baroclinic_tend_Q
!      *eli said to tune by assuming mid-latitude transport is roughly 0.5 SV 
!       i think this means 5.0e8 kg vap/s
!******************************************************************************
function baroclinic_tend_q result ( BCLIN_TEND_Q )
  !return 
  real*8    :: BCLIN_TEND_Q(JMAX,KMAX)
  !internal 
  real*8    :: F_Y0(JMAX+1),F_Y(JMAX+1),D_DSE_DY0(JMAX-1),DQ_DY(JMAX-1)
  integer   :: K

  BCLIN_TEND_Q(:,:) = 0.0d0
  D_DSE_DY0(:)      = 0.0d0
  DQ_DY(:)          = 0.0d0
  F_Y0(:)           = 0.0d0
  F_Y(:)            = 0.0d0
  !use lower level DSE, see notes (dsa, 5/27/05)
  D_DSE_DY0(:)      = derivative( DSE(:,2,2), Y )
  !like a diffusion coefficient
  F_Y0(2:JMAX)      = - KY_BCLIN_Q * ( abs( D_DSE_DY0(:) ) ** N_BCLIN ) 

  do K = 1, KMAX

     !derivative at level of interest
     DQ_DY(:)          = derivative( Q(:,K,2), Y )

     !add friction effect and diffuse
     F_Y(2:JMAX)       = BCLIN_FRIC(K) * F_Y0(2:JMAX) * DQ_DY(:)


     !playing with a slight variation on the eddy param. (dsa, 11/14/05)
!     where( DQ_DY(:) * D_DSE_DY0(:) .lt. 0.0d0 )
!        F_Y(2:JMAX) = 0.0d0
!     endwhere


     !since represents correlation treat like advection instead of like other fluxes
     BCLIN_TEND_Q(:,K)  = ( F_Y(1:JMAX  ) * cos( LAT_EDGE_RADS(1:JMAX  ) )   - &
                            F_Y(2:JMAX+1) * cos( LAT_EDGE_RADS(2:JMAX+1) ) ) / &
                          R / ( sin( LAT_EDGE_RADS(2:JMAX+1) )               - &
                                sin( LAT_EDGE_RADS(1:JMAX  ) ) )
     
     EDDY_MOIST(:,K)    = F_Y(:) * DX(:) * ( P_EDGE(K+1) - P_EDGE(K) ) / G

     !using dp/dz=-rho*g (dsa, 5/30/05)
     !uses upwind scheme (dsa, 5/31/05)
!     where( F_Y(2:JMAX) .ge. 0.0d0 )
!        EDDY_LATENT(:,K) = LV(1:JMAX-1,K) * F_Y(2:JMAX) * DX(2:JMAX) * ( P_EDGE(K+1) - P_EDGE(K) ) / G
!     elsewhere
!        EDDY_LATENT(:,K) = LV(2:JMAX,K)   * F_Y(2:JMAX) * DX(2:JMAX) * ( P_EDGE(K+1) - P_EDGE(K) ) / G
!     endwhere
     
  end do

  !using constant LV now (dsa, 1/5/06)
  EDDY_LATENT(:,:) = LV0 * EDDY_MOIST(:,:)

!  BCLIN_TEND_Q(:,:) = 0.0d0
end function baroclinic_tend_q

!******************************************************************************
! (19) function derivative
!      *takes the discretized derivative of ARR1 w.r.t ARR2
!      *ARR1 and ARR2 must be 1D
!      *D1_D2 has dimension one less than ARR1 and ARR2
!      *tested (4/11/05) appears to fcn properly, does not deal with error 
!       cases like equal consecutive elements of ARR2
!******************************************************************************
function derivative( ARR1, ARR2 ) result ( D1_D2 )
  !inputs
  real*8, dimension(:), intent(in)  :: ARR1, ARR2
  !outputs
  real*8, dimension(size(ARR1,1)-1) :: D1_D2
  !internal
  real*8                            :: TMP1(size(ARR1,1)),TMP2(size(ARR1,1)),TMP3(size(ARR1,1))
  
  TMP1  = cshift( ARR1, SHIFT = 1 ) - ARR1
  TMP2  = cshift( ARR2, SHIFT = 1 ) - ARR2

  TMP3  = TMP1 / TMP2

  D1_D2 = TMP3(1:(size(ARR1,1)-1))

end function derivative



!******************************************************************************
! (20) subroutine check_advect
!      *checks tracer consevation during advection
!      *turn all tendencies but advection off (in subroutine atm_step) and
!       call a number of times during a run to make sure tracer is conserved
!      *if tracer is conserved THETA_SUM and Q_SUM will be same each call
!******************************************************************************
subroutine check_advect

  real*8 DSE_SUM, Q_SUM

  DSE_SUM = 0.0d0
  Q_SUM   = 0.0d0

  DSE_SUM = sum ( DSE(:,:,2) * MASS(:,:) )
  Q_SUM   = sum ( Q(:,:,2)   * MASS(:,:) )
  
  write(DEBUG_FID,*) 'TIME    = ', TIME / ( 3600.0d0 * 24.0d0 * 365.25d0 )
  write(DEBUG_FID,*) 'DSE_SUM = ', DSE_SUM
  write(DEBUG_FID,*) 'Q_SUM   = ', Q_SUM
  write(DEBUG_FID,*) '***************************************'  

end subroutine check_advect


!******************************************************************************
! (21) subroutine update_air_surface_flux
!
!******************************************************************************
subroutine update_air_surface_flux
  integer                 :: I
  real*8                  :: SPEED(JMAX),E_SAT_SURF_OCN(JMAX),                        &
                             Q_SAT_SURF_OCN(JMAX),E_SAT_SURF_LAND(JMAX),              &
                             Q_SAT_SURF_LAND(JMAX),LV_SST(JMAX),LV_TS(JMAX),          &
                             PRECIP_FLUX(JMAX)
!  real*8, parameter       :: U_STAR = 0.0d0

  
  

  !calculate speed (velocity magnitude) in lower level
  !U_STAR accounts for gusts and zonal velocity
  !average meridional velocities on edges
  !neglect vertical velocity
!  SPEED(1:JMAX)      = sqrt( ( max( abs( V_EDGE(1:JMAX,2) ), abs( V_EDGE(2:JMAX+1,2) ) ) ) ** 2 + U_STAR ** 2 )

!for now just set surface speed to a constant
!rethink later if we want to include WISHE effects (dsa, 4/18/05)
  SPEED(1:JMAX)      = (/ (V_SURF, I=1,JMAX) /)

  !calculate sensible heat fluxes (J m^-2 s^-1)
!  OCN_SH_FLUX(:)     = CP_TOT(:,2) * SPEED(:) * C_SH_OCN  * ( SST(:,2) - THETA(:,2,2) ) / ALPHA(:,2)
!  LAND_SH_FLUX(:)    = CP_TOT(:,2) * SPEED(:) * C_SH_LAND * ( TS(:,2)  - THETA(:,2,2) ) / ALPHA(:,2)

  !unclear on which CP I should be using here (dsa,6/10/05)
  OCN_SH_FLUX(:)     = CPD * SPEED(:) * C_SH_OCN  * ( SST(:,2) - THETA(:,2) ) / ALPHA(:,2)
  LAND_SH_FLUX(:)    = CPD * SPEED(:) * C_SH_LAND * ( TS(:,2)  - THETA(:,2) ) / ALPHA(:,2)

  !calculate saturation vapor pressures
  E_SAT_SURF_OCN(:)  = calc_e_star( SST(:,2) )
  E_SAT_SURF_LAND(:) = calc_e_star( TS(:,2)  )

  !calculate saturation specific humidities
  Q_SAT_SURF_OCN(:)  = calc_q_from_e( P_EDGE(3), E_SAT_SURF_OCN(:)  )
  Q_SAT_SURF_LAND(:) = calc_q_from_e( P_EDGE(3), E_SAT_SURF_LAND(:) )
  
  !calculate specific humidity fluxes ((kg H20 vap) m^-2 s^-1))
  OCN_EVAP_FLUX(:)   = SPEED(:) * C_Q_OCN  * ( Q_SAT_SURF_OCN(:)  - Q(:,2,2) ) / ALPHA(:,2)
  LAND_EVAP_FLUX(:)  = EVAP_FRAC(:) * SPEED(:) * C_Q_LAND * ( Q_SAT_SURF_LAND(:) - Q(:,2,2) ) / ALPHA(:,2)


  PRECIP_G(:)        = sum(PRECIP(:,:),2) - RE_EVAP(:)

  !flux precip into surface (kg H20 m^-2 s^-1) (same for ocean and atm)
  PRECIP_FLUX(:)     = PRECIP_G(:) / SURF_AREA(:)


  !assume land has no water storage capacity so :
  ! -land evaporation flux cannot exceed land precipitation flux 
  ! (dsa, 6/10/05)
  where( LAND_EVAP_FLUX(:) .gt. PRECIP_FLUX(:) )
     LAND_EVAP_FLUX(:) = PRECIP_FLUX(:)
  endwhere
  

  RUNOFF(:)          = FRAC_LAND(:) * SURF_AREA(:) * ( PRECIP_FLUX(:) - LAND_EVAP_FLUX(:) )

!  if( ( TEND - TIME ) / DT .lt. 10.0d0 ) then
!
!     write(DEBUG_FID2,*)'++++++++++++++++++++++++++++++++++++'
!     write(DEBUG_FID2,*)'++++++++++++++++++++++++++++++++++++'
!     write(DEBUG_FID2,*) ( TEND - TIME ) / DT
!     write(DEBUG_FID2,*)'------------------------------------'
!
!     write(DEBUG_FID2,*) 'PRECIP_FLUX(:)'
!     write(DEBUG_FID2,*) PRECIP_FLUX(:)
!     write(DEBUG_FID2,*) 'LAND_EVAP_FLUX(:)'
!     write(DEBUG_FID2,*) LAND_EVAP_FLUX(:)
!     write(DEBUG_FID2,*) 'RUNOFF(:)'
!     write(DEBUG_FID2,*) RUNOFF(:)
!
!  endif

  FW_FLUX(:)         = ( 1.0d0 - FRAC_LAND(:) ) * PRECIP_G(:) + RUNOFF(:)
  

  !calculate latent heat based on SST
!  LV_SST(:)          = LV0 + ( CPV - CL ) * ( SST(:) - 273.16d0 )
!  LV_TS(:)           = LV0 + ( CPV - CL ) * ( LAND_TEMP(:) - 273.16d0 )

  LV_SST(:)          = (/ (LV0, I=1,JMAX) /)
  LV_TS(:)           = (/ (LV0, I=1,JMAX) /)

  FLUX_IN_OCN(:)     = F_SW_OCN_DOWN(:) + EMISS_OCN * ( F_LW_DOWN(:) - SIGMA * SST(:,2) ** 4.0d0 ) - &
                       OCN_SH_FLUX(:) - LV_SST(:) * OCN_EVAP_FLUX(:) 

  !total heat flux into atmosphere minus heat flux out at each latitude (dsa, 6/3/05)
  HEATING(:) = FRAC_LAND(:) * ( LV_TS(:) * LAND_EVAP_FLUX(:) + LAND_SH_FLUX(:)            + &
               EMISS_LAND * ( SIGMA * TS(:,2) ** 4.0d0 - F_LW_DOWN(:) ) )                    + &
               ( 1.0d0 - FRAC_LAND(:) ) * ( LV_SST(:) * OCN_EVAP_FLUX(:) + OCN_SH_FLUX(:) + &
               EMISS_OCN * ( SIGMA * SST(:,2) ** 4.0d0 - F_LW_DOWN(:) ) ) - OLR(:)

end subroutine update_air_surface_flux

!******************************************************************************
! (22) subroutine investigate_steady_state
!      *should be called after model has reached steady-state to investigate 
!       balances that lead to steady-state
!******************************************************************************
subroutine investigate_steady_state
  integer                      :: J,K
  real*8, dimension(JMAX,KMAX) :: AD_DSE,AD_Q,RAD_DSE,BCLIN_DSE,BCLIN_Q,EVAP_Q,SENS_DSE, &
                                  PRECIP_DSE,PRECIP_Q,CONVECT_DSE,CONVECT_Q,RE_EVAP_DSE, &
                                  RE_EVAP_Q,SUM_Q,SUM_DSE

  real*8, dimension(JMAX)      :: SUM_TS,SUM_SST

  AD_DSE      = advection_tend_dse()
  AD_Q        = advection_tend_q()

  RAD_DSE      = rad_tend_dse()
  
  SENS_DSE     = sens_heat_tend_dse()
  EVAP_Q       = evaporation_tend_q()

!  BCLIN_Q      = baroclinic_tend_q()
!  BCLIN_DSE    = baroclinic_tend_dse()
  
  BCLIN_Q      = get_eddy_tend( Q(:,:,2),   2 )
  BCLIN_DSE    = get_eddy_tend( DSE(:,:,2), 1 )

  PRECIP_DSE   = precipitation_tend_dse()
  PRECIP_Q     = precipitation_tend_q()

  RE_EVAP_DSE  = re_evaporation_tend_dse()
  RE_EVAP_Q    = re_evaporation_tend_q()

  CONVECT_DSE  = convection_tend_dse()
  CONVECT_Q    = convection_tend_q()

  SUM_DSE(:,:) = AD_DSE+RAD_DSE+SENS_DSE+BCLIN_DSE+PRECIP_DSE+RE_EVAP_DSE+CONVECT_DSE
  SUM_Q(:,:)   = AD_Q+EVAP_Q+BCLIN_Q+PRECIP_Q+RE_EVAP_Q+CONVECT_Q


  write(UNIT=SS_BAL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_BAL_FID,FMT='(A)') 'DSE Tendencies (K/s)'
  write(UNIT=SS_BAL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_BAL_FID,FMT='(2A3,9A12)') 'J','K','Advection','Eddies',        &
                                          'Sens. Heat','Precip.','Radiation.', &
                                          'Convect.','Re-evap.','Con.+Re.',    &
                                          'SUM'
  do K=1,KMAX
     do J=1,JMAX
        

        write(UNIT=SS_BAL_FID,FMT='(2I3,9E12.3)') J,                                 &
                                                  K,                                 &
                                                  AD_DSE(J,K),                       &
                                                  BCLIN_DSE(J,K),                    &
                                                  SENS_DSE(J,K),                     &
                                                  PRECIP_DSE(J,K),                   &
                                                  RAD_DSE(J,K),                      &
                                                  CONVECT_DSE(J,K),                  &
                                                  RE_EVAP_DSE(J,K),                  &
                                                  RE_EVAP_DSE(J,K)+CONVECT_DSE(J,K), &
                                                  SUM_DSE(J,K)
        

        write(UNIT=SS_BAL_PLOT_FID,FMT='(9E12.3)') AD_DSE(J,K),                       &
                                                   BCLIN_DSE(J,K),                    &
                                                   SENS_DSE(J,K),                     &
                                                   PRECIP_DSE(J,K),                   &
                                                   RAD_DSE(J,K),                      &
                                                   CONVECT_DSE(J,K),                  &
                                                   RE_EVAP_DSE(J,K),                  &
                                                   RE_EVAP_DSE(J,K)+CONVECT_DSE(J,K), &
                                                   SUM_DSE(J,K)
     enddo
  enddo
  


  write(UNIT=SS_BAL_FID,FMT='(A)') ''
  write(UNIT=SS_BAL_FID,FMT='(A)') ''
  write(UNIT=SS_BAL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_BAL_FID,FMT='(A)') 'Q Tendencies ((kg vap/kg air)/s)'
  write(UNIT=SS_BAL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_BAL_FID,FMT='(2A3,8A12)') 'J','K','Advection','Eddies',           &
                                          'Evap','Precip.','Convect.','Re-evap.', &
                                          'Con.+Re.','SUM'
  do K=1,KMAX
     do J=1,JMAX

        write(UNIT=SS_BAL_FID,FMT='(2I3,8E12.3)') J,                             &
                                                  K,                             &
                                                  AD_Q(J,K),                     &
                                                  BCLIN_Q(J,K),                  &
                                                  EVAP_Q(J,K),                   &  
                                                  PRECIP_Q(J,K),                 &
                                                  CONVECT_Q(J,K),                &
                                                  RE_EVAP_Q(J,K),                &
                                                  CONVECT_Q(J,K)+RE_EVAP_Q(J,K), &
                                                  SUM_Q(J,K)


        write(UNIT=SS_BAL_PLOT_FID,FMT='(9E12.3)') AD_Q(J,K),                     &
                                                   BCLIN_Q(J,K),                  &
                                                   EVAP_Q(J,K),                   &  
                                                   PRECIP_Q(J,K),                 &
                                                   CONVECT_Q(J,K),                &
                                                   RE_EVAP_Q(J,K),                &
                                                   CONVECT_Q(J,K)+RE_EVAP_Q(J,K), &
                                                   SUM_Q(J,K),                    &
                                                   DUMMY

     enddo
  enddo
  

  SUM_SST(:)  = ( OHT(1:JMAX) - OHT(2:JMAX+1) )             / &
                ( ( 1.0d0 - FRAC_LAND(:) ) * SURF_AREA(:) )  + &
                F_SW_OCN_DOWN(:) + EMISS_OCN * F_LW_DOWN(:) - &
                EMISS_OCN * SIGMA *  SST(:,2)**4.0d0            - &
                OCN_SH_FLUX(:) - LV0 * OCN_EVAP_FLUX(:)

  write(UNIT=SS_BAL_FID,FMT='(A)') ''
  write(UNIT=SS_BAL_FID,FMT='(A)') ''

  write(UNIT=SS_BAL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_BAL_FID,FMT='(A)') 'Forcings on SST (W/m^2)'
  write(UNIT=SS_BAL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_BAL_FID,FMT='(1A3,7A9)') 'J','OHT','SW','LW DOWN','LW UP','SH','LH','SUM'
  do J=1,JMAX
     write(UNIT=SS_BAL_FID,FMT='(1I3,7f9.2)') J,                                                     &
                                              (OHT(J)-OHT(J+1))/((1.0d0-FRAC_LAND(J))*SURF_AREA(J)), &
                                              F_SW_OCN_DOWN(J),                                      &
                                              EMISS_OCN*F_LW_DOWN(J),                                &
                                              -EMISS_OCN*SIGMA*SST(J,2)**4.0d0,                          &
                                              -OCN_SH_FLUX(J),                                       &
                                              -LV0*OCN_EVAP_FLUX(J),                                 &
                                              SUM_SST(J)


     write(UNIT=SS_BAL_PLOT_FID,FMT='(9E12.3)') (OHT(J)-OHT(J+1))/((1.0d0-FRAC_LAND(J))*SURF_AREA(J)), &
                                                F_SW_OCN_DOWN(J),                                     &
                                                EMISS_OCN*F_LW_DOWN(J),                               &
                                                -EMISS_OCN*SIGMA*SST(J,2)**4.0d0,                         &
                                                -OCN_SH_FLUX(J),                                      &
                                                -LV0*OCN_EVAP_FLUX(J),                                &
                                                SUM_SST(J),                                           &
                                                DUMMY,                                                &
                                                DUMMY
  enddo



  SUM_TS(:) = F_SW_LAND_DOWN(:) + EMISS_LAND * F_LW_DOWN(:) - &
              EMISS_LAND * SIGMA * TS(:,2) **4.0d0              - &
              LAND_SH_FLUX(:) - LV0 * LAND_EVAP_FLUX(:)

  write(UNIT=SS_BAL_FID,FMT='(A)') ''
  write(UNIT=SS_BAL_FID,FMT='(A)') ''

  write(UNIT=SS_BAL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_BAL_FID,FMT='(A)') 'Forcings on TS (W/m^2)'
  write(UNIT=SS_BAL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_BAL_FID,FMT='(1A3,7A9)') 'J','SW','LW DOWN','LW UP','SH','LH','SUM'
  do J=1,JMAX
     write(UNIT=SS_BAL_FID,FMT='(1I3,6f9.2)') J,                                       &
                                              F_SW_LAND_DOWN(J),                       &
                                              EMISS_LAND * F_LW_DOWN(J),               &
                                              - EMISS_LAND * SIGMA * TS(J,2) **4.0d0,      &
                                              -LAND_SH_FLUX(J),                        &
                                              -LV0 * LAND_EVAP_FLUX(J),                &
                                              SUM_TS(J)


     write(UNIT=SS_BAL_PLOT_FID,FMT='(9E12.3)') F_SW_LAND_DOWN(J),                  &
                                                EMISS_LAND * F_LW_DOWN(J),          &
                                                - EMISS_LAND * SIGMA * TS(J,2) **4.0d0, &
                                                -LAND_SH_FLUX(J),                   &
                                                -LV0 * LAND_EVAP_FLUX(J),           &
                                                SUM_TS(J),                          &
                                                DUMMY,                              &
                                                DUMMY,                              &
                                                DUMMY

  enddo


end subroutine investigate_steady_state

!******************************************************************************
! (23) function calc_e_star
!      *calculates water vapor saturation pressure 
!      *based on eqn (4.4.13) in Atmospheric Convection by Emanuel
!      *takes a 1d array of temperature (could be just one element)
!      *TEMP in K, E_STAR returned in PA
!******************************************************************************
function calc_e_star( TEMP ) result ( E_STAR )
  !inputs
  real*8, dimension(:), intent(in)  :: TEMP
  !outputs
  real*8, dimension(size(TEMP,1))   :: E_STAR

  E_STAR(:) = exp( 53.67957d0 - 6743.769d0 / TEMP(:) - 4.8451d0 * log( TEMP(:) ) )

  !convert to Pa
  E_STAR(:) = E_STAR(:) * 100.0d0

end function calc_e_star

!******************************************************************************
! (24) function calc_q_from_e
!      *calculate specific humidity from vapor pressure
!      *based on eqn (4.1.4) in Atmospheric Convection by Emanuel
!******************************************************************************
function calc_q_from_e( PRES, E  ) result ( Q )
  !inputs
  real*8                            :: PRES
  real*8, dimension(:), intent(in)  :: E
  !outputs
  real*8, dimension(size(E,1))      :: Q

  Q(:) = EPSILON * E(:) / ( PRES + ( EPSILON - 1 ) * E(:) )

end function calc_q_from_e


!******************************************************************************
! (25) function calc_e_from_q
!      *calculate vapor pressure from specific humidity 
!      *based on eqn (4.1.4) in Atmospheric Convection by Emanuel
!******************************************************************************
function calc_e_from_q( PRES, Q  ) result ( E )
  !inputs
  real*8                            :: PRES
  real*8, dimension(:), intent(in)  :: Q
  !outputs
  real*8, dimension(size(Q,1))      :: E

  E(:) = PRES * Q(:) / ( EPSILON + ( 1 - EPSILON ) * Q(:) )

end function calc_e_from_q

!******************************************************************************
! (26) function evaporation_tend_q
!      *calculates the specific humidity tendency in lower boxes due to 
!       evaporation flux from the surface
!******************************************************************************
function evaporation_tend_q result ( EVAP_TEND_Q )
  !return
  real*8    ::  EVAP_TEND_Q(JMAX,KMAX)
  
  !initialize
   EVAP_TEND_Q(:,:) = 0.0d0

  !only evaporation into lower boxes, leave upper box tendencies at zero
  EVAP_TEND_Q(:,2) =  G * ( LAND_EVAP_FLUX(:) * FRAC_LAND(:) +         & 
                            OCN_EVAP_FLUX(:) * ( 1 - FRAC_LAND(:) ) )  &
                        / ( P_EDGE(3) - P_EDGE(2) )
 
end function evaporation_tend_q    


!******************************************************************************
! (27) function sens_heat_tend_dse
!      *calculates the dry static energy tendency in lower boxes due to 
!       sensible heat flux from the surface
!
!      *modified to return a dry static energy tendency (dsa, 10/7/05)
!******************************************************************************
function sens_heat_tend_dse result ( SH_TEND_DSE )
  !return
  real*8    ::  SH_TEND_DSE(JMAX,KMAX)

  !initialize
  SH_TEND_DSE(:,:) = 0.0d0
  
  !only sh flux into lower boxes, leave upper box tendencies at zero
  SH_TEND_DSE(:,2) =  ( LAND_SH_FLUX(:) * FRAC_LAND(:) + OCN_SH_FLUX(:) * ( 1 - FRAC_LAND(:) ) ) &
                      * G / ( P_EDGE(3) - P_EDGE(2) )
 
end function sens_heat_tend_dse
                                                                                    

!******************************************************************************
! (28) subroutine debug_routine
!      *use for debugging purposes, constantly re-write this one as appropriate
!******************************************************************************
subroutine debug_routine
!  real*8, dimension(JMAX)   :: BTROP_OF_BCLIN_U
!  real*8, dimension(JMAX+1) :: BTROP_OF_BCLIN_V
!
!  BTROP_OF_BCLIN_U(:) = btrop_compute( BCLIN_U(:,:,2) )
!  BTROP_OF_BCLIN_V(:) = btrop_compute( BCLIN_V(:,:,2) )
!  
!  write(DEBUG_FID,'(30g20.12)') BTROP_U(:,2),BCLIN_U(:,:,2),BCLIN_V(:,:,2), &
!                                BTROP_OF_BCLIN_U(:),BTROP_OF_BCLIN_V(:),    &
!                                ANG_MOM(:,:)

end subroutine debug_routine

!******************************************************************************
! (29) subroutine get_precip
!      *finds the precipitation in each box
!      *should be called every time step
!******************************************************************************
subroutine get_precip
  integer                            :: K,I
  real*8                             :: E_MAX(JMAX,KMAX),Q_MAX(JMAX,KMAX)
  real*8, parameter, dimension(KMAX) :: Q_SCALE = (/ 0.002d0, 0.02d0 /)
  

  E_MAX(:,:)  = 0.0d0
  Q_MAX(:,:)  = 0.0d0

  do K=1,KMAX
     E_MAX(:,K)  = CRIT_RH(K) * E_STAR(:,K)
     Q_MAX(:,K)  = EPSILON * E_MAX(:,K) / ( P(K) + ( EPSILON - 1 ) * E_MAX(:,K) ) 

     where( Q(:,K,2) .gt. Q_MAX(:,K) )
        !piece-wise continuous only
!        PRECIP(:,K) = SURF_AREA(:) * ( Q(:,K,2) - Q_MAX(:,K) ) * ( P_EDGE(K+1) - P_EDGE(K) ) / G  &
!                      / ( PRECIP_T_FACTOR * DT )

        PRECIP(:,K) = ( Q(:,K,2) - Q_MAX(:,K) ) * MASS(:,K) / ( PRECIP_T_SCALE )

        !piece-wise continuous and differentiable
!        PRECIP(:,K) = ( SURF_AREA(:) * ( Q(:,K,2) - Q_MAX(:,K) ) * ( P_EDGE(K+1) - P_EDGE(K) ) / G ) * &
!                      ( 1.0d0 - exp( - ( Q(:,K,2) - Q_MAX(:,K) ) / ( PRECIP_SCALE * Q_SCALE(K) ) ) )   &
!                      / ( PRECIP_T_FACTOR * DT )
     elsewhere
        PRECIP(:,K) = 0.0d0
     endwhere
  enddo



end subroutine get_precip


!******************************************************************************
! (30) function precipitation_tend_dse
!      *calculates the dry static evergy tendency due to precipitation
!
!      *modified to return a dry static energy tendency (dsa, 10/7/05)
!******************************************************************************
function precipitation_tend_dse result ( PRECIP_TEND_DSE )
  !return
  real*8    :: PRECIP_TEND_DSE(JMAX,KMAX)
  integer   :: K

  !initialize
  PRECIP_TEND_DSE(:,:) = 0.0d0
  
  do K=1,KMAX    
     PRECIP_TEND_DSE(:,K) = LV(:,K) * PRECIP(:,K) / MASS(:,K) 
  enddo

end function  precipitation_tend_dse

!******************************************************************************
! (31) function precipitation_tend_q
!      *calculates the specific humidity tendency due to precipitation
!******************************************************************************
function precipitation_tend_q result ( PRECIP_TEND_Q )
  !return
  real*8    :: PRECIP_TEND_Q(JMAX,KMAX)
  integer   :: K

  !initialize
  PRECIP_TEND_Q(:,:) = 0.0d0
  
  do K=1,KMAX
!     PRECIP_TEND_Q(:,K) =  - G * PRECIP(:,K) / SURF_AREA(:) / ( P_EDGE(K+1) - P_EDGE(K) )
     PRECIP_TEND_Q(:,K) =  - PRECIP(:,K) / MASS(:,K)
  enddo 

end function  precipitation_tend_q

!******************************************************************************
! (32) subroutine update_all_vars
!      *calls all subroutines which update diagnostic variables
!      *should be placed in subroutineatm_step after the time step is taken
!       and at the end of subroutine initialize_vars
!******************************************************************************
subroutine update_all_vars

!the calling order is very important! 
  call update_r_tot
  call update_cp_tot
  call update_t
  call update_theta
  call update_alpha
  call update_moisture
  call update_re_evap
  call update_lv
!  call update_theta_e
  call update_mse
  if( DIAG_CLDS ) then
     call update_clds
  endif

!the following make use of the above:

!  call calc_velocities
  call update_velocities
  call update_phi
!uses specific humidities
  call update_atm_emissivity
  call get_precip
!!uses emissivities, temperatures, cld fractions
!!  call update_land_temp
!uses precipitation, velocity, land_temp
  call update_air_surface_flux

end subroutine update_all_vars



!******************************************************************************
! (33) subroutine update_moisture
!      *updates the following
!         1. mixing ratio 
!         2. vapor pressure
!         3. saturation vapor pressure 
!         4. saturation specific humidity
!         5. saturation mixing ratio
!         6. relative humidity
!      *should be called each time Q is calculated 
!      *see section 4.1 in Atmospheric Convection by Emanuel
!******************************************************************************
subroutine update_moisture
  integer  :: K

  R_MIX(:,:) = Q(:,:,2) / ( 1 - Q(:,:,2) )

  do K=1,KMAX
     E(:,K)          = calc_e_from_q( P(K), Q(:,K,2) ) 
     E_STAR(:,K)     = calc_e_star( T(:,K) )
     Q_STAR(:,K)     = calc_q_from_e( P(K), E_STAR(:,K) ) 
     R_MIX_STAR(:,K) = Q_STAR(:,K) / ( 1.0d0 - Q_STAR(:,K) )
  enddo
  
  RH(:,:) = E(:,:) / E_STAR(:,:)

end subroutine update_moisture


!******************************************************************************
! (34) subroutine update_theta_e
!      *calculates the equivalent potential temperature and saturation 
!       equivalent potential temperatre in each box
!      *based on eqn (4.5.11) in Atmospheric Convection by Emanuel
!      *note the following:
!          1. neglect condensed water here, i.e. take r_t=r 
!          2. use surface pressure for reference pressure
!          3. use total pressure instead of dry air pressure
!      *should be updated after LV, R_MIX, T
!******************************************************************************
subroutine update_theta_e
  integer   :: K
  real*8    :: C1(JMAX,KMAX), C2(JMAX,KMAX) 

  !here mixing ratio of water vapor and liquid water should be used (dsa, 5/13/05)
  C1(:,:) = CPD + R_MIX(:,:) * CL
  C2(:,:) = CPD + R_MIX_STAR(:,:) * CL

  do K = 1, KMAX
     THETA_E(:,K)      = T(:,K) * ( P_EDGE(3) / P(K) ) ** ( RD / C1(:,K) )      * &
                         RH(:,K) ** ( - R_MIX(:,K) * RV / C1(:,K) )             * &
                         exp( LV(:,K) * R_MIX(:,K) / ( C1(:,K) * T(:,K) ) )

     THETA_E_STAR(:,K) = T(:,K) * ( P_EDGE(3) / P(K) ) ** ( RD / C2(:,K) )      * &
                         exp( LV(:,K) * R_MIX_STAR(:,K) / ( C2(:,K) * T(:,K) ) )
  end do

end subroutine update_theta_e



!******************************************************************************
! (35) subroutine update_lv
!      *should be called each time T is calculated 
!      *see equation (4.4.4) in Atmospheric Convection by Emanuel
!      *273.16 K is the temperature at the triple point of water
!      *should be updated after temperature
!******************************************************************************
subroutine update_lv
!  LV(:,:) = LV0 + ( CPV - CL ) * ( T(:,:) - 273.16d0 )
  LV(:,:) = LV0
end subroutine update_lv



!!******************************************************************************
!! (36) function convection_tend_dse
!!      *calculates the dry static energy tendency due to convection
!!      *convection is parameterized as strong mixing when b.l. entropy
!!       exceeds free tropospheric moist entropy
!!
!!      *modified to return a dry static energy tendency (dsa, 10/7/05)
!!******************************************************************************
!function convection_tend_dse result ( CONVECT_TEND_DSE )
!  !return
!  real*8            :: CONVECT_TEND_DSE(JMAX,KMAX)
!  real*8            :: K_CON(JMAX)
!  real*8, parameter :: THETA_E_SCALE = 1.0d0
!
!
!  !initialize
!  CONVECT_TEND_DSE(:,:) = 0.0d0
!  
!  !smoothly switch convection on
!  K_CON(:) = K_CON_MIN + 0.5d0 * ( K_CON_MAX - K_CON_MIN ) *    &
!             ( 1 + TANH( ( THETA_E(:,2) - THETA_E_STAR(:,1) ) / &
!             ( THETA_E_SCALE * EPS_CON ) ) )
!
!!  CONVECT_TEND_TH(:,1) = K_CON(:) * ( THETA(:,2,2) - THETA(:,1,2) )
!  CONVECT_TEND_DSE(:,2) = K_CON(:) * ( DSE(:,1,2) - DSE(:,2,2) )
!
!  !conserve energy
!  CONVECT_TEND_DSE(:,1) = - CONVECT_TEND_DSE(:,2) * ( P_EDGE(3) - P_EDGE(2) ) / ( P_EDGE(2) - P_EDGE(1) ) 
!
!end function  convection_tend_dse
!
!
!!******************************************************************************
!! (37) function convection_tend_q
!!      *calculates the q tendency due to convection
!!      *convection is parameterized as strong mixing when b.l. entropy
!!       exceeds free tropospheric moist entropy
!!******************************************************************************
!function convection_tend_q result ( CONVECT_TEND_Q )
!  !return
!  real*8            :: CONVECT_TEND_Q(JMAX,KMAX)
!  real*8            :: K_CON(JMAX)
!  real*8, parameter :: THETA_E_SCALE = 1.0d0
!
!
!  !initialize
!  CONVECT_TEND_Q(:,:) = 0.0d0
!  
!  !smoothly switch convection on
!  K_CON(:) = K_CON_MIN + 0.5d0 * ( K_CON_MAX - K_CON_MIN ) *    &
!             ( 1 + TANH( ( THETA_E(:,2) - THETA_E_STAR(:,1) ) / &
!             ( THETA_E_SCALE * EPS_CON ) ) )
!
!!  CONVECT_TEND_Q(:,1) = K_CON(:) * ( Q(:,2,2) - Q(:,1,2) )
!  CONVECT_TEND_Q(:,2) = K_CON(:) * ( Q(:,1,2) - Q(:,2,2) )
!
!  !conserve q
!  CONVECT_TEND_Q(:,1)= - CONVECT_TEND_Q(:,2) * ( P_EDGE(3) - P_EDGE(2) ) / ( P_EDGE(2) - P_EDGE(1) ) 
!end function convection_tend_q


!******************************************************************************
! (36) function convection_tend_dse
!      *calculates the dry static energy tendency due to convection
!      *convection is parameterized as strong mixing when b.l. entropy
!       exceeds free tropospheric moist entropy
!
!      *modified to return a dry static energy tendency (dsa, 10/7/05)
!******************************************************************************
function convection_tend_dse result ( CONVECT_TEND_DSE )
  !return
  real*8            :: CONVECT_TEND_DSE(JMAX,KMAX)

  !initialize
  CONVECT_TEND_DSE(:,:) = 0.0d0
  
  CONVECT_TEND_DSE(:,2) = K_CON(:) * ( DSE(:,1,2) - DSE(:,2,2) )

  !conserve energy
  CONVECT_TEND_DSE(:,1) = - CONVECT_TEND_DSE(:,2) * ( P_EDGE(3) - P_EDGE(2) ) / ( P_EDGE(2) - P_EDGE(1) ) 

end function  convection_tend_dse


!******************************************************************************
! (37) function convection_tend_q
!      *calculates the q tendency due to convection
!      *convection is parameterized as strong mixing when b.l. entropy
!       exceeds free tropospheric moist entropy
!******************************************************************************
function convection_tend_q result ( CONVECT_TEND_Q )
  !return
  real*8            :: CONVECT_TEND_Q(JMAX,KMAX)

  !initialize
  CONVECT_TEND_Q(:,:) = 0.0d0
  
  CONVECT_TEND_Q(:,2) = K_CON(:) * ( Q(:,1,2) - Q(:,2,2) )

  !conserve q
  CONVECT_TEND_Q(:,1)= - CONVECT_TEND_Q(:,2) * ( P_EDGE(3) - P_EDGE(2) ) / ( P_EDGE(2) - P_EDGE(1) ) 

end function convection_tend_q



!******************************************************************************
! (38) function emissivity_h2o
!      *Sasamori (1968) eqns (14) and (15)
!      *U is the effective path length in gm cm^-2
!******************************************************************************
function emissivity_h2o( U ) result ( EMISS_H2O )
!inputs
  real*8, dimension(:), intent(in) :: U
!outputs
  real*8, dimension(size(U,1))     :: EMISS_H2O

  if( RAD_CHEAT3 ) then
     !don't worry about U < 0.01 here -> this doesn't normally happen (dsa, 10/15/05)   
     EMISS_H2O(:) = A_H2O * log10( U(:) + C_H2O ) + B_H2O
     !in case new param. puts emiss below 0 at low path lengths (dsa, 10/15/05)
     where( EMISS_H2O(:) .lt. 0.0d0 )
        EMISS_H2O(:) = 0.0d0
     ENDWHERE
     !in case new param. puts emiss above 1 at low high lengths (dsa, 10/15/05)
     where( EMISS_H2O(:) .gt. 1.0d0 )
        EMISS_H2O(:) = 1.0d0
     ENDWHERE
  else
     where( U(:) .lt. 0.01d0 )
        EMISS_H2O(:) = 0.846d0 * ( U(:) + 3.59d-5 ) ** ( 0.243d0 ) - 6.9d-2
     elsewhere
        EMISS_H2O(:) = 0.24d0 * log10( U(:) + 0.01d0 ) + 0.622d0
     end where
  endif
end function emissivity_h2o

!******************************************************************************
! (39) function emissivity_co2
!      *Sasamori (1968) eqns (17) and (18)
!      *U is the effective path length in cm at STP
!******************************************************************************
function emissivity_co2( U ) result ( EMISS_CO2 )
!inputs
  real*8, dimension(:), intent(in) :: U
!outputs
  real*8, dimension(size(U,1))     :: EMISS_CO2

  if( RAD_CHEAT3 ) then    
     !don't worry about U < 1 here -> this doesn't normally happen (dsa, 10/15/05)   
     EMISS_CO2(:) = A_CO2 * log10( U(:) ) + B_CO2
     !in case new param. puts emiss below 0 at low path lengths (dsa, 10/15/05)
     where( EMISS_CO2(:) .lt. 0.0d0 )
        EMISS_CO2(:) = 0.0d0
     ENDWHERE
  else
     where( U(:) .lt. 1.0d0 )
        EMISS_CO2(:) = 0.0676d0 * ( U(:) + 0.01022d0 ) ** ( 0.421d0 ) - 0.00982d0
     elsewhere
        EMISS_CO2(:) = 0.0546d0 * log10( U(:) ) + 0.0581d0
     end where
  endif

end function emissivity_co2


!******************************************************************************
! (40) function transmissivity_h2o_15mu
!      *Sasamori (1968) eqn (21) 
!      *U is the effective path length in gm cm^-2
!      *transmissivity of water vapor in 15 micron band
!      *this is to account for overlap between water vapor and co2 
!       absorption in this band
!******************************************************************************
function transmissivity_h2o_15mu( U ) result ( TRANS )
!inputs
  real*8, dimension(:), intent(in) :: U
!outputs
  real*8, dimension(size(U,1))     :: TRANS

  TRANS(:) = 1.33d0 - 0.832d0 * ( U(:) + 0.0286d0 ) ** ( 0.260d0 )

!a test to see the effect of removing overlap parameterization
!  TRANS(:) = 1.0d0

end function transmissivity_h2o_15mu



!******************************************************************************
! (41) function advect_y
!      *take an array (theta,q,rho,etc.) and returns v*array at each interface
!      *upwind differencing scheme
!******************************************************************************
function advect_y( ARR ) result ( ADV )
  !inputs
  real*8,  intent(in) :: ARR(JMAX,KMAX)
  !return
  real*8              :: ADV(JMAX+1,KMAX)
  !internal
  integer             :: J,K

  !initialize
  ADV(:,:) = 0.0d0

  where( V_EDGE(2:JMAX,:) .gt. 0.0d0 )
     ADV(2:JMAX,:) = V_EDGE(2:JMAX,:) * ARR(1:JMAX-1,:)
  elsewhere
     ADV(2:JMAX,:) = V_EDGE(2:JMAX,:) * ARR(2:JMAX,:)
  endwhere

end function advect_y

!******************************************************************************
! (42) function advect_p
!      *takes an array (theta,q,rho,etc.) and returns omega*array 
!       at each interface
!      *have to be careful with direction!
!      *upwind differencing scheme
!******************************************************************************
function advect_p( ARR ) result ( ADV )
  !inputs
  real*8,  intent(in) :: ARR(JMAX,KMAX)
  !return
  real*8              :: ADV(JMAX,KMAX+1)
  !internal
  integer             :: J,K

  !initialize
  ADV(:,:) = 0.0d0

!assume two vertical layers!
  where( OMEGA(:) .gt. 0.0d0 )
     !descent
     ADV(:,2) = OMEGA(:) * ARR(:,1)
  elsewhere
     !ascent
     ADV(:,2) = OMEGA(:) * ARR(:,2)
  endwhere

end function advect_p



!******************************************************************************
! (43) function advection_tend_dse
!      *gives the dry static energy tendency in each box due to advection
!      *updates the advective print-out variables
!
!      *modified to return a dry static energy tendency (dsa, 10/8/05)
!******************************************************************************
function advection_tend_dse result ( ADV_TEND_DSE )
  !return 
  real*8    :: ADV_TEND_DSE(JMAX,KMAX)
  !internal 
  integer   :: J,K
  real*8    :: ADV_Y_DSE(JMAX+1,KMAX),ADV_P_DSE(JMAX,KMAX+1)

  ADV_TEND_DSE(:,:)     = 0.0d0

  !horizontal advection
  ADV_Y_DSE(:,:)        = advect_y( DSE(:,:,2) )
  !vertical advection
  ADV_P_DSE(:,:)        = advect_p( DSE(:,:,2) )

  do J = 1, JMAX
     ADV_TEND_DSE(J,:)  = ( ADV_Y_DSE(J  ,:) * cos( LAT_EDGE_RADS(J  ) )          - &
                            ADV_Y_DSE(J+1,:) * cos( LAT_EDGE_RADS(J+1) ) ) / R    / &
                          ( sin( LAT_EDGE_RADS(J+1) ) - sin( LAT_EDGE_RADS(J) ) ) 
  enddo

 
  do K = 1, KMAX
     ADV_TEND_DSE(:,K)  = (ADV_TEND_DSE(:,K) + ( ADV_P_DSE(:,K) -  ADV_P_DSE(:,K+1) ) &
                          / ( P_EDGE(K+1) - P_EDGE(K) ) ) 
     
     ADVECT_HEAT_Y(:,K) = ADV_Y_DSE(:,K) * DX(:) * ( P_EDGE(K+1) - P_EDGE(K) ) / G 
  enddo 

!  ADV_TEND_DSE(:,:)     = 0.0d0
end function advection_tend_dse



!******************************************************************************
! (44) function advection_tend_q
!      *gives the q tendency in each box due to advection
!      *updates the advective print-out variables
!******************************************************************************
function advection_tend_q result ( ADV_TEND_Q )
  !return 
  real*8    :: ADV_TEND_Q(JMAX,KMAX)
  !internal 
  integer   :: J,K
  real*8    :: ADV_Y_Q(JMAX+1,KMAX),ADV_P_Q(JMAX,KMAX+1),   &
               ADV_Y_LE(JMAX+1,KMAX),ADV_P_LE(JMAX,KMAX+1), &
               LE(JMAX,KMAX)

  ADV_TEND_Q(:,:)         = 0.0d0
  ADVECT_LATENT_Y(:,:)    = 0.0d0
!  LE(:,:)                 = LV(:,:) * Q(:,:,2)

  !horizontal advection
  ADV_Y_Q(:,:)            = advect_y( Q(:,:,2) )
!  ADV_Y_LE(:,:)           = advect_y( LE(:,:) )
  !vertical advection
  ADV_P_Q(:,:)            = advect_p( Q(:,:,2) )
!  ADV_P_LE(:,:)           = advect_p( LE(:,:) )

  do J = 1, JMAX
     ADV_TEND_Q(J,:)      = ( ADV_Y_Q(J  ,:) * cos( LAT_EDGE_RADS(J  ) )        - &
                              ADV_Y_Q(J+1,:) * cos( LAT_EDGE_RADS(J+1) ) ) / R  / &
                            ( sin( LAT_EDGE_RADS(J+1) ) - sin( LAT_EDGE_RADS(J) ) ) 

     ADVECT_LATENT_P(J,:) = - LV0 * ADV_P_Q(J,2:KMAX) * SURF_AREA(J) / G
  enddo

  do K = 1, KMAX
     ADV_TEND_Q(:,K)      = ADV_TEND_Q(:,K) + ( ADV_P_Q(:,K) -  ADV_P_Q(:,K+1) ) &
                            / ( P_EDGE(K+1) - P_EDGE(K) )

!     ADVECT_LATENT_Y(:,K) = ADV_Y_LE(:,K) * DX(:) * ( P_EDGE(K+1) - P_EDGE(K) ) / G

     ADV_MOIST(:,K)       = ADV_Y_Q(:,K) * DX(:) * ( P_EDGE(K+1) - P_EDGE(K) ) / G

     ADVECT_LATENT_Y(:,K) = LV0 * ADV_MOIST(:,K)
  enddo 

!  ADV_TEND_Q(:,:) = 0.0d0

end function advection_tend_q


!******************************************************************************
! (45) function re_evap_tend_dse
!      *calculates the dry static energy tendency due to re-evaporation of 
!       precipitation in lower layer
!
!      *modified to return a dry static energy tendency (dsa, 10/7/05)
!******************************************************************************
function re_evaporation_tend_dse result ( RE_EVAP_TEND_DSE )
  !return
  real*8    :: RE_EVAP_TEND_DSE(JMAX,KMAX)

  !initialize
  RE_EVAP_TEND_DSE(:,:) = 0.0d0


  !only re-evaporation into lower layer
  !assuming 2 vertical layers here!

  !only do re-evaporation if convection is switched on
!  where( K_CON(:) .eq. K_CON_MAX )
!     RE_EVAP_TEND_DSE(:,2) = - LV(:,2) * RE_EVAP(:) / MASS(:,2) 
!  endwhere
  RE_EVAP_TEND_DSE(:,2) = - LV(:,2) * RE_EVAP(:) / MASS(:,2) 

end function  re_evaporation_tend_dse


!******************************************************************************
! (46) function re_evap_tend_q
!      *calculates the specific humidity tendency due to re-evaporation of
!       precipitation in the lower layer
!******************************************************************************
function re_evaporation_tend_q result ( RE_EVAP_TEND_Q )
  !return
  real*8    :: RE_EVAP_TEND_Q(JMAX,KMAX)

  !initialize
  RE_EVAP_TEND_Q(:,:) = 0.0d0


  !only re-evaporation into lower layer
  !assuming 2 vertical layers here!
!  RE_EVAP_TEND_Q(:,2) = G * RE_EVAP(:) / SURF_AREA(:) / ( P_EDGE(3) - P_EDGE(2) )

  !only do re-evaporation if convection is switched on
!  where( K_CON(:) .eq. K_CON_MAX )
!     RE_EVAP_TEND_Q(:,2) = RE_EVAP(:) / MASS(:,2)
!  endwhere

  RE_EVAP_TEND_Q(:,2) = RE_EVAP(:) / MASS(:,2)

end function  re_evaporation_tend_q


!******************************************************************************
! (47) subroutine update_re_evap
!      *should be called each time RH is calculated 
!      *rate of precipitation from upper layer re-evaporated into 
!       lower layer
!      *this isn't a crazy way to deal with this, but its not exactly 
!       physical either.  Should be viewed as a tuning method.
!      *assumes two vertical layers!
!******************************************************************************
subroutine update_re_evap

! only do re-evaporation where convection is occuring (dsa, 10/27/05)
  EPS_RE_EVAP(:) = EPS_RE_EVAP0 * ( 1.0d0 - RH(:,2) ) * &
                   ( K_CON(:) - K_CON_MIN ) / ( K_CON_MAX - K_CON_MIN )
  where( EPS_RE_EVAP(:) .gt. 1.0d0 )
     EPS_RE_EVAP(:) = 1.0d0
  endwhere
!  RE_EVAP(:)     = EPS_RE_EVAP(:) * PRECIP(:,1) 
  RE_EVAP(:)     = EPS_RE_EVAP(:) * PRECIP(:,1)  
end subroutine update_re_evap


!******************************************************************************
! (48) function calc_tend_ts
!      *calculates the surface temperature tendency 
!******************************************************************************
function calc_tend_ts result ( TEND_TS )
  !return
  real*8, dimension(JMAX) :: TEND_TS,SUM_TS
  integer                 :: J

  !initialize
  TEND_TS(:)        = 0.0d0


  TEND_TS(:)        = ( F_SW_LAND_DOWN(:) + EMISS_LAND * F_LW_DOWN(:)                  - &
                        EMISS_LAND * SIGMA * TS(:,2) **4.0d0 - LAND_SH_FLUX(:)             - &
                        LV0 * LAND_EVAP_FLUX(:) ) / ( C_LAND * D_LAND )

!    if( ( TEND - TIME ) / DT .lt. 10.0d0 ) then
!
!  SUM_TS(:) = F_SW_LAND_DOWN(:) + EMISS_LAND * F_LW_DOWN(:) - &
!              EMISS_LAND * SIGMA * TS(:,2) **4.0d0          - &
!              LAND_SH_FLUX(:) - LV0 * LAND_EVAP_FLUX(:)
!
!
!  write(DEBUG_FID,*)'++++++++++++++++++++++++++++++++++++'
!  write(DEBUG_FID,*)'++++++++++++++++++++++++++++++++++++'
!  write(DEBUG_FID,*) ( TEND - TIME ) / DT
!  write(DEBUG_FID,*)'------------------------------------'
!  write(UNIT=DEBUG_FID,FMT='(1A3,7A9)') 'J','SW','LW DOWN','LW UP','SH','LH','SUM'
!
!  do J=1,JMAX
!     write(UNIT=DEBUG_FID,FMT='(1I3,6f9.2)') J,                                       &
!                                             F_SW_LAND_DOWN(J),                       &
!                                             EMISS_LAND * F_LW_DOWN(J),               &
!                                             -EMISS_LAND * SIGMA * TS(J,2) **4.0d0,   &
!                                             -LAND_SH_FLUX(J),                        &
!                                             -LV0 * LAND_EVAP_FLUX(J),                &
!                                             SUM_TS(J)
!  enddo
!  endif

end function calc_tend_ts


!******************************************************************************
! (49) function calc_tend_sst
!      *calculates the surface temperature tendency 
!******************************************************************************
function calc_tend_sst result ( TEND_SST )
  !return
  real*8        :: TEND_SST(JMAX)

!  TEND_SST(1:JMAX) = ( THC(1:JMAX) - THC(2:JMAX+1) + ( 1.0d0 - FRAC_LAND(1:JMAX) ) * &
!                       SURF_AREA(1:JMAX) * FLUX_IN_OCN(1:JMAX) )                  / &
!                     ( CL * (1.0d0 - FRAC_LAND(1:JMAX) ) * SURF_AREA(1:JMAX) * D_OCN )

  
!  TEND_SST(1:JMAX) = ( THC(1:JMAX) - THC(2:JMAX+1) + ( 1.0d0 - FRAC_LAND(1:JMAX) ) * &
!                       SURF_AREA(1:JMAX) * FLUX_IN_OCN(1:JMAX) ) / ( CL * RHO_H2O * &
!                       (1.0d0 - FRAC_LAND(1:JMAX) ) * SURF_AREA(1:JMAX) * D_OCN )

  TEND_SST(1:JMAX) = ( ( OHT(1:JMAX) - OHT(2:JMAX+1) )                    / &
                     ( ( 1.0d0 - FRAC_LAND(1:JMAX) ) * SURF_AREA(1:JMAX) ) + &
                     FLUX_IN_OCN(1:JMAX) ) / ( CL * RHO_H2O * D_OCN )

  
end function calc_tend_sst


!******************************************************************************
! (50) function btrop_compute
!      *computes the barotropic component of an array
!      *assumes two vertical layers
!      *takes array with variable elements in 1st direction, 
!       but should only have 2 elements in 2nd direction 
!       (any further elements in this direction will be ignored.
!******************************************************************************
function btrop_compute( ARR ) result ( BTROP )
  !inputs
  real*8, dimension(:,:), intent(in) :: ARR
  !return
  real*8, dimension(size(ARR,1))     :: BTROP

  BTROP(:) = ( ( P_EDGE(3) - P_EDGE(2) ) * ARR(:,2) + &
               ( P_EDGE(2) - P_EDGE(1) ) * ARR(:,1) ) &
             / ( P_EDGE(3) - P_EDGE(1) )

end function btrop_compute

!******************************************************************************
! (51) function btrop_compute1
!      *my annoying quick fix for an array that has only one 
!       element in the meridional direction
!******************************************************************************
function btrop_compute1( ARR ) result ( BTROP )
  !inputs
  real*8, dimension(KMAX), intent(in) :: ARR
  !return
  real*8                              :: BTROP

  BTROP = ( ( P_EDGE(3) - P_EDGE(2) ) * ARR(2) + &
            ( P_EDGE(2) - P_EDGE(1) ) * ARR(1) ) &
          / ( P_EDGE(3) - P_EDGE(1) )

end function btrop_compute1

!******************************************************************************
! (52) function bclin_compute
!      *computes the baroclinic component of an array
!      *assumes two vertical layers
!******************************************************************************
function bclin_compute( ARR ) result ( BCLIN )
  !inputs
  real*8, dimension(:,:), intent(in)         :: ARR
  !return
  real*8, dimension(size(ARR,1),size(ARR,2)) :: BCLIN

  BCLIN(:,1) = ( ( P_EDGE(3) - P_EDGE(2) ) /  ( P_EDGE(3) - P_EDGE(1) ) ) * ( ARR(:,1) - ARR(:,2) )
  BCLIN(:,2) = ( ( P_EDGE(2) - P_EDGE(1) ) /  ( P_EDGE(3) - P_EDGE(1) ) ) * ( ARR(:,2) - ARR(:,1) )

end function bclin_compute

!******************************************************************************
! (53) function bclin_compute1
!      *my annoying quick fix for an array that has only one 
!       element in the meridional direction
!******************************************************************************
function bclin_compute1( ARR ) result ( BCLIN )
  !inputs
  real*8, dimension(KMAX), intent(in)  :: ARR
  !return
  real*8, dimension(KMAX)              :: BCLIN

  BCLIN(1) = ( ( P_EDGE(3) - P_EDGE(2) ) /  ( P_EDGE(3) - P_EDGE(1) ) ) * ( ARR(1) - ARR(2) )
  BCLIN(2) = ( ( P_EDGE(2) - P_EDGE(1) ) /  ( P_EDGE(3) - P_EDGE(1) ) ) * ( ARR(2) - ARR(1) )

end function bclin_compute1

!******************************************************************************
! (54) function tendency_btrop_u
!      *computes the tendency of the barotropic zonal velocity
!******************************************************************************
function tendency_btrop_u result ( TEND_BTROP_U )
  !internal
  integer                        :: J,K
  real*8, dimension(JMAX+1,KMAX) :: UV
  real*8, dimension(JMAX+1)      :: BTROP_UV
  real*8, dimension(JMAX,KMAX)   :: R_FRIC_TMP, V_CENT, UV_CENT
  real*8, dimension(JMAX)        :: BTROP_RU, BTROP_UV_CENT,EDDY_TERM,DDU
  real*8                         :: BTROP_R_FRIC
  !return
  real*8, dimension(JMAX)        :: TEND_BTROP_U 


  !use upwind scheme
  UV              = advect_y(       U      )
  BTROP_UV        = btrop_compute(  UV     )

  V_CENT(:,:)     = center_interp( BCLIN_V(:,:,2) )
  UV_CENT(:,:)    = U(:,:) * V_CENT(:,:)

  BTROP_UV_CENT   = btrop_compute( UV_CENT )

  do J=1,JMAX
     R_FRIC_TMP(J,:)   = R_FRIC(:)
  enddo

  BTROP_RU(:) = btrop_compute( R_FRIC_TMP(:,:) * U(:,:) )

!  DO K=1,KMAX
!     U_MINUS(:,K) = BTROP_U(:,1) + BCLIN_U(:,K,1)
!  ENDDO

  EDDY_TERM(:) = btrop_compute( get_eddy_tend( U(:,:), 3 ) )
  

  !artificial term to get velocity scheme to converge
!  DDU(:) = vel_2der_zonal( BTROP_U(:,2) )
!try evaluating diffusion terms at time n-1 on Eli's advice (dsa, 2/10/06)
!  DDU(:) = vel_2der_zonal( BTROP_U(:,1) )

!  TEND_BTROP_U(:) = ( DX(1:JMAX)   * cos( LAT_EDGE_RADS(1:JMAX)   ) * BTROP_UV(1:JMAX)     - &
!                      DX(2:JMAX+1) * cos( LAT_EDGE_RADS(2:JMAX+1) ) * BTROP_UV(2:JMAX+1) ) / &
!                    SURF_AREA(:) + tan( LAT_RADS(:) ) * BTROP_UV_CENT(:) / R - BTROP_RU(:)

  TEND_BTROP_U(:) = ( DX(1:JMAX)   * cos( LAT_EDGE_RADS(1:JMAX)   ) * BTROP_UV(1:JMAX)     - &
                      DX(2:JMAX+1) * cos( LAT_EDGE_RADS(2:JMAX+1) ) * BTROP_UV(2:JMAX+1) ) / &
                    SURF_AREA(:) + tan( LAT_RADS(:) ) * BTROP_UV_CENT(:) / R - BTROP_RU(:) + &
                    EDDY_TERM(:)
!                    VEL_VISC * DDU(:)
  
  !for testing w/o non-linear terms
!  TEND_BTROP_U(:) = - BTROP_RU(:)

end function tendency_btrop_u

!******************************************************************************
! (55) function tendency_bclin_u
!      *computes the tendency of the baroclinic zonal velocity
!      *i've tried to code this in a relatively straight-forward way.  
!       see the notes for details on the formulation.
!******************************************************************************
function tendency_bclin_u result ( TEND_BCLIN_U )
  !internal
  integer                        :: J,K
  real*8, dimension(JMAX+1,KMAX) :: UV,BCLIN_UV
  real*8, dimension(JMAX)        :: DDU
  real*8, dimension(JMAX+1)      :: BTROP_UV
  real*8, dimension(JMAX,KMAX)   :: R_FRIC_TMP
  real*8, dimension(JMAX,KMAX)   :: BCLIN_RU,UV_CENT,BCLIN_UV_CENT,EDDY_TERM
  real*8, dimension(JMAX,KMAX+1) :: BCLIN_OMU
  real*8, dimension(JMAX,KMAX)   :: V_CENT
  !return
  real*8, dimension(JMAX,KMAX)   :: TEND_BCLIN_U 

  !use upwind scheme
  UV                   = advect_y(       U      )
  BTROP_UV             = btrop_compute(  UV     )

  !using the upwind scheme makes the following method of 
  !computing the baroclinic component of UV necessary
  do K=1,KMAX
     BCLIN_UV(:,K)     = UV(:,K) - BTROP_UV(:)
  enddo

  do J=1,JMAX
     R_FRIC_TMP(J,:)   = R_FRIC(:)
  enddo

  BCLIN_RU(:,:)        = bclin_compute( R_FRIC_TMP(:,:) * U(:,:) )

  BCLIN_OMU(:,:)       = advect_p( BCLIN_U )

  V_CENT(:,:)          = center_interp( BCLIN_V(:,:,2) )

  UV_CENT(:,:)         = U(:,:) * V_CENT(:,:)
  BCLIN_UV_CENT(:,:)   = bclin_compute( UV_CENT(:,:) )

 
!  DO K=1,KMAX
!     U_MINUS(:,K) = BTROP_U(:,1) + BCLIN_U(:,K,1)
!  ENDDO

  EDDY_TERM(:,:) = bclin_compute( get_eddy_tend( U(:,:), 3 ) )


  do K=1,KMAX
     
!     DDU(:) = vel_2der_zonal( BCLIN_U(:,K,2) )
!try evaluating diffusion terms at time n-1 on Eli's advice (dsa, 2/10/06)
!     DDU(:) = vel_2der_zonal( BCLIN_U(:,K,1) )

!     TEND_BCLIN_U(:,K) = ( DX(1:JMAX)   * cos( LAT_EDGE_RADS(1:JMAX)   ) * BCLIN_UV(1:JMAX,K)     - &
!                           DX(2:JMAX+1) * cos( LAT_EDGE_RADS(2:JMAX+1) ) * BCLIN_UV(2:JMAX+1,K) ) / &
!                           SURF_AREA(:) +                                                           &
!                         ( BCLIN_OMU(:,K) - BCLIN_OMU(:,K+1 ) ) / ( P_EDGE(K+1) - P_EDGE(K) ) +     &
!                         F(:) * V_CENT(:,K) + BCLIN_UV_CENT(:,K) * tan( LAT_RADS(:) ) / R         - &
!                         BCLIN_RU(:,K)

     TEND_BCLIN_U(:,K) = ( DX(1:JMAX)   * cos( LAT_EDGE_RADS(1:JMAX)   ) * BCLIN_UV(1:JMAX,K)     - &
                           DX(2:JMAX+1) * cos( LAT_EDGE_RADS(2:JMAX+1) ) * BCLIN_UV(2:JMAX+1,K) ) / &
                           SURF_AREA(:) +                                                           &
                         ( BCLIN_OMU(:,K) - BCLIN_OMU(:,K+1 ) ) / ( P_EDGE(K+1) - P_EDGE(K) ) +     &
                         F(:) * V_CENT(:,K) + BCLIN_UV_CENT(:,K) * tan( LAT_RADS(:) ) / R         - &
                         BCLIN_RU(:,K) + EDDY_TERM(:,K)
!                         BCLIN_RU(:,K) + VEL_VISC * DDU(:)

     !for testing w/o non-linear terms
!     TEND_BCLIN_U(:,K) = F(:) * V_CENT(:,K) - BCLIN_RU(:,K)

  enddo

end function tendency_bclin_u


!******************************************************************************
! (56) function tendency_bclin_v
!      *computes the tendency of the baroclinic meridional velocity
!      *set to zero at model edges, (Dirichlet boundary conditions)
!******************************************************************************
function tendency_bclin_v result ( TEND_BCLIN_V )
  !return
  real*8, dimension(JMAX+1,KMAX) :: TEND_BCLIN_V 
  !internal
  integer                        :: J,K
  real*8, dimension(JMAX+1)      :: DDV
  real*8, dimension(JMAX+1,KMAX) :: R_FRIC_TMP_EDGE,BCLIN_RV,EDDY_TERM
  real*8, dimension(JMAX,KMAX)   :: R_FRIC_TMP
  real*8, dimension(JMAX-1,KMAX) :: BCLIN_U_EDGE

  TEND_BCLIN_V(:,:)         = 0.0d0

  do J=1,JMAX
     R_FRIC_TMP(J,:)        = R_FRIC(:)
  enddo

  R_FRIC_TMP_EDGE(:,:)      = 0.0d0
  R_FRIC_TMP_EDGE(2:JMAX,:) = edge_interp( R_FRIC_TMP )

  BCLIN_RV(:,:)             = bclin_compute( R_FRIC_TMP_EDGE(:,:) * V_EDGE(:,:) )

  BCLIN_U_EDGE(:,:)         = edge_interp( BCLIN_U(:,:,2) )

!  EDDY_TERM = get_eddy_tend_v()

  do K=1,KMAX
!     TEND_BCLIN_V(2:JMAX,K) = - F_EDGE(2:JMAX) * BCLIN_U_EDGE(:,K)            - &
!                             ( BCLIN_PHI(2:JMAX,K) -  BCLIN_PHI(1:JMAX-1,K) ) / &
!                             ( LAT_RADS(2:JMAX) - LAT_RADS(1:JMAX-1) ) / R    - &
!                             BCLIN_RV(2:JMAX,K)

     DDV(:) = vel_2der_merid( BCLIN_V(:,K,2) )
!try evaluating diffusion terms at time n-1 on Eli's advice (dsa, 2/10/06)
!     DDV(:) = vel_2der_merid( BCLIN_V(:,K,1) )

     TEND_BCLIN_V(2:JMAX,K) = - F_EDGE(2:JMAX) * BCLIN_U_EDGE(:,K)            - &
                             ( BCLIN_PHI(2:JMAX,K) -  BCLIN_PHI(1:JMAX-1,K) ) / &
                             ( LAT_RADS(2:JMAX) - LAT_RADS(1:JMAX-1) ) / R    - &
                             BCLIN_RV(2:JMAX,K) + VEL_VISC * DDV(2:JMAX)
!                             BCLIN_RV(2:JMAX,K) + VEL_VISC * DDV(2:JMAX) + EDDY_TERM(2:JMAX,K)
  enddo

  !leave tendency at edges at zero

end function tendency_bclin_v

!******************************************************************************
! (57) function center_interp
!      *a way to interpolate variables defined on edges to centers
!      *calculated as the cross-sectional surface area weighted average 
!       of adjacent values
!      *this makes some sense for meridional velocity interpolation, 
!       which is what this function was originally written for
!******************************************************************************
function center_interp( EDGE_ARR ) result ( CENT_ARR )
!inputs
  real*8, dimension(JMAX+1,KMAX) :: EDGE_ARR
!internal
  integer                        :: K
!return 
  real*8, dimension(JMAX,KMAX)   :: CENT_ARR

  do K=1,KMAX
     CENT_ARR(:,K) = ( DX(1:JMAX)   * EDGE_ARR(1:JMAX,K)     + & 
                       DX(2:JMAX+1) * EDGE_ARR(2:JMAX+1,K) ) / &
                     ( DX(1:JMAX)   + DX(2:JMAX+1) )
  enddo

end function center_interp
!function center_interp( EDGE_ARR ) result ( CENT_ARR )
!!inputs
!  real*8, dimension(:,:), intent(in) :: EDGE_ARR
!!internal
!  integer                            :: K,JM,KM
!!return 
!  real*8, dimension(size(EDGE_ARR,1)-1,size(EDGE_ARR,2))   :: CENT_ARR
!
!  JM = size(EDGE_ARR,1)-1
!  KM = size(EDGE_ARR,2)
!
!  do K=1,KM
!     CENT_ARR(:,K) = ( DX(1:JM)   * EDGE_ARR(1:JM,K)     + & 
!                       DX(2:JM+1) * EDGE_ARR(2:JM+1,K) ) / &
!                     ( DX(1:JM)   + DX(2:JM+1) )
!  enddo
!
!end function center_interp

function center_interp1( EDGE_ARR ) result ( CENT_ARR )
!inputs
  real*8, dimension(JMAX+1) :: EDGE_ARR
  integer                        :: K
!return 
  real*8, dimension(JMAX)   :: CENT_ARR

  CENT_ARR(:) = ( DX(1:JMAX)   * EDGE_ARR(1:JMAX)     + & 
                  DX(2:JMAX+1) * EDGE_ARR(2:JMAX+1) ) / &
                ( DX(1:JMAX)   + DX(2:JMAX+1) )

end function center_interp1


!******************************************************************************
! (58) function edge_int
!      *a way to interpolate variables defined on box centers to box edges
!      *most important for zonal velocity
!      *calculated as the cross-sectional surface area weighted average 
!       of adjacent values (this makes some sense in context of zonal velocity)
!!******************************************************************************
function edge_interp( CENT_ARR ) result ( EDGE_ARR )
!inputs
  real*8, dimension(JMAX,KMAX)   :: CENT_ARR
!internal
  integer                        :: K
!return 
  real*8, dimension(JMAX-1,KMAX) :: EDGE_ARR

!doesn't matter if we use radians or degrees
  do K=1,KMAX
     EDGE_ARR(:,K) = ( ( LAT_EDGE(2:JMAX  ) - LAT_EDGE(1:JMAX-1) ) * CENT_ARR(1:JMAX-1,K) + &
                       ( LAT_EDGE(3:JMAX+1) - LAT_EDGE(2:JMAX  ) ) * CENT_ARR(2:JMAX,  K) ) &
                     / ( LAT_EDGE(3:JMAX+1) - LAT_EDGE(1:JMAX-1) )
  enddo

end function edge_interp

!function edge_interp( CENT_ARR ) result ( EDGE_ARR )
!!inputs
!  real*8, dimension(:,:), intent(in) :: CENT_ARR
!!internal
!  integer                   :: K,JM,KM
!!return 
!  real*8, dimension(size(CENT_ARR,1)-1,size(CENT_ARR,2)) :: EDGE_ARR
!
!  JM = size(CENT_ARR,1)
!  KM = size(CENT_ARR,2)
!
!!doesn't matter if we use radians or degrees
!  do K=1,KM
!     EDGE_ARR(:,K) = ( ( LAT_EDGE(2:JM  ) - LAT_EDGE(1:JM-1) ) * CENT_ARR(1:JM-1,K) + &
!                       ( LAT_EDGE(3:JM+1) - LAT_EDGE(2:JM  ) ) * CENT_ARR(2:JM,  K) ) &
!                     / ( LAT_EDGE(3:JM+1) - LAT_EDGE(1:JM-1) )
!  enddo
!
!end function edge_interp

!******************************************************************************
! (59) function edge_interp1
!      *if variable has only 1 vertical dimension
!******************************************************************************
function edge_interp1( CENT_ARR ) result ( EDGE_ARR )
!inputs
  real*8, dimension(JMAX)   :: CENT_ARR
!internal
  integer                   :: K
!return 
  real*8, dimension(JMAX-1) :: EDGE_ARR

!doesn't matter if we use radians or degrees
  EDGE_ARR(:) = ( ( LAT_EDGE(2:JMAX  ) - LAT_EDGE(1:JMAX-1) ) * CENT_ARR(1:JMAX-1) + &
                  ( LAT_EDGE(3:JMAX+1) - LAT_EDGE(2:JMAX  ) ) * CENT_ARR(2:JMAX  ) ) &
                / ( LAT_EDGE(3:JMAX+1) - LAT_EDGE(1:JMAX-1) )

end function edge_interp1

!******************************************************************************
! (60) subroutine update_phi
!      *updates barotropic, baroclinic, and total geopotential
!******************************************************************************
subroutine update_phi
  real*8, dimension(JMAX-1) :: BTROP_U_EDGE
  integer                   :: J,K


!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!baroclinic component of geopotential
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

  BCLIN_PHI(:,1) = ( P_EDGE(3) - P_EDGE(2) ) / ( P_EDGE(3) - P_EDGE(1) ) *                   &
                   ( ( P(2) - P_EDGE(2) ) * ALPHA(:,2) + ( P_EDGE(2) - P(1) ) * ALPHA(:,1) )
  BCLIN_PHI(:,2) = - ( P_EDGE(2) - P_EDGE(1) ) / ( P_EDGE(3) - P_EDGE(1) ) *                 &
                   ( ( P(2) - P_EDGE(2) ) * ALPHA(:,2) + ( P_EDGE(2) - P(1) ) * ALPHA(:,1) )

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!barotropic component of geopotential
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

  BTROP_U_EDGE    = edge_interp1( BTROP_U )
  
  !an arbitrary choice in this model 
!  BTROP_PHI(1)    = 5.0d4
  BTROP_PHI(1)    = BTROP_PHI0

  do J=2,JMAX
     BTROP_PHI(J) = BTROP_PHI(J-1) - R * F_EDGE(J) * ( LAT_RADS(J) - LAT_RADS(J-1) ) * BTROP_U_EDGE(J-1)
  enddo 

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!total geopotential
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

  do K=1,KMAX
     PHI(:,K) = BCLIN_PHI(:,K) + BTROP_PHI(:)
  enddo

end subroutine update_phi


!******************************************************************************
! (61) subroutine update_velocities
!******************************************************************************
subroutine update_velocities
  integer :: K
  
  do K=1,KMAX
     U(:,K)        = BTROP_U(:,2) + BCLIN_U(:,K,2)
     ANG_MOM(:,K)  = ( U(:,K) + OMEGA_ROT * R * cos( LAT_RADS(:) ) ) * R * cos( LAT_RADS(:) ) 
  enddo

  V_EDGE(2:JMAX,:) = BCLIN_V(2:JMAX,:,2)

!using continuity equation
  OMEGA(:)         = ( V_EDGE(2:JMAX+1,2) * cos( LAT_EDGE_RADS(2:JMAX+1) )             - &
                       V_EDGE(1:JMAX,2  ) * cos( LAT_EDGE_RADS(1:JMAX  ) ) )           * &
                     ( P_EDGE(3) - P_EDGE(2) ) / R                                     / & 
                     ( sin( LAT_EDGE_RADS(2:JMAX+1) ) - sin( LAT_EDGE_RADS(1:JMAX) ) )

!  U(:,:) = 0.0d0
!  ANG_MOM(:,:) = 0.0d0
!  V_EDGE(:,:) = 0.0d0
!  OMEGA(:) = 0.0d0
  
end subroutine update_velocities


!******************************************************************************
! (62) subroutine investigate_steady_v
!      *to see balance of terms in v equation at steady-state
!******************************************************************************
subroutine investigate_steady_v
  !internal
  integer                        :: J,K
  real*8, dimension(JMAX+1,KMAX) :: R_FRIC_TMP_EDGE,BCLIN_RV,TERM1_V,TERM2_V, &
                                    TERM3_V,TERM4_V,TERM5_V,SUM_V
  real*8, dimension(JMAX-1,KMAX) :: BCLIN_U_EDGE

  real*8, dimension(JMAX+1,KMAX) :: UV
  real*8, dimension(JMAX+1)      :: BTROP_UV
  real*8, dimension(JMAX+1)      :: JMAXp1_WRITE_ARR
  real*8, dimension(JMAX,KMAX)   :: R_FRIC_TMP,UV_CENT
  real*8, dimension(JMAX)        :: BTROP_RU,TERM1_BTROPU,TERM2_BTROPU,      &
                                    TERM3_BTROPU,TERM4_BTROPU,BTROP_UV_CENT, &
                                    JMAX_WRITE_ARR,SUM_BTROPU,TERM5_BTROPU
  real*8                         :: BTROP_R_FRIC,JMAX_WRITE_AVG

  real*8, dimension(JMAX+1,KMAX) :: BCLIN_UV
  real*8, dimension(JMAX,KMAX)   :: BCLIN_RU,BCLIN_UV_CENT
  real*8, dimension(JMAX,KMAX+1) :: BCLIN_OMU
  real*8, dimension(JMAX,KMAX)   :: V_CENT,TERM1_BCLINU,TERM2_BCLINU,TERM3_BCLINU, &
                                    TERM4_BCLINU,TERM5_BCLINU,TERM6_BCLINU,        &
                                    SUM_BCLINU,TERM7_BCLINU


!=========================================
!baroclinic v diagnostic
!=========================================

  do J=1,JMAX
     R_FRIC_TMP(J,:)        = R_FRIC(:)
  enddo

  R_FRIC_TMP_EDGE(:,:)      = 0.0d0
  R_FRIC_TMP_EDGE(2:JMAX,:) = edge_interp( R_FRIC_TMP )

  BCLIN_RV(:,:)             = bclin_compute( R_FRIC_TMP_EDGE(:,:) * V_EDGE(:,:) )

  BCLIN_U_EDGE(:,:)         = edge_interp( BCLIN_U(:,:,2) )


  TERM1_V(:,:) = 0.0d0
  TERM2_V(:,:) = 0.0d0
  TERM3_V(:,:) = 0.0d0
  TERM4_V(:,:) = 0.0d0
!  TERM5_V(:,:) = get_eddy_tend_v()
  TERM5_V(:,:) = 0.0d0

!calculate terms in the balance
  do K=1,KMAX
     TERM1_V(2:JMAX,K) = - F_EDGE(2:JMAX) * BCLIN_U_EDGE(:,K)
     TERM2_V(2:JMAX,K) = - ( BCLIN_PHI(2:JMAX,K) -  BCLIN_PHI(1:JMAX-1,K) ) / &
                           ( LAT_RADS(2:JMAX) - LAT_RADS(1:JMAX-1) ) / R 
     TERM3_V(2:JMAX,K) = - BCLIN_RV(2:JMAX,K)
     TERM4_V(:,K)      = VEL_VISC * vel_2der_merid( BCLIN_V(:,K,2) )
!try evaluating diffusion terms at time n-1 on Eli's advice (dsa, 2/10/06)
!     TERM4_V(:,K)      = VEL_VISC * vel_2der_merid( BCLIN_V(:,K,1) )
  enddo

  SUM_V(:,:) = TERM1_V(:,:) + TERM2_V(:,:) + TERM3_V(:,:) + TERM4_V(:,:) + TERM5_V(:,:)

  write(UNIT=SS_VEL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_VEL_FID,FMT='(A)') 'terms in bclin_v balance (m s^-2)'
  write(UNIT=SS_VEL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_VEL_FID,FMT='(1A4,1A3,6A12)') 'J','K','Coriolis','P. Grad.','Fric','Diff','Diff2','SUM'
  do K=1,KMAX
     do J=2,JMAX
 
        write(UNIT=SS_VEL_FID,FMT='(1F4.1,1I3,6E12.3)') real(J)-0.5,K,TERM1_V(J,K),TERM2_V(J,K), &
                                                        TERM3_V(J,K),TERM4_V(J,K),TERM5_V(J,K),SUM_V(J,K)
       
        write(UNIT=SS_VEL_PLOT_FID,FMT='(7E12.3)') TERM1_V(J,K),TERM2_V(J,K), &
                                                   TERM3_V(J,K),TERM4_V(J,K), &
                                                   SUM_V(J,K),DUMMY,DUMMY

     enddo
  enddo

!=========================================
!barotropic u diagnostic
!=========================================

  !use upwind scheme
  UV              = advect_y(       U      )
  BTROP_UV        = btrop_compute(  UV     )

  BTROP_RU(:) = btrop_compute( R_FRIC_TMP(:,:) * U(:,:) )

  V_CENT(:,:)     = center_interp( BCLIN_V(:,:,2) )
  UV_CENT(:,:)    = U(:,:) * V_CENT(:,:)

  BTROP_UV_CENT   = btrop_compute( UV_CENT )

  TERM1_BTROPU = ( DX(1:JMAX)   * cos( LAT_EDGE_RADS(1:JMAX)   ) * BTROP_UV(1:JMAX)     - &
                   DX(2:JMAX+1) * cos( LAT_EDGE_RADS(2:JMAX+1) ) * BTROP_UV(2:JMAX+1) ) / &
                   SURF_AREA(:)  
  TERM2_BTROPU = tan( LAT_RADS(:) ) * BTROP_UV_CENT(:) / R
  TERM3_BTROPU = -BTROP_RU
!  TERM4_BTROPU = VEL_VISC * vel_2der_zonal( BTROP_U(:,2) )
!try evaluating diffusion terms at time n-1 on Eli's advice (dsa, 2/10/06)
!  TERM4_BTROPU = VEL_VISC * vel_2der_zonal( BTROP_U(:,1) )
  TERM4_BTROPU = 0.0d0
  TERM5_BTROPU = btrop_compute( get_eddy_tend( U(:,:), 3 ) )

  SUM_BTROPU   = TERM1_BTROPU + TERM2_BTROPU + TERM3_BTROPU + TERM4_BTROPU + TERM5_BTROPU

  write(UNIT=SS_VEL_FID,FMT='(A)') ''
  write(UNIT=SS_VEL_FID,FMT='(A)') ''
  write(UNIT=SS_VEL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_VEL_FID,FMT='(A)') 'terms in btrop_u balance (m s^-2)'
  write(UNIT=SS_VEL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_VEL_FID,FMT='(1A3,6A12)') 'J','Adv.','Geo. Term.','Fric','Diff','Diff2','SUM'
  do J=1,JMAX

     write(UNIT=SS_VEL_FID,FMT='(1I3,6E12.3)') J,TERM1_BTROPU(J),TERM2_BTROPU(J), &
                                               TERM3_BTROPU(J),TERM4_BTROPU(J),   &
                                               TERM5_BTROPU(J),SUM_BTROPU(J)
    
     write(UNIT=SS_VEL_PLOT_FID,FMT='(7E12.3)') TERM1_BTROPU(J),TERM2_BTROPU(J), &
                                                TERM3_BTROPU(J),TERM4_BTROPU(J), &
                                                SUM_BTROPU(J),DUMMY,DUMMY

  enddo


!=========================================
!baroclinic u diagnostic
!=========================================

  !use upwind scheme
  UV                   = advect_y(       U      )
  BTROP_UV             = btrop_compute(  UV     )

  !using the upwind scheme makes the following method of 
  !computing the baroclinic component of UV necessary
  do K=1,KMAX
     BCLIN_UV(:,K)     = UV(:,K) - BTROP_UV(:)
  enddo

  BCLIN_RU(:,:)        = bclin_compute( R_FRIC_TMP(:,:) * U(:,:) )

  BCLIN_OMU(:,:)       = advect_p( BCLIN_U )

  V_CENT(:,:)          = center_interp( BCLIN_V(:,:,2) )
  BCLIN_UV_CENT(:,:)   = bclin_compute( UV_CENT(:,:) )

  TERM7_BCLINU(:,:) = bclin_compute( get_eddy_tend( U(:,:), 3 ) )

  do K=1,KMAX
     TERM1_BCLINU(:,K) = ( DX(1:JMAX)   * cos( LAT_EDGE_RADS(1:JMAX)   ) * BCLIN_UV(1:JMAX,K)     - &
                           DX(2:JMAX+1) * cos( LAT_EDGE_RADS(2:JMAX+1) ) * BCLIN_UV(2:JMAX+1,K) ) / &
                           SURF_AREA(:) 

     TERM2_BCLINU(:,K) = ( BCLIN_OMU(:,K) - BCLIN_OMU(:,K+1 ) ) / ( P_EDGE(K+1) - P_EDGE(K) )

     TERM3_BCLINU(:,K) = F(:) * V_CENT(:,K)

     TERM4_BCLINU(:,K) = BCLIN_UV_CENT(:,K) * tan( LAT_RADS(:) ) / R

     TERM5_BCLINU(:,K) = - BCLIN_RU(:,K)
     
!     TERM6_BCLINU(:,K) = VEL_VISC * vel_2der_zonal( BCLIN_U(:,K,2) )
!try evaluating diffusion terms at time n-1 on Eli's advice (dsa, 2/10/06)
!     TERM6_BCLINU(:,K) = VEL_VISC * vel_2der_zonal( BCLIN_U(:,K,1) )
     TERM6_BCLINU(:,K) = 0.0d0
  enddo

  SUM_BCLINU(:,:) = TERM1_BCLINU(:,:) + TERM2_BCLINU(:,:) + TERM3_BCLINU(:,:) + &
                    TERM4_BCLINU(:,:) + TERM5_BCLINU(:,:) + TERM6_BCLINU(:,:) + &
                    TERM7_BCLINU(:,:)

  write(UNIT=SS_VEL_FID,FMT='(A)') ''
  write(UNIT=SS_VEL_FID,FMT='(A)') ''
  write(UNIT=SS_VEL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_VEL_FID,FMT='(A)') 'terms in bclin_u balance (m s^-2)'
  write(UNIT=SS_VEL_FID,FMT='(A)') '********************************'
  write(UNIT=SS_VEL_FID,FMT='(2A3,8A12)') 'J','K','ADVY','ADVP','Coriolis','Geo. Term','Fric','Diff','Diff2','SUM'
  do K=1,KMAX
     do J=1,JMAX
 
        write(UNIT=SS_VEL_FID,FMT='(2I3,8E12.3)') J,K,TERM1_BCLINU(J,K),TERM2_BCLINU(J,K),   &
                                                  TERM3_BCLINU(J,K),TERM4_BCLINU(J,K),       &
                                                  TERM5_BCLINU(J,K),TERM6_BCLINU(J,K),       &
                                                  TERM7_BCLINU(J,K),SUM_BCLINU(J,K)
       
        write(UNIT=SS_VEL_PLOT_FID,FMT='(7E12.3)')TERM1_BCLINU(J,K),TERM2_BCLINU(J,K), &
                                                  TERM3_BCLINU(J,K),TERM4_BCLINU(J,K), &
                                                  TERM5_BCLINU(J,K),TERM6_BCLINU(J,K), &
                                                  SUM_BCLINU(J,K)
 
     enddo
  enddo
  write(UNIT=SS_VEL_FID,FMT='(A)') ''
  write(UNIT=SS_VEL_FID,FMT='(A)') ''


  !High Meridional Mass Flux
  JMAXp1_WRITE_ARR(:) = V_EDGE(:,1) * DX(:) * ( P_EDGE(2) - P_EDGE(1) ) / G / 1.0d9
  call write_out(SS_VEL_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Hi Merid. Mass Flux [10^9 kg/s]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Hi Y Mass Flux [10^9 kg/s]')


  !Low Meridional Mass Flux
  JMAXp1_WRITE_ARR(:) = V_EDGE(:,2) * DX(:) * ( P_EDGE(3) - P_EDGE(2) ) / G / 1.0d9
  call write_out(SS_VEL_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Low Merid. Mass Flux [10^9 kg/s]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Low Y Mass Flux [10^9 kg/s]')


  !Vertical Mass Flux
  !put into 10^9 kg/s
  JMAX_WRITE_ARR(:)  = - OMEGA(:) * SURF_AREA(:) / G / 1.0d9
  call write_out(SS_VEL_FID,JMAX,JMAX_WRITE_ARR(:),DUMMY,'Vert. Mass Flux [10^9 kg/s]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),DUMMY,'Vert. Mass Flux [10^9 kg/s]')


  !Barotropic geopotential height
  JMAX_WRITE_ARR(:)  = BTROP_PHI(:) / G / 1000.0d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_VEL_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'BTROP PHI [km]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'BTROP PHI [km]')


  !High Baroclinic Geopotential Height
  JMAX_WRITE_ARR(:)  = BCLIN_PHI(:,1) / G / 1000.0d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_VEL_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'BCLIN PHI(:,1) [km]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'BCLIN PHI(:,1) [km]')


  !Low Baroclinic Geopotential Height
  JMAX_WRITE_ARR(:)  = BCLIN_PHI(:,2) / G / 1000.0d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_VEL_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'BCLIN PHI(:,2) [km]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'BCLIN PHI(:,2) [km]')


  !High Total Geopotential Height
  JMAX_WRITE_ARR(:)  = PHI(:,1) / G / 1000.0d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_VEL_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'PHI(:,1) [km]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'PHI(:,1) [km]')


  !Low Total Geopotential Height
  JMAX_WRITE_ARR(:)  = PHI(:,2) / G / 1000.0d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_VEL_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'PHI(:,2) [km]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'PHI(:,2) [km]')


  !Barotropic Zonal Velocity
  JMAX_WRITE_ARR(:)  = BTROP_U(:,2)
  call write_out(SS_VEL_FID,JMAX,JMAX_WRITE_ARR(:),DUMMY,'BTROP U [m/s]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),DUMMY,'BTROP U [m/s]')


  !High Baroclinic Zonal Velocity
  JMAX_WRITE_ARR(:)  = BCLIN_U(:,1,2)
  call write_out(SS_VEL_FID,JMAX,JMAX_WRITE_ARR(:),DUMMY,'High BCLIN U [m/s]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),DUMMY,'High BCLIN U [m/s]')


  !Low Baroclinic Zonal Velocity
  JMAX_WRITE_ARR(:)  = BCLIN_U(:,2,2)
  call write_out(SS_VEL_FID,JMAX,JMAX_WRITE_ARR(:),DUMMY,'Low BCLIN  U [m/s]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),DUMMY,'Low BCLI U [m/s]')


  !High Total Zonal Velocity
  JMAX_WRITE_ARR(:)  = U(:,1)
  call write_out(SS_VEL_FID,JMAX,JMAX_WRITE_ARR(:),DUMMY,'High Tot. U [m/s]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),DUMMY,'High Tot. U [m/s]')


  !Low Total Zonal Velocity
  JMAX_WRITE_ARR(:)  = U(:,2)
  call write_out(SS_VEL_FID,JMAX,JMAX_WRITE_ARR(:),DUMMY,'Low Tot. U [m/s]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),DUMMY,'Low Tot. U [m/s]')


  !High Meridional Velocity
  JMAXp1_WRITE_ARR(:) = V_EDGE(:,1)
  call write_out(SS_VEL_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'V EDGE(:,1) [m/s]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'V EDGE(:,1) [m/s]')


  !Low Meridional Velocity
  JMAXp1_WRITE_ARR(:) = V_EDGE(:,2)
  call write_out(SS_VEL_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'V EDGE(:,2) [m/s]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'V EDGE(:,2) [m/s]')


  !Pressure Velocity
  JMAX_WRITE_ARR(:)  = OMEGA(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_VEL_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'OMEGA [Pa/s]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'OMEGA [Pa/s]')




end subroutine investigate_steady_v


!******************************************************************************
! (63) subroutine investigate_steady_temps
!      *see what final temperatures and mean temperatures are
!      *this subroutine is written to investigate climate sensitivities
!******************************************************************************
subroutine investigate_steady_temps

!internal
  real*8                       :: JMAX_WRITE_AVG
  real*8, dimension(JMAX)      :: ABSORB_CLEAR,SW_CLD,               &
                                  F_LW_DOWN_CLEAR,                   &
                                  F_LW_SURF_UP_CLEAR,OLR_CLEAR,      &
                                  MEAN_ST,ALB,JMAX_WRITE_ARR
  real*8, dimension(JMAX+1)    :: JMAXp1_WRITE_ARR
  

  !SW stuff
  ABSORB_CLEAR(:)  = SW_FLUX(:) * ( ( 1.0d0 - FRAC_LAND(:) ) * ( 1.0d0 - ALPHA_OCN(:) )    + &
                                              FRAC_LAND(:)   * ( 1.0d0 - ALPHA_LAND(:) ) ) * &
                                  ( 1.0d0 - ALPHA_ATM )

  SW_CLD(:)        = ABSORB_SW(:) - ABSORB_CLEAR(:)

  !LW stuff
  F_LW_DOWN_CLEAR(:)     = ( EMISS_CLEAR(:,2) * SIGMA * T(:,2) ** 4.0d0                                + &
                           ( 1.0d0 - EMISS_CLEAR(:,2) ) * EMISS_CLEAR(:,1) * SIGMA * T(:,1) ** 4.0d0 )
  
  F_LW_SURF_UP_CLEAR(:)  = FRAC_LAND(:) * ( EMISS_LAND * SIGMA * TS(:,2) ** 4.0d0                  + &
                           ( 1.0d0 - EMISS_LAND ) * F_LW_DOWN_CLEAR(:) )                          + &
                           ( 1.0d0 - FRAC_LAND(:) ) * ( EMISS_OCN * SIGMA * SST(:,2) ** 4.0d0      + &
                           ( 1.0d0 - EMISS_OCN ) * F_LW_DOWN_CLEAR(:) )

  OLR_CLEAR(:)           = EMISS_CLEAR(:,1) * SIGMA * T(:,1) ** 4.0d0 + &
                           ( 1.0d0 - EMISS_CLEAR(:,1) )           * & 
                           EMISS_CLEAR(:,2) * SIGMA * T(:,2) ** 4.0d0 + &
                           ( 1.0d0 - EMISS_CLEAR(:,1) )           * &
                           ( 1.0d0 - EMISS_CLEAR(:,2) )           * &
                           F_LW_SURF_UP_CLEAR(:)
                      

  !mean surface temperature
  MEAN_ST(:) = TS(:,2) * FRAC_LAND(:) + ( 1.0d0 - FRAC_LAND(:) ) * SST(:,2)

  !albedo
  ALB(:)  = 1.d0 - ABSORB_SW(:) / SW_FLUX(:)


!-------------------------------
!start writing out diagnostics
!-------------------------------

  !High DSE
  JMAX_WRITE_ARR(:)  = DSE(:,1,2)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
!  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'High DSE [J/Kg]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'DSE(:,1) [J/Kg]')

  !Low DSE
  JMAX_WRITE_ARR(:)  = DSE(:,2,2)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
!  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Low DSE [J/Kg]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'DSE(:,2) [J/Kg]')

  !High Cloud Temperature                                                                                     
  JMAX_WRITE_ARR(:)  = T_H_CLD(:) - 273.15d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'High Cloud Temperature [C]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'T_H_CLD(:) [C]')


  !High Temperature
  JMAX_WRITE_ARR(:)  = T(:,1) - 273.15d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'High Temperature [C]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'T(:,1) [C]')


  !High Theta
  JMAX_WRITE_ARR(:)  = THETA(:,1) - 273.15d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'High Theta [C]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'THETA(:,1) [C]')


  !Low Temperature
  JMAX_WRITE_ARR(:)  = T(:,2) - 273.15d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Low Temperature [C]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'T(:,2) [C]')


  !Low Theta
  JMAX_WRITE_ARR(:)  = THETA(:,2) - 273.15d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Low Theta [C]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'THETA(:,2) [C]')


  !Land Tempature
  !average is calculated only over land area
  JMAX_WRITE_ARR(:)  = TS(:,2) - 273.15d0
  if( sum( FRAC_LAND(:) * SURF_AREA(:), 1 ) .eq. 0.0d0 ) then
     !avoids NaN average when doing a waterworld run (dsa, 1/28/06)
     JMAX_WRITE_AVG  = 0.0d0
  else
     JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * FRAC_LAND(:) * SURF_AREA(:), 1) / &
                          sum( FRAC_LAND(:) * SURF_AREA(:), 1 )
  endif
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Land Temperature [C]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'TS(:) [C]')


  !Sea Surface Temperature
  !average is calculated only over ocean area
  JMAX_WRITE_ARR(:)  = SST(:,2) - 273.15d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * ( 1.0d0 - FRAC_LAND(:) ) * SURF_AREA(:), 1) / &
                       sum( ( 1.0d0 - FRAC_LAND(:) ) * SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Sea Surface Temperature [C]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'SST(:) [C]')


  !Mean Surface Temperature
  JMAX_WRITE_ARR(:)  = TS(:,2) * FRAC_LAND(:) + ( 1.0d0 - FRAC_LAND(:) ) * SST(:,2) - 273.15d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Mean Surface Temperature [C]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'MEAN ST [C]')

 
  !Albedo
  JMAX_WRITE_ARR(:)  = ALB(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Albedo')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Albedo')


  !High Emissivity
  JMAX_WRITE_ARR(:)  = EMISS(:,1)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'High Emissivity')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'EMISS(:,1)')


  !Low Emissivity
  JMAX_WRITE_ARR(:)  = EMISS(:,2)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Low Emissivity')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'EMISS(:,2)')
  

  !High Clear Emissivity
  JMAX_WRITE_ARR(:)  = EMISS_CLEAR(:,1)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'High Clear Emissivity')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'EMISS CLEAR(:,1)')
 

  !Low Clear Emissivity
  JMAX_WRITE_ARR(:)  = EMISS_CLEAR(:,2)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Low Clear Emissivity')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'EMISS CLEAR(:,2)')
 

  !Incoming SW
  JMAX_WRITE_ARR(:)  = SW_FLUX(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'TOA SW [W m^-2]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'TOA SW [W m^-2]')
 

  !Absorbed SW
  JMAX_WRITE_ARR(:)  = ABSORB_SW(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Abs. SW [W m^-2]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Abs. SW [W m^-2]')


  !Cld SW Forcing
  JMAX_WRITE_ARR(:)  = SW_CLD(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Cld SW Forcing [W m^-2]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Cld SW [W m^-2]')


  !OLR
  JMAX_WRITE_ARR(:)  = OLR(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'OLR [W m^-2]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'OLR [W m^-2]')
 

  !OLR Clear
  JMAX_WRITE_ARR(:)  = OLR_CLEAR(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'OLR Clear [W m^-2]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'OLR Clear [W m^-2]')

 
  !Cld LW Forcing
  JMAX_WRITE_ARR(:)  = OLR_CLEAR(:)-OLR(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Cld LW Forcing[W m^-2]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Cld LW [W m^-2]')


  !Total Cld Forcing
  JMAX_WRITE_ARR(:)  = OLR_CLEAR(:)-OLR(:)+SW_CLD(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Total Cld [W m^-2]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Total Cld [W m^-2]')
     

  !SW-OLR - global avg. zero if code has properly converged
  JMAX_WRITE_ARR(:)  = ABSORB_SW(:) - OLR(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'SW-OLR [W m^-2]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'SW-OLR [W m^-2]')
 

  !Energy Flux into Ocean - global avg. zero if code has properly converged
  JMAX_WRITE_ARR(:)  = FLUX_IN_OCN(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Heat Flux Into OCN [W m^-2]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Heat Flux Into OCN [W m^-2]')
 

  !Ocean Sensible Heat Flux
  JMAX_WRITE_ARR(:)  = OCN_SH_FLUX(:)  
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'OCN SH FLUX [W m^-2]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'OCN SH FLUX [W m^-2]')


  !Land Sensible Heat Flux
  JMAX_WRITE_ARR(:)  = LAND_SH_FLUX(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_TEMP_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'LAND SH FLUX [W m^-2]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'LAND SH FLUX [W m^-2]')

    
  !High Eddy DSE Transport
  JMAXp1_WRITE_ARR(:) = EDDY_HEAT(:,1) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'High Eddy DSE Trans. [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Hi Eddy DSE Trans. [PW]')


  !Low Eddy DSE Transport
  JMAXp1_WRITE_ARR(:) = EDDY_HEAT(:,2) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Low Eddy DSE Trans. [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Low Eddy DSE Trans. [PW]')


  !High Eddy LE Transport
  JMAXp1_WRITE_ARR(:) = EDDY_LATENT(:,1) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Hi Eddy LE Trans. [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Hi Eddy LE Trans. [PW]')

          
  !Low Eddy LE Transport
  JMAXp1_WRITE_ARR(:) = EDDY_LATENT(:,2) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Low Eddy LE Trans. [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Low Eddy LE Trans. [PW]')

        
  !High Average Meridional DSE Transport 
  JMAXp1_WRITE_ARR(:) = ADVECT_HEAT_Y(:,1) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'ADVECT HEAT Y(:,1) [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'ADVECT HEAT Y(:,1) [PW]')


  !Low Average Meridional DSE Transport 
  JMAXp1_WRITE_ARR(:) = ADVECT_HEAT_Y(:,2) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'ADVECT HEAT Y(:,2) [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'ADVECT HEAT Y(:,2) [PW]')


  !High Average Meridional LE Transport 
  JMAXp1_WRITE_ARR(:) = ADVECT_LATENT_Y(:,1) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'ADVECT LATENT Y(:,1) [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'ADVECT LATENT Y(:,1) [PW]')


  !Low Average Meridional LE Transport 
  JMAXp1_WRITE_ARR(:) = ADVECT_LATENT_Y(:,2) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'ADVECT LATENT Y(:,2) [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'ADVECT LATENT Y(:,2) [PW]')


  !Total Eddy DSE Transport
  JMAXp1_WRITE_ARR(:) = sum( EDDY_HEAT(:,:), 2 ) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Eddy DSE Trans. [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Eddy DSE Trans. [PW]')


  !Total Eddy DSE Transport
  JMAXp1_WRITE_ARR(:) = sum( EDDY_LATENT(:,:), 2 ) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Eddy LE Trans. [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Eddy LE Trans. [PW]')


  !Total Average Meridional DSE Transport
  JMAXp1_WRITE_ARR(:) = sum( ADVECT_HEAT_Y(:,:), 2 ) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Avg. DSE Trans. [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Avg. DSE Trans. [PW]')


  !Total Average Meridional LE Transport
  JMAXp1_WRITE_ARR(:) = sum( ADVECT_LATENT_Y(:,:), 2 ) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Avg. LE Trans. [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Avg. LE Trans. [PW]')


  !Ocean Heat Transport
  JMAXp1_WRITE_ARR(:) = OHT(:) / 1.0d15
  call write_out(SS_TEMP_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'OHT [PW]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'OHT [PW]')


end subroutine investigate_steady_temps


!******************************************************************************
! (64) subroutine investigate_steady_moist
!      *see what final moisture variables are
!******************************************************************************
subroutine investigate_steady_moist
  real*8, dimension(JMAX)      :: JMAX_WRITE_ARR
  real*8, dimension(JMAX+1)    :: JMAXp1_WRITE_ARR
  real*8                       :: JMAX_WRITE_AVG


!-------------------------------
!start writing out diagnostics
!-------------------------------

  !High Specific Humidity
  JMAX_WRITE_ARR(:)  = Q(:,1,2) * 1000.0d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'High Q [g/kg]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'High Q [g/kg]')


  !Low Specific Humidity
  JMAX_WRITE_ARR(:)  = Q(:,2,2) * 1000.0d0
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Low Q [g/kg]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Low Q [g/kg]')


  !High Relative Humidity
  JMAX_WRITE_ARR(:)  = RH(:,1) 
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'High RH')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'High RH')


  !Low Relative Humidity
  JMAX_WRITE_ARR(:)  = RH(:,2) 
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Low RH')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Low RH')


  !(Low MSE - High MSE^star)/Cp (K)
  JMAX_WRITE_ARR(:)  = ( MSE(:,2) - MSE_STAR(:,1) ) / CPD
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),DUMMY,'MSE Diff / CP [K]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),DUMMY,'MSE Diff / CP [K]')

  !(Low MSE - High MSE^star)/Cp (K)
  !for special descent case 
  JMAX_WRITE_ARR(:)  = ( MSE(:,2) - MSE_STAR_TILDE(:) ) / CPD
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),DUMMY,'Descent MSE Diff / CP [K]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),DUMMY,'Descent MSE Diff / CP [K]')

  !convection constant
  JMAX_WRITE_ARR(:) = ( K_CON(:) - K_CON_MIN ) / ( K_CON_MAX - K_CON_MIN )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),DUMMY,'Norm. Con. Const. []')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),DUMMY,'Norm. Con. Const. []')

  !High Precipitation 
  !convert from kg/s to m/yr each box would produce
  JMAX_WRITE_ARR(:)  = PRECIP(:,1) * YEAR / RHO_H2O / SURF_AREA(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'High Precip [m/yr]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'High Precip [m/yr]')


  !Low Precipitation 
  !convert from kg/s to m/yr each box would produce
  JMAX_WRITE_ARR(:)  = PRECIP(:,2) * YEAR / RHO_H2O / SURF_AREA(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Low Precip [m/yr]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Low Precip [m/yr]')


  !Epsilon Re-evaporation 
  JMAX_WRITE_ARR(:)  = EPS_RE_EVAP(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'EPS Re-Evap')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'EPS Re-Evap')


  !Re-evaporation 
  !convert from kg/s to m/yr each box would produce
  JMAX_WRITE_ARR(:)  = RE_EVAP(:) * YEAR / RHO_H2O / SURF_AREA(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Re-Evap [m/yr]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Re-Evap [m/yr]')

  !Precipitation Hitting the Ground
  !convert from kg/s to m/yr each box would produce
  JMAX_WRITE_ARR(:)  = PRECIP_G(:) * YEAR / RHO_H2O / SURF_AREA(:)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Ground Precip. [m/yr]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Ground Precip. [m/yr]')


  !Ocean Evaporation
  !convert from ((kg H20 vap) m^-2 s^-1)) to m/yr
  JMAX_WRITE_ARR(:)  = OCN_EVAP_FLUX(:)  * YEAR / RHO_H2O
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Ocn Evap. [m/yr]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Ocn Evap [m/yr]')


  !Land Evaporation
  !convert from ((kg H20 vap) m^-2 s^-1)) to m/yr
  JMAX_WRITE_ARR(:)  = LAND_EVAP_FLUX(:)  * YEAR / RHO_H2O
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Land Evap. [m/yr]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Land Evap [m/yr]')


  !Precipitation minus Evaporation 
  !convert from ((kg H20 vap) m^-2 s^-1)) to m/yr
  JMAX_WRITE_ARR(:)  = PRECIP_G(:) * YEAR / RHO_H2O / SURF_AREA(:) - & 
                       ( ( 1.0d0 - FRAC_LAND(:) ) * OCN_EVAP_FLUX(:) + &
                       FRAC_LAND(:) * LAND_EVAP_FLUX(:) ) * YEAR / RHO_H2O
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'P-E [m/yr]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'P-E [m/yr]')


  !Run-off from land 
  !convert from kg/s to m/yr each box would produce
  where( FRAC_LAND(:) .gt. 0.0d0 )
     !this where statement avoids NaN when FRAC_LAND=0 (dsa, 1/28/06)
     JMAX_WRITE_ARR(:) = RUNOFF * YEAR / RHO_H2O / SURF_AREA(:) / FRAC_LAND(:)
  elsewhere
     JMAX_WRITE_ARR(:) = 0.d0
  endwhere
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Run-off [m/yr]')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'Run-off [m/yr]')


  !High Cloud Fraction 
  !now back on extra high cloud layer (dsa, 6/9/06)
  JMAX_WRITE_ARR(:)  = FRAC_H_CLD(:)
  JMAX_WRITE_AVG     = SUM( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / SUM( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'HCLD FRAC')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'HCLD FRAC')


  !High Cloud Fraction 
  !stopped using strange extra high cloud layer and so am just 
  !calling the upper box high clds (dsa, 12/18/05)
  !now back on extra high cloud layer (dsa, 6/9/06)
  JMAX_WRITE_ARR(:)  = FRAC_CLD(:,1)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'FRAC CLD(:,1)')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'FRAC CLD(:,1)')


  !Low Cloud Fraction 
  JMAX_WRITE_ARR(:)  = FRAC_CLD(:,2)
  JMAX_WRITE_AVG     = sum( JMAX_WRITE_ARR(:) * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
  call write_out(SS_MOIST_FID,JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'FRAC CLD(:,2)')
  call write_for_plotting(JMAX,JMAX_WRITE_ARR(:),JMAX_WRITE_AVG,'FRAC CLD(:,2)')


  !High Eddy Mositure Transport
  JMAXp1_WRITE_ARR(:) = EDDY_MOIST(:,1) / 1.0d9
  call write_out(SS_MOIST_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'High Eddy Moist. Trans. [Sv]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'High Eddy Moist. Trans. [Sv]')


  !Low Eddy Mositure Transport
  JMAXp1_WRITE_ARR(:) = EDDY_MOIST(:,2) / 1.0d9
  call write_out(SS_MOIST_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Low Eddy Moist. Trans. [Sv]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Low Eddy Moist. Trans. [Sv]')


  !High Mean Mositure Transport
  JMAXp1_WRITE_ARR(:) = ADV_MOIST(:,1) / 1.0d9
  call write_out(SS_MOIST_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'High Mean Moist. Trans. [Sv]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'High Mean Moist. Trans. [Sv]')


  !Low Mean Mositure Transport
  JMAXp1_WRITE_ARR(:) = ADV_MOIST(:,2) / 1.0d9
   call write_out(SS_MOIST_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Low Mean Moist. Trans. [Sv]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Low Mean Moist. Trans. [Sv]')


  !Total Eddy Mositure Transport
  JMAXp1_WRITE_ARR(:) = sum( EDDY_MOIST(:,:), 2 ) / 1.0d9
  call write_out(SS_MOIST_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Tot. Eddy Moist. Trans. [Sv]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Tot. Eddy Moist. Trans. [Sv]')


  !Total Mean Mositure Transport
  JMAXp1_WRITE_ARR(:) = sum( ADV_MOIST(:,:), 2 ) / 1.0d9
  call write_out(SS_MOIST_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Tot. Mean Moist. Trans. [Sv]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Tot. Mean Moist. Trans. [Sv]')


  !Total Mositure Transport
  JMAXp1_WRITE_ARR(:) = sum( ADV_MOIST(:,:) + EDDY_MOIST(:,:), 2 ) / 1.0d9
  call write_out(SS_MOIST_FID,JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Tot. Moist. Trans. [Sv]')
  call write_for_plotting(JMAX+1,JMAXp1_WRITE_ARR(:),DUMMY,'Tot. Moist. Trans. [Sv]')


end subroutine investigate_steady_moist

!******************************************************************************
! (65) function emissivity_o3
!      *Sasamori (1968) eqns (22) and (23)
!      *U is the effective path length in cm at STP
!******************************************************************************
function emissivity_o3( U ) result ( EMISS_O3 )
!inputs
  real*8, dimension(:), intent(in) :: U
!outputs
  real*8, dimension(size(U,1))     :: EMISS_O3

  where( U(:) .lt. 0.01d0 )
     EMISS_O3(:) = 0.209d0 * ( U(:) + 7.0d-5 ) ** ( 0.436d0 ) - 0.00321d0
  elsewhere
     EMISS_O3(:) = 0.0212d0 * log10( U(:) ) + 0.0748d0
  end where
 
end function emissivity_o3

!******************************************************************************
! (66) function path_length_o3
!      *PUMA notes, eqn (2.70)
!      *assumes an idealized O3 distribution given in Green (1963)
!      *really a mid-latitude winter distribution
!      *Z should be input in km
!******************************************************************************
function path_length_o3( Z ) result ( U )
!inputs
  real*8, intent(in) :: Z
!outputs
  real*8             :: U
!internal
  real*8, parameter  :: A=0.5d0, & !cm @ STP
                        B=20.d0, & !km
                        C=5.d0     !km

  U = A * ( 1.0d0 + exp( - B / C ) ) / ( 1.0d0 + exp( ( Z - B ) / C ) )
  

end function path_length_o3


!******************************************************************************
! (67) subroutine co2_vary_exp
!      *use this function to slowly vary co2 for quasi-equilibrium 
!       experiments
!      *specify how co2 is varied here
!      *also prints out variables of interest - written so that JMAX can be 
!       easily varied
!       (dsa, 10/18/05)
!******************************************************************************
subroutine co2_vary_exp

  real*8, dimension(JMAX)  :: M_FLUX_Z,MSE_DIFF
  real*8                   :: T1,T2,T3

  !don't do anything initially - let system equilibrate
  if( TIME .gt. T_START_VARY ) then
     !print out variables of interest at appropriate times
     !variables are printed at the same time as in diags.out, so if I really
     !want to I can compare variables between the files
     if ( abs ( TIME - int( TIME / TPRINT ) * TPRINT ) .lt. DT / 2.0d0 ) then

        !vertical mass flux (10^9 kg/s)
        M_FLUX_Z(:) = - OMEGA(:) * SURF_AREA(:) / G / 1.0d9

        !MSE difference that acts as convection switch
        MSE_DIFF(:) = ( MSE(:,2) - MSE_STAR(:,1) ) / CPD

        write(PARAM_VARY_FID,'(250E13.5)') TIME/YEAR,M_FLUX_Z,T(:,2),SST(:,2), &
                                           TS(:,2),FRAC_CLD,MSE_DIFF,CO2

     endif


!     if( TIME .lt. TEND / 2.0d0 ) then
!        CO2 = CO2 + DT * 10.0d-6 / YEAR
!     else
!        CO2 = CO2 - DT * 10.0d-6 / YEAR
!     endif


!     if( CO2 .lt. CO20 ) then
!        CO2 = CO20
!     endif


!------------------------------------
     !linear changes with:
     ! CO2(T1) = CO2_1
     ! CO2(T2) = CO2_2                                                                                            
     ! CO2(T3) = CO2_1
     ! (dsa, 3/30/06)

     T1 = T_START_VARY
     T3 = TEND
     T2 = ( T1 + T3 ) / 2.0d0

!     if( TIME .lt. T1 ) then
!        CO2 = CO2_1
!     elseif( ( TIME .ge. T1 ) .and. ( TIME .lt. T2 ) ) then
!        CO2 = CO2_1 + ( CO2_2 - CO2_1 ) * ( TIME - T1 ) / ( T2 - T1 )
!     else
!        CO2 = CO2_2 + ( CO2_1 - CO2_2 ) * ( TIME - T2 ) / ( T3 - T2 )
!     endif

     !vary CO2 in one direction only
     !(dsa, 6/14/06)
     if( TIME .lt. T1 ) then
        CO2 = CO2_1
     else
        CO2 = CO2_1 + ( CO2_2 - CO2_1 ) * ( TIME - T1 ) / ( T3 - T1 )
     endif



  endif

end subroutine co2_vary_exp


!******************************************************************************
! watch out, hard-wired for JMAX=3 right now!
!******************************************************************************
subroutine oht_vary_exp

  real*8, dimension(JMAX)  :: M_FLUX_Z,MSE_DIFF
  real*8                   :: TIME_BAR


  !don't do anything initially - let system equilibrate
  if( TIME .gt. T_START_VARY ) then
     !print out variables of interest at appropriate times
     !variables are printed at the same time as in diags.out, so if I really
     !want to I can compare variables between the files
     if ( abs ( TIME - int( TIME / TPRINT ) * TPRINT ) .lt. DT / 2.0d0 ) then

        !vertical mass flux (10^9 kg/s)
        M_FLUX_Z(:) = - OMEGA(:) * SURF_AREA(:) / G / 1.0d9


        !MSE difference that acts as convection switch
        MSE_DIFF(:) = ( MSE(:,2) - MSE_STAR(:,1) ) / CPD
        

        write(PARAM_VARY_FID,'(250E13.5)') TIME/YEAR,OHT,M_FLUX_Z,T(:,2),SST(:,2), &
                                           TS(:,2),FRAC_CLD,MSE_DIFF


     endif



     TIME_BAR = ( T_START_VARY + TEND ) / 2.0d0

!vary up, then down
     if( TIME .lt. T_START_VARY ) then
        OHT_GMAX = OHT_B
     elseif( ( TIME .ge. T_START_VARY ) .and. ( TIME .lt. TIME_BAR ) ) then
        OHT_GMAX = OHT_B + ( OHT_VARY_MAX - OHT_B ) * &
                 ( TIME - T_START_VARY ) / ( TIME_BAR - T_START_VARY )
     elseif( TIME .ge. TIME_BAR ) then
        OHT_GMAX = OHT_B + ( OHT_VARY_MAX - OHT_B ) * &
                 ( TEND - TIME ) / ( TEND - TIME_BAR )
     endif

!vary up only
!     if( TIME .lt. T_START_VARY ) then
!        OHT_GMAX = OHT_B
!     else 
!        OHT_GMAX = OHT_B + ( OHT_VARY_MAX - OHT_B ) * &
!                 ( TIME - T_START_VARY ) / ( TEND - T_START_VARY )
!     endif


     OHT(2:JMAX) = OHT_S(2:JMAX) + OHT_GMAX * 1.0d15 *                &
                   exp( - ( LAT_EDGE(2:JMAX) - LAT_GAUSS ) ** 2.0d0 / &
                   ( 2 * SIG_GAUSS ** 2.0d0) )
     
  endif

end subroutine oht_vary_exp



!******************************************************************************
! (67) subroutine cld_vary_exp
!      *use this function to slowly vary clds for quasi-equilibrium 
!       experiments
!      *specify how clds are varied here
!      *also prints out variables of interest - written so that JMAX can be 
!       easily varied
!       (dsa, 10/18/05)
!******************************************************************************
subroutine cld_vary_exp

  real*8, dimension(JMAX)  :: M_FLUX_Z,MEAN_ST
  real*8                   :: AVG_SST,AVG_TS,AVG_MEAN_ST

  !don't do anything initially - let system equilibrate
  if( TIME .gt. T_START_VARY ) then
     !print out variables of interest at appropriate times
     !variables are printed at the same time as in diags.out, so if I really
     !want to I can compare variables between the files
     if ( abs ( TIME - int( TIME / TPRINT ) * TPRINT ) .lt. DT / 2.0d0 ) then

        !vertical mass flux (10^9 kg/s)
        M_FLUX_Z(:) = - OMEGA(:) * SURF_AREA(:) / G / 1.0d9

        !mean surface temperature
        MEAN_ST(:) = TS(:,2) * FRAC_LAND(:) + ( 1.0d0 - FRAC_LAND(:) ) * SST(:,2)

        !area-weighted averages
        AVG_TS        = sum( TS(:,2)          * FRAC_LAND(:) * SURF_AREA(:), 1)            / &
                        sum( FRAC_LAND(:) * SURF_AREA(:), 1 )
        AVG_SST       = sum( SST(:,2)         * (1.0d0 - FRAC_LAND(:) ) * SURF_AREA(:), 1) / &
                        sum( (1.0d0 - FRAC_LAND(:) ) * SURF_AREA(:), 1 )
        AVG_MEAN_ST   = sum( MEAN_ST(:)       * SURF_AREA(:), 1) / sum( SURF_AREA(:), 1 )
        
!        write(PARAM_VARY_FID,'(100g20.12)') TIME/YEAR,CO2*1.d6,M_FLUX_Z,TS(:,2),AVG_TS, &
!                                            SST(:,2),AVG_SST,MEAN_ST,AVG_MEAN_ST,       &
!                                            FRAC_CLD,FRAC_H_CLD
!
!        write(PARAM_VARY_FID,'(100g20.12)') TIME/YEAR,M_FLUX_Z,MEAN_ST,AVG_MEAN_ST,      &
!                                            FRAC_CLD

     endif

     !in this experiment, slowly pivoting clds all at once (dsa, 10/18/05)


     !high clouds
!     FRAC_H_CLD(:) = HCLD0(:) + ( ( TIME - T_START_VARY ) / ( TEND - T_START_VARY ) ) * ( HCLDF(:) - HCLD0(:) )
     !middle clouds
!     FRAC_CLD(:,1) = MCLD0(:) + ( ( TIME - T_START_VARY ) / ( TEND - T_START_VARY ) ) * ( MCLDF(:) - MCLD0(:) )
     !low clouds
!     FRAC_CLD(:,2) = LCLD0(:) + ( ( TIME - T_START_VARY ) / ( TEND - T_START_VARY ) ) * ( LCLDF(:) - LCLD0(:) )

!     !middle low
!     FRAC_CLD(2,2) = LCLD0(2) + ( ( TIME - T_START_VARY ) / ( TEND - T_START_VARY ) ) * (  - LCLD0(2) )
!     !north low
!     FRAC_CLD(3,2) = LCLD0(3) + ( ( TIME - T_START_VARY ) / ( TEND - T_START_VARY ) ) * (  - LCLD0(3) )
!     !middle high
!     FRAC_H_CLD(2) = HCLD0(2) + ( ( TIME - T_START_VARY ) / ( TEND - T_START_VARY ) ) * ( 0.00d0 - HCLD0(2) )
!     !north high
!     FRAC_H_CLD(3) = HCLD0(3) + ( ( TIME - T_START_VARY ) / ( TEND - T_START_VARY ) ) * ( 0.75d0 - HCLD0(3) )


  endif

end subroutine cld_vary_exp


!******************************************************************************
! (68) subroutine update_mse
!      *calculates the moist static energy and saturation 
!       moist static energy in each box
!      *based on eqn (4.5.23) in Atmospheric Convection by Emanuel
!      *simplified by neglegting variation of Cp and Lv with moisture and 
!       temperature
!      *should be updated after LV, R_MIX, T
!******************************************************************************
subroutine update_mse

  real*8, dimension(JMAX) :: DEL_PHI,T_TILDE,E_TILDE_STAR, &
                             Q_TILDE_STAR,R_MIX_TILDE_STAR, &
                             PHI_TILDE

  !moist static energies
  MSE(:,:)      = DSE(:,:,2) + LV0 * R_MIX(:,:)
  MSE_STAR(:,:) = DSE(:,:,2) + LV0 * R_MIX_STAR(:,:)

!==================================================================
!what follows was added as a round-about, but seemingly consistent,
!way to shut off convection when there's descent. See notes for
!details.  
!(dsa, 2/13/06)
!==================================================================

!  if( P_TILDE .lt. P_EDGE(2) ) then
!     PHI_TILDE(:) = PHI(:,1) + ALPHA(:,1) * ( P(1) - P_TILDE )
!  else
!     PHI_TILDE(:) = PHI(:,2) + ALPHA(:,2) * ( P(2) - P_TILDE )
!  endif

!  DEL_PHI(:)      = PHI_TILDE - PHI(:,1)
!  T_TILDE(:)      = T(:,1) - DEL_PHI / CPD
!  E_TILDE_STAR(:) = calc_e_star( T_TILDE(:) )
!  Q_TILDE_STAR(:) = calc_q_from_e( P(1), E_TILDE_STAR(:) )
!  R_MIX_TILDE_STAR(:) = Q_TILDE_STAR(:) / ( 1.0d0 - Q_TILDE_STAR(:) )

!  MSE_STAR_TILDE(:) = DSE(:,1,2) + LV0 * R_MIX_TILDE_STAR(:)

!  where( OMEGA(:) .gt. 0.0d0 )
!      K_CON(:) = K_CON_MIN + 0.5d0 * ( K_CON_MAX - K_CON_MIN ) *  &
!                 ( 1 + tanh( ( MSE(:,2) - MSE_STAR_TILDE(:) ) /   &
!                 ( CPD * EPS_CON ) ) )
!  elsewhere
!      K_CON(:) = K_CON_MIN + 0.5d0 * ( K_CON_MAX - K_CON_MIN ) *  &
!                 ( 1 + tanh( ( MSE(:,2) - MSE_STAR(:,1) )      /  &
!                 ( CPD * EPS_CON ) ) )
!  endwhere

!==================================================================
!end of new descent formulation
!==================================================================
  
  !convection constant (smoothly switch convection on)
  K_CON(:) = K_CON_MIN + 0.5d0 * ( K_CON_MAX - K_CON_MIN ) *    &
             ( 1 + tanh( ( MSE(:,2) - MSE_STAR(:,1) ) / ( CPD * EPS_CON ) ) )

  where( OMEGA(:) .gt. OMEGA_NO_CONV )
     K_CON(:) = K_CON_MIN
  endwhere

end subroutine update_mse



!******************************************************************************
! (69) subroutine update_clds
!      *based on section 3 in Slingo and Slingo (1991)
!       (dsa, 10/24/05)
!      * rewritten, now based on XU and Krueger (1991), which seems simpler and 
!        more useful for this model
!       (dsa, 11/29/05)
!******************************************************************************
subroutine update_clds

  integer                        :: K
  real*8, dimension(JMAX)        :: C_C,M
  real*8, dimension(JMAX,KMAX)   :: RH_PRIME,C_S
  real*8, dimension(JMAX,KMAX+1) :: CLD_TEMP


!#######################
!convective clouds
!#######################
  !analogue to convective mass flux
  M(:)   = ( K_CON(:) - K_CON_MIN ) / ( K_CON_MAX - K_CON_MIN )

  where( M(:) .gt. M_MIN )
     C_C(:) = C_C_MAX * ( log( M(:) / M_MIN ) / log( M_MIN ) ) ** 2.0d0
  elsewhere
     C_C(:) = 0.0d0
  endwhere


!#######################
!stratiform clouds
!#######################

  !adjust RH to remove saturated convective clouds
  RH_PRIME(:,1) = ( 1.0d0 - C_C(:) ) * RH(:,1)
  RH_PRIME(:,2) =  RH(:,2)

  do K = 1, KMAX

     !stratiform clouds
     if( CLD_OPT ) then
        where( RH_PRIME(:,K) .ge. RH0(K) )
           C_S(:,K) = C_S_MAX(K) * ( ( RH_PRIME(:,K) - RH0(K) ) / ( CRIT_RH(K) - RH0(K) ) ) ** 2.0d0
        elsewhere
           C_S(:,K) = 0.0d0
        endwhere
     else
        where( RH_PRIME(:,K) .ge. RH0(2) )
           C_S(:,K) = C_S_MAX(2) * ( ( RH_PRIME(:,K) - RH0(2) ) / ( CRIT_RH(2) - RH0(2) ) ) ** 2.0d0
        elsewhere
           C_S(:,K) = 0.0d0
        endwhere
     endif

  enddo
     
  !high clouds
  CLD_TEMP(:,1) = C_C(:)
  !normal high and low clouds
  CLD_TEMP(:,2) = C_C(:) + C_S(:,1)
  CLD_TEMP(:,3) = C_S(:,2)

  where( M(:) .gt. M_MIN )
     CLD_TEMP(:,1) = C_C(:)
     CLD_TEMP(:,2) = C_C(:)
     CLD_TEMP(:,3) = 0.0d0
  elsewhere
     CLD_TEMP(:,1) = 0.0d0
     CLD_TEMP(:,2) = C_S(:,1)
     CLD_TEMP(:,3) = C_S(:,2)
  endwhere

  where( CLD_TEMP(:,:) .gt. 1.0d0 )
     CLD_TEMP(:,:) = 1.0d0
  endwhere

  FRAC_H_CLD(:) = CLD_TEMP(:,1)
!  FRAC_H_CLD(:) = 0.0d0
  FRAC_CLD(:,:) = CLD_TEMP(:,2:3)

!  if ( TIME .gt. ( TEND - 2.0d0 * DT ) ) then
!     write(UNIT=CLD_FID,FMT='(A)') '$$$$$$$$$$$$$$$$$$$$$$$$'
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') 'C_C_MAX'
!     write(UNIT=CLD_FID,FMT='(2F10.3)') C_C_MAX
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') '$$$$$$$$$$$$$$$$$$$$$$$$'
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') 'C_S_MAX(2)'
!     write(UNIT=CLD_FID,FMT='(2F10.3)') C_S_MAX(:)
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') '$$$$$$$$$$$$$$$$$$$$$$$$'
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') 'Convective'
!     write(UNIT=CLD_FID,FMT='(3F10.3)') C_C
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') '$$$$$$$$$$$$$$$$$$$$$$$$'
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') 'Mass Flux'
!     write(UNIT=CLD_FID,FMT='(3F10.3)') M
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') '$$$$$$$$$$$$$$$$$$$$$$$$'
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') 'Stratiform High'
!     write(UNIT=CLD_FID,FMT='(3F10.3)') C_S(:,1)
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') '$$$$$$$$$$$$$$$$$$$$$$$$'
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') 'Stratiform Low'
!     write(UNIT=CLD_FID,FMT='(3F10.3)') C_S(:,2)
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') '$$$$$$$$$$$$$$$$$$$$$$$$'
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') 'FRAC_H_CLD(:)'
!     write(UNIT=CLD_FID,FMT='(3F10.3)') FRAC_H_CLD(:)
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') '$$$$$$$$$$$$$$$$$$$$$$$$'
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') 'FRAC_CLD(:,1)'
!     write(UNIT=CLD_FID,FMT='(3F10.3)') FRAC_CLD(:,1)
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') '$$$$$$$$$$$$$$$$$$$$$$$$'
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') 'FRAC_CLD(:,2)'
!     write(UNIT=CLD_FID,FMT='(3F10.3)') FRAC_CLD(:,2)
!     write(UNIT=CLD_FID,FMT='(A)') ''
!     write(UNIT=CLD_FID,FMT='(A)') '$$$$$$$$$$$$$$$$$$$$$$$$'
!  endif

end subroutine update_clds

!******************************************************************************
! (70) function write_out
!      *this function is used to write the contents of arrays to a specified file
!
!      *inputs
!         FID      - FID of file you want to write to
!         SIZE     - length of array to be written
!         ARR      - a real array of size SIZE
!         AVG      - the area-weighted average of ARR
!                    only used when SIZE.eq.JMAX, a dummy argument otherwise
!         NAME_STR - Name of string, printed to label data
!
!      (dsa, 12/15/05)
!******************************************************************************
subroutine write_out(FID,SIZE,ARR,AVG,NAME_STR) 

!inputs
  integer, intent(in)                  :: FID
  integer, intent(in)                  :: SIZE
  real*8,  intent(in), dimension(SIZE) :: ARR
  real*8,  intent(in)                  :: AVG
  character(len=*), intent(in)         :: NAME_STR

!internal
  integer                              :: J

  if( SIZE .eq. JMAX ) then

     write(UNIT=FID,FMT='(A)')              '*********************'
     write(UNIT=FID,FMT='(A)')              NAME_STR
     write(UNIT=FID,FMT='(A)')              '*********************'
     write(UNIT=FID,FMT='(A17,I1,100I9,A3)') '      AVG      J=',(/ (J,J=1,SIZE) /)
     write(UNIT=FID,FMT='(A)')              '---------------------'
     write(UNIT=FID,FMT='(100F9.3)')        AVG,ARR
     write(UNIT=FID,FMT='(A)')              ''
     write(UNIT=FID,FMT='(A)')              ''

  else if( SIZE .eq. JMAX-1 ) then

     write(UNIT=FID,FMT='(A)')               '*********************'
     write(UNIT=FID,FMT='(A)')               NAME_STR
     write(UNIT=FID,FMT='(A)')               '*********************'
     write(UNIT=FID,FMT='(A6,F3.1,100F9.1)') '    J=',(/ (J+0.5,J=1,SIZE) /)
     write(UNIT=FID,FMT='(A)')               '---------------------'
     write(UNIT=FID,FMT='(100F9.3)')         ARR
     write(UNIT=FID,FMT='(A)')               ''
     write(UNIT=FID,FMT='(A)')               ''

  else if( SIZE .eq. JMAX+1 ) then

     write(UNIT=FID,FMT='(A)')               '********************'
     write(UNIT=FID,FMT='(A)')               NAME_STR
     write(UNIT=FID,FMT='(A)')               '*********************'
     write(UNIT=FID,FMT='(A6,F3.1,100F9.1)') '    J=',(/ (J-0.5,J=1,SIZE) /)
     write(UNIT=FID,FMT='(A)')               '---------------------'
     write(UNIT=FID,FMT='(100F9.3)')         ARR
     write(UNIT=FID,FMT='(A)')               ''
     write(UNIT=FID,FMT='(A)')               ''

  else

     write(UNIT=6,FMT='(A,A,A)') 'Invalid array named ', NAME_STR, ' passed to subroutine write_out'
 
  end if

end subroutine write_out



!******************************************************************************
! (71) function write_for_plotting
!      *this function is used to write the contents of arrays to a file and 
!       the names of these arrays to another file.  Only arrays of the same 
!       size should be written to the same file, or else MATLAB will get 
!       confused reading in. The label file is used by a MATLAB function for 
!       the ylabels on the plots.
!
!      *inputs
!         SIZE     - length of array to be written
!         ARR      - a real array of size SIZE!         
!         AVG      - the area-weighted average of ARR
!                    only used when SIZE.eq.JMAX, a dummy argument otherwise
!         NAME_STR - Name of string, printed to label data
!
!      (dsa, 12/15/05)
!******************************************************************************
subroutine write_for_plotting(SIZE,ARR,AVG,NAME_STR)

!inputs
  integer, intent(in)                  :: SIZE
  real*8,  intent(in), dimension(SIZE) :: ARR
  real*8,  intent(in)                  :: AVG
  character(len=*), intent(in)         :: NAME_STR

  if ( SIZE .eq. JMAX ) then
     write(UNIT=JMAX_FID,FMT='(100g20.12)')    ARR,AVG
     write(UNIT=JMAX_LBL_FID,FMT='(A30,A1)')   NAME_STR,'-'
  else if( SIZE .eq. JMAX-1 ) then
     write(UNIT=JMAXm1_FID,FMT='(100g20.12)')  ARR,AVG
     write(UNIT=JMAXm1_LBL_FID,FMT='(A30,A1)') NAME_STR,'-'
  else if( SIZE .eq. JMAX+1 ) then
     write(UNIT=JMAXp1_FID,FMT='(100g20.12)')  ARR,AVG
     write(UNIT=JMAXp1_LBL_FID,FMT='(A30,A1)') NAME_STR,'-'
  else
     write(UNIT=6,FMT='(A,A,A)') 'Invalid array named: ', NAME_STR, &
                                 ' in subroutine write_for_plotting'
  end if


end subroutine write_for_plotting


!******************************************************************************
! (66) function interpolate_frac
!      -interpolate a fraction (should work for other things) from one grid 
!       onto another
!      -used for mapping land fraction from 5 degree latitude bins onto the 
!       grid used in a run.
!      -inputs:
!        LAT_EDGE_1 - initial grid (edges of boxes)
!        LAT_EDGE_2 - final grid (edges of boxes)
!        FRAC1      - Fractions on initial grid (box centers)
!      -output:
!        FRAC2      - Fractions on final grid (box centers)
!   
!   (dsa, 12/30/05)
!******************************************************************************
function interpolate_frac( LAT_EDGE_1, LAT_EDGE_2, FRAC1 ) result ( FRAC2 )
!inputs
  real*8, dimension(:), intent(in)                    :: LAT_EDGE_1
  real*8, dimension(:), intent(in)                    :: LAT_EDGE_2
  real*8, dimension(size(LAT_EDGE_1,1)-1), intent(in) :: FRAC1
!outputs
  real*8, dimension(size(LAT_EDGE_2,1)-1)             :: FRAC2
!internal
  real*8  :: TMP,L1,L2
  integer :: N1,N2,J,M

  N1 = size(LAT_EDGE_1,1)
  N2 = size(LAT_EDGE_2,1)


  !requires latitudes have same range
  if( ( lat_edge_1(1) .ne. lat_edge_1(1) ) .or. ( lat_edge_1(1) .ne. lat_edge_1(1) ) ) then
     write(6,'(A)') 'Latitude range mismatch in function interpolate_frac!'
  endif

  M = 2

  do J = 1, N2-1
     TMP = 0.0d0
     L1  = LAT_EDGE_2(J)
     L2  = LAT_EDGE_2(J+1)

     if( L2 .lt. LAT_EDGE_1(M) ) then
        FRAC2(J) = FRAC1(M-1)
     elseif( L2 .eq. LAT_EDGE_1(M) ) then
        FRAC2(J) = FRAC1(M-1)
        M        = M + 1
     else
        
        TMP = FRAC1(M-1) * ( sin( LAT_EDGE_1(M) ) - sin( L1 ) )
        M   = M + 1

        do while( l2 .gt. lat_edge_1(M) )
           TMP = TMP + FRAC1(M-1) * ( sin( LAT_EDGE_1(M) ) - sin( LAT_EDGE_1(M-1) ) )
           M   = M + 1
        enddo

        TMP = TMP + FRAC1(M-1) * ( sin( L2 ) - sin( LAT_EDGE_1(M-1) ) )
        FRAC2(J) = TMP / ( sin(L2) - sin(L1) )

     endif
  enddo
  

end function interpolate_frac


!******************************************************************************
! (66) function vel_2der_zonal
!      - should be used on zonal velocities, on center grid
!      - not using fancy f90 built-in shifting functions to make the code 
!        easier to read (can change if it really slows runs down)
!      - assumes d(vel)/dy=0 at equator and pole 
!      - may want to go back and put more effort into the exact specification of 
!        this function
!   (dsa, 01/01/06)
!******************************************************************************
function vel_2der_zonal( VEL ) result ( DDVEL )
!inputs
  real*8, dimension(JMAX), intent(in)  :: VEL
!outputs
  real*8, dimension(JMAX)              :: DDVEL
!internal
  integer                              :: J

  DDVEL(1)    = ( VEL(2) - VEL(1) ) / &
                ( ( Y(2) - Y(1) ) * ( Y_EDGE(2) - Y_EDGE(1) ) )
  DDVEL(JMAX) = - ( VEL(JMAX) - VEL(JMAX-1) ) / &
                ( ( Y(JMAX) - Y(JMAX-1) ) * ( Y_EDGE(JMAX+1) - Y_EDGE(JMAX) ) )

  DDVEL(2:JMAX-1) = ( ( VEL(3:JMAX)   - VEL(2:JMAX-1) ) / ( Y(3:JMAX)   - Y(2:JMAX-1)   ) - &
                      ( VEL(2:JMAX-1) - VEL(1:JMAX-2) ) / ( Y(2:JMAX-1) - Y(1:JMAX-2) ) ) / &
                      ( Y_EDGE(3:JMAX) - Y_EDGE(2:JMAX-1) )

end function vel_2der_zonal

!******************************************************************************
! (66) function vel_2der_merid
!      - should be used on meridional velocities, on edge grid
!      - not using fancy f90 built-in shifting functions to make the code 
!        easier to read (can change if it really slows runs down)
!      - assumes d2(vel)/dy2=0 at equator and pole 
!        (d(vel)/dy can't be evaluated at J-1 here, so this seemed like the 
!         best option) 
!      - may want to go back and put more effort into the exact specification of 
!        this function
!   (dsa, 01/01/06)
!******************************************************************************
function vel_2der_merid( VEL ) result ( DDVEL )
!inputs
  real*8, dimension(JMAX+1), intent(in)  :: VEL
!outputs
  real*8, dimension(JMAX+1)              :: DDVEL
!internal
  integer                                :: J

  DDVEL(1)      = 0
  DDVEL(JMAX+1) = 0

  DDVEL(2:JMAX) = ( ( VEL(3:JMAX+1) - VEL(2:JMAX)   ) / ( Y_EDGE(3:JMAX+1) - Y_EDGE(2:JMAX)   )   - &
                    ( VEL(2:JMAX)   - VEL(1:JMAX-1) ) / ( Y_EDGE(2:JMAX)   - Y_EDGE(1:JMAX-1) ) ) / &
                  ( Y_EDGE(2:JMAX) - Y_EDGE(1:JMAX-1) ) 

end function vel_2der_merid

!******************************************************************************
! (XX) function get_eddy_tend
!  inputs:
!     ARR   - the array to be diffused 
!     IMODE - what you are diffusing (for constant and diagnostics)
!              + 1 - DSE
!              + 2 - Q
!              + 3 - u
!  outputs:
!     EDDY_TEND - tendency due to eddy diffusion
!  notes:
!        -Can use Y instead of LAT b/c they're proportional to eachother
!   (dsa, 10/31/06)
!******************************************************************************

function get_eddy_tend( ARR, IMODE ) result ( EDDY_TEND )
!inputs
  real*8, dimension(JMAX,KMAX), intent(in)  :: ARR
  integer,                      intent(in)  :: IMODE
!outputs
  real*8, dimension(JMAX,KMAX)              :: EDDY_TEND
!internal
  real*8                                    :: K
  real*8, dimension(JMAX-1)                 :: D_DSE_DY0,D_ARR_DY
  real*8, dimension(JMAX+1)                 :: DIFF,FLUX

  !initialize variables
  DIFF(:)      = 0.0d0
  D_DSE_DY0(:) = 0.0d0
  FLUX(:)      = 0.0d0

  if ( IMODE .eq. 1 ) then
     K = KY_BCLIN_TH
  elseif ( IMODE .eq. 2 ) then
     K = KY_BCLIN_Q
  elseif ( IMODE .eq. 3 ) then
     K = KY_BCLIN_MOM
  endif

  !meridional derivative of lower level DSE
  D_DSE_DY0(:) = derivative( DSE(:,2,2), Y )

  !diffusion constant
  DIFF(2:JMAX) = - K * ( abs( D_DSE_DY0(:) ) ** N_BCLIN )

  do K = 1, KMAX
     
     D_ARR_DY(:) = derivative( ARR(:,K), Y )

     FLUX(2:JMAX) = BCLIN_FRIC(K) * DIFF(2:JMAX) * D_ARR_DY(:)

     EDDY_TEND(:,K) = ( FLUX(1:JMAX  ) * cos( LAT_EDGE_RADS(1:JMAX  ) )   - &
                        FLUX(2:JMAX+1) * cos( LAT_EDGE_RADS(2:JMAX+1) ) ) / &
                       R / ( sin( LAT_EDGE_RADS(2:JMAX+1) )               - &
                             sin( LAT_EDGE_RADS(1:JMAX  ) ) )
     
     if ( IMODE .eq. 1 ) then
        EDDY_HEAT(:,K)   = FLUX(:) * DX(:) * ( P_EDGE(K+1) - P_EDGE(K) ) / G
     elseif ( IMODE .eq. 2 ) then
        EDDY_MOIST(:,K)  = FLUX(:) * DX(:) * ( P_EDGE(K+1) - P_EDGE(K) ) / G
        EDDY_LATENT(:,K) = LV0 * EDDY_MOIST(:,K)
     endif
  enddo


end function get_eddy_tend


function get_eddy_tend_v result ( EDDY_TEND )
!outputs
  real*8, dimension(JMAX+1,KMAX)              :: EDDY_TEND
!internal
  real*8                                    :: K
  real*8, dimension(JMAX+1)                 :: D_DSE_DY0
  real*8, dimension(JMAX)                   :: DIFF,FLUX,D_DSE_DY_CENT,D_ARR_DY

  !initialize variables
  DIFF(:)      = 0.0d0
  D_DSE_DY0(:) = 0.0d0
  FLUX(:)      = 0.0d0

  K = KY_BCLIN_MOM

  !meridional derivative of lower level DSE
  D_DSE_DY0(2:JMAX) = derivative( DSE(:,2,2), Y )

  D_DSE_DY_CENT(:) = center_interp1( D_DSE_DY0(:) )

  !diffusion constant
  DIFF(:) = - K * ( abs( D_DSE_DY_CENT(:) ) ** N_BCLIN )
!  DIFF(:) = - K 

  do K = 1, KMAX
     
     D_ARR_DY(:) = derivative( BCLIN_V(:,K,1), Y_EDGE )

     FLUX(:) = BCLIN_FRIC(K) * DIFF(:) * D_ARR_DY(:)


     EDDY_TEND(2:JMAX,K) = ( FLUX(1:JMAX-1) * cos( LAT_RADS(1:JMAX-1) )   - &
                             FLUX(2:JMAX  ) * cos( LAT_RADS(2:JMAX  ) ) ) / &
                            R / ( sin( LAT_RADS(2:JMAX) )                 - &
                                  sin( LAT_RADS(1:JMAX-1) ) )

  enddo


  !since v is zero on these edges anyway, it doesn't matter
  EDDY_TEND(1,:)      = 0.0d0
  EDDY_TEND(JMAX+1,:) = 0.0d0

end function get_eddy_tend_v

end module atm_mod
